@@ -1,4 +1,4 @@
-# Copyright 2011-2016, Damian Johnson and The Tor Project
+# Copyright 2011-2017, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -86,15 +86,19 @@ Library for working with the tor process.
 
   Statuses that a circuit can be in. Tor may provide statuses not in this enum.
 
-  ============ ===========
-  CircStatus   Description
-  ============ ===========
-  **LAUNCHED** new circuit was created
-  **BUILT**    circuit finished being created and can accept traffic
-  **EXTENDED** circuit has been extended by a hop
-  **FAILED**   circuit construction failed
-  **CLOSED**   circuit has been closed
-  ============ ===========
+  .. versionchanged:: 1.6.0
+     Added the GUARD_WAIT signal.
+
+  ============== ===========
+  CircStatus     Description
+  ============== ===========
+  **LAUNCHED**   new circuit was created
+  **BUILT**      circuit finished being created and can accept traffic
+  **GUARD_WAIT** waiting to see if there's a circuit with a better guard before using
+  **EXTENDED**   circuit has been extended by a hop
+  **FAILED**     circuit construction failed
+  **CLOSED**     circuit has been closed
+  ============== ===========
 
 .. data:: CircBuildFlag (enum)
 
@@ -442,6 +446,9 @@ Library for working with the tor process.
   .. versionchanged:: 1.4.0
      Added the UPLOAD_REJECTED reason.
 
+  .. versionchanged:: 1.6.0
+     Added the QUERY_NO_HSDIR reason.
+
   =================== ===========
   HSDescReason        Description
   =================== ===========
@@ -449,6 +456,7 @@ Library for working with the tor process.
   **QUERY_REJECTED**  hidden service directory refused to provide the descriptor
   **UPLOAD_REJECTED** descriptor was rejected by the hidden service directory
   **NOT_FOUND**       descriptor with the given identifier wasn't found
+  **QUERY_NO_HSDIR**  no hidden service directory was found
   **UNEXPECTED**      failure type is unknown
   =================== ===========
 
@@ -470,7 +478,7 @@ Library for working with the tor process.
 
 import stem.util.enum
 
-__version__ = '1.5.3'
+__version__ = '1.6.0'
 __author__ = 'Damian Johnson'
 __contact__ = 'atagar@torproject.org'
 __url__ = 'https://stem.torproject.org/'
@@ -608,6 +616,7 @@ class SocketError(ControllerError):
 class SocketClosed(SocketError):
   'Control socket was closed before completing the message.'
 
+
 Runlevel = stem.util.enum.UppercaseEnum(
   'DEBUG',
   'INFO',
@@ -653,6 +662,7 @@ Signal = stem.util.enum.UppercaseEnum(
 CircStatus = stem.util.enum.UppercaseEnum(
   'LAUNCHED',
   'BUILT',
+  'GUARD_WAIT',
   'EXTENDED',
   'FAILED',
   'CLOSED',
@@ -842,6 +852,7 @@ HSDescReason = stem.util.enum.UppercaseEnum(
   'QUERY_REJECTED',
   'UPLOAD_REJECTED',
   'NOT_FOUND',
+  'QUERY_NO_HSDIR',
   'UNEXPECTED',
 )
 
