@@ -1,4 +1,4 @@
-# Copyright 2011-2016, Damian Johnson and The Tor Project
+# Copyright 2011-2017, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -198,6 +198,12 @@ CONNECT_MESSAGES = {
   'wrong_socket_type': WRONG_SOCKET_TYPE_MSG.strip(),
 }
 
+COMMON_TOR_COMMANDS = (
+  'tor',
+  'tor.real',  # TBB command ran
+  '/usr/local/bin/tor',  # FreeBSD expands the whole path, this is the default location
+)
+
 
 def connect(control_port = ('127.0.0.1', 'default'), control_socket = '/var/run/tor/control', password = None, password_prompt = False, chroot_path = None, controller = stem.control.Controller):
   """
@@ -273,12 +279,10 @@ def connect(control_port = ('127.0.0.1', 'default'), control_socket = '/var/run/
   # If unable to connect to either a control socket or port then finally fail
   # out. If we only attempted to connect to one of them then provide the error
   # output from that. Otherwise we provide a more generic error message.
-  #
-  # We check for a 'tor.real' process name because that's what TBB uses.
 
   if not control_connection:
     if control_socket and control_port:
-      is_tor_running = stem.util.system.is_running('tor') or stem.util.system.is_running('tor.real')
+      is_tor_running = stem.util.system.is_running(COMMON_TOR_COMMANDS)
       error_msg = CONNECT_MESSAGES['no_control_port'] if is_tor_running else CONNECT_MESSAGES['tor_isnt_running']
 
     print(error_msg)
@@ -584,6 +588,9 @@ def authenticate(controller, password = None, chroot_path = None, protocolinfo_r
         else:
           authenticate_cookie(controller, cookie_path, False)
 
+      if isinstance(controller, stem.control.BaseController):
+        controller._post_authentication()
+
       return  # success!
     except OpenAuthRejected as exc:
       auth_exceptions.append(exc)
@@ -982,11 +989,6 @@ def get_protocolinfo(controller):
   the tor process running on it. If the socket is already closed then it is
   first reconnected.
 
-  According to the control spec the cookie_file is an absolute path. However,
-  this often is not the case (especially for the Tor Browser Bundle). If the
-  path is relative then we'll make an attempt (which may not work) to correct
-  this (:trac:`1101`).
-
   This can authenticate to either a :class:`~stem.control.BaseController` or
   :class:`~stem.socket.ControlSocket`.
 
@@ -1018,27 +1020,6 @@ def get_protocolinfo(controller):
       raise stem.SocketError(exc)
 
   stem.response.convert('PROTOCOLINFO', protocolinfo_response)
-
-  # attempt to expand relative cookie paths
-
-  if protocolinfo_response.cookie_path:
-    _expand_cookie_path(protocolinfo_response, stem.util.system.pid_by_name, 'tor')
-
-  # attempt to expand relative cookie paths via the control port or socket file
-
-  if isinstance(controller, stem.socket.ControlSocket):
-    control_socket = controller
-  else:
-    control_socket = controller.get_socket()
-
-  if isinstance(control_socket, stem.socket.ControlPort):
-    if control_socket.get_address() == '127.0.0.1':
-      pid_method = stem.util.system.pid_by_port
-      _expand_cookie_path(protocolinfo_response, pid_method, control_socket.get_port())
-  elif isinstance(control_socket, stem.socket.ControlSocketFile):
-    pid_method = stem.util.system.pid_by_open_file
-    _expand_cookie_path(protocolinfo_response, pid_method, control_socket.get_socket_path())
-
   return protocolinfo_response
 
 
@@ -1119,40 +1100,6 @@ def _read_cookie(cookie_path, is_safecookie):
     raise UnreadableCookieFile(exc_msg, cookie_path, is_safecookie)
 
 
-def _expand_cookie_path(protocolinfo_response, pid_resolver, pid_resolution_arg):
-  """
-  Attempts to expand a relative cookie path with the given pid resolver. This
-  leaves the cookie_path alone if it's already absolute, **None**, or the
-  system calls fail.
-  """
-
-  cookie_path = protocolinfo_response.cookie_path
-  if cookie_path and not os.path.isabs(cookie_path):
-    try:
-      tor_pid = pid_resolver(pid_resolution_arg)
-
-      if not tor_pid:
-        raise IOError('pid lookup failed')
-
-      tor_cwd = stem.util.system.cwd(tor_pid)
-
-      if not tor_cwd:
-        raise IOError('cwd lookup failed')
-
-      cookie_path = stem.util.system.expand_path(cookie_path, tor_cwd)
-    except IOError as exc:
-      resolver_labels = {
-        stem.util.system.pid_by_name: ' by name',
-        stem.util.system.pid_by_port: ' by port',
-        stem.util.system.pid_by_open_file: ' by socket file',
-      }
-
-      pid_resolver_label = resolver_labels.get(pid_resolver, '')
-      log.debug('unable to expand relative tor cookie path%s: %s' % (pid_resolver_label, exc))
-
-  protocolinfo_response.cookie_path = cookie_path
-
-
 class AuthenticationFailure(Exception):
   """
   Base error for authentication failures.
@@ -1297,7 +1244,9 @@ class NoAuthCookie(MissingAuthInfo):
     super(NoAuthCookie, self).__init__(message)
     self.is_safecookie = is_safecookie
 
+
 # authentication exceptions ordered as per the authenticate function's pydocs
+
 AUTHENTICATE_EXCEPTIONS = (
   IncorrectSocketType,
   UnrecognizedAuthMethods,
