@@ -1,4 +1,4 @@
-# Copyright 2012-2016, Damian Johnson and The Tor Project
+# Copyright 2012-2017, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -9,7 +9,7 @@ Documents can be obtained from a few sources...
 * The 'cached-consensus' file in Tor's data directory.
 
 * Archived descriptors provided by `CollecTor
-  <https://collector.torproject.org/>`_.
+  <https://metrics.torproject.org/collector.html>`_.
 
 * Directory authorities and mirrors via their DirPort.
 
@@ -66,14 +66,21 @@ from stem.descriptor import (
   PGP_BLOCK_END,
   Descriptor,
   DocumentHandler,
-  _get_descriptor_components,
+  _descriptor_content,
+  _descriptor_components,
   _read_until_keywords,
   _value,
   _parse_simple_line,
   _parse_if_present,
   _parse_timestamp_line,
   _parse_forty_character_hex,
+  _parse_protocol_line,
   _parse_key_block,
+  _random_nickname,
+  _random_fingerprint,
+  _random_ipv4_address,
+  _random_date,
+  _random_crypto_blob,
 )
 
 from stem.descriptor.router_status_entry import (
@@ -123,6 +130,10 @@ HEADER_STATUS_DOCUMENT_FIELDS = (
   ('shared-rand-commit', True, False, False),
   ('shared-rand-previous-value', True, True, False),
   ('shared-rand-current-value', True, True, False),
+  ('recommended-client-protocols', True, True, False),
+  ('recommended-relay-protocols', True, True, False),
+  ('required-client-protocols', True, True, False),
+  ('required-relay-protocols', True, True, False),
   ('params', True, True, False),
 )
 
@@ -132,9 +143,6 @@ FOOTER_STATUS_DOCUMENT_FIELDS = (
   ('directory-signature', True, True, True),
 )
 
-HEADER_FIELDS = [attr[0] for attr in HEADER_STATUS_DOCUMENT_FIELDS]
-FOOTER_FIELDS = [attr[0] for attr in FOOTER_STATUS_DOCUMENT_FIELDS]
-
 AUTH_START = 'dir-source'
 ROUTERS_START = 'r'
 FOOTER_START = 'directory-footer'
@@ -155,6 +163,10 @@ DEFAULT_PARAMS = {
   'UseOptimisticData': 1,
   'Support022HiddenServices': 1,
   'usecreatefast': 1,
+  'max-consensuses-age-to-cache-for-diff': 72,
+  'try-diff-for-consensus-newer-than': 72,
+  'onion-key-rotation-days': 28,
+  'onion-key-grace-period-days': 7,
 }
 
 # KeyCertificate fields, tuple is of the form...
@@ -202,6 +214,10 @@ PARAM_RANGE = {
   'NumNTorsPerTAP': (1, 100000),
   'AllowNonearlyExtend': (0, 1),
   'AuthDirNumSRVAgreements': (1, MAX_PARAM),
+  'max-consensuses-age-to-cache-for-diff': (0, 8192),
+  'try-diff-for-consensus-newer-than': (0, 8192),
+  'onion-key-rotation-days': (1, 90),
+  'onion-key-grace-period-days': (1, 90),  # max is the highest onion-key-rotation-days
 }
 
 
@@ -447,6 +463,22 @@ class NetworkStatusDocumentV2(NetworkStatusDocument):
     'directory-signature': _parse_directory_signature_line,
   }
 
+  @classmethod
+  def content(cls, attr = None, exclude = (), sign = False):
+    if sign:
+      raise NotImplementedError('Signing of %s not implemented' % cls.__name__)
+
+    return _descriptor_content(attr, exclude, (
+      ('network-status-version', '2'),
+      ('dir-source', '%s %s 80' % (_random_ipv4_address(), _random_ipv4_address())),
+      ('fingerprint', _random_fingerprint()),
+      ('contact', 'arma at mit dot edu'),
+      ('published', _random_date()),
+      ('dir-signing-key', _random_crypto_blob('RSA PUBLIC KEY')),
+    ), (
+      ('directory-signature', 'moria2' + _random_crypto_blob('SIGNATURE')),
+    ))
+
   def __init__(self, raw_content, validate = False):
     super(NetworkStatusDocumentV2, self).__init__(raw_content, lazy_load = not validate)
 
@@ -469,7 +501,7 @@ class NetworkStatusDocumentV2(NetworkStatusDocument):
 
     self.routers = dict((desc.fingerprint, desc) for desc in router_iter)
 
-    entries = _get_descriptor_components(document_content + b'\n' + document_file.read(), validate)
+    entries = _descriptor_components(document_content + b'\n' + document_file.read(), validate)
 
     if validate:
       self._check_constraints(entries)
@@ -636,11 +668,6 @@ def _parse_header_parameters_line(descriptor, entries):
 
   value = _value('params', entries)
 
-  # should only appear in consensus-method 7 or later
-
-  if not descriptor.meets_consensus_method(7):
-    raise ValueError("A network status document's 'params' line should only appear in consensus-method 7 or later")
-
   if value != '':
     descriptor.params = _parse_int_mappings('params', value, True)
     descriptor._check_params_constraints()
@@ -756,6 +783,10 @@ _parse_header_server_versions_line = _parse_versions_line('server-versions', 'se
 _parse_header_known_flags_line = _parse_simple_line('known-flags', 'known_flags', func = lambda v: [entry for entry in v.split(' ') if entry])
 _parse_footer_bandwidth_weights_line = _parse_simple_line('bandwidth-weights', 'bandwidth_weights', func = lambda v: _parse_int_mappings('bandwidth-weights', v, True))
 _parse_shared_rand_participate_line = _parse_if_present('shared-rand-participate', 'is_shared_randomness_participate')
+_parse_recommended_client_protocols_line = _parse_protocol_line('recommended-client-protocols', 'recommended_client_protocols')
+_parse_recommended_relay_protocols_line = _parse_protocol_line('recommended-relay-protocols', 'recommended_relay_protocols')
+_parse_required_client_protocols_line = _parse_protocol_line('required-client-protocols', 'required_client_protocols')
+_parse_required_relay_protocols_line = _parse_protocol_line('required-relay-protocols', 'required_relay_protocols')
 
 
 class NetworkStatusDocumentV3(NetworkStatusDocument):
@@ -793,24 +824,21 @@ class NetworkStatusDocumentV3(NetworkStatusDocument):
   :var int consensus_method: method version used to generate this consensus
   :var dict bandwidth_weights: dict of weight(str) => value(int) mappings
 
+  :var str shared_randomness_current_value: base64 encoded current shared
+    random value
+  :var str shared_randomness_previous_value: base64 encoded last shared random
+    value
+
   **Vote Attributes:**
 
   :var list consensus_methods: list of ints for the supported method versions
   :var datetime published: time when the document was published
   :var dict flag_thresholds: **\*** mapping of internal performance thresholds used while making the vote, values are **ints** or **floats**
 
-  :var bool is_shared_randomness_participate: **\*** **True** if this authority
-    participates in establishing a shared random value, **False** otherwise
-  :var list shared_randomness_commitments: **\*** list of
-    :data:`~stem.descriptor.networkstatus.SharedRandomnessCommitment` entries
-  :var int shared_randomness_previous_reveal_count: number of commitments
-    used to generate the last shared random value
-  :var str shared_randomness_previous_value: base64 encoded last shared random
-    value
-  :var int shared_randomness_current_reveal_count: number of commitments
-    used to generate the current shared random value
-  :var str shared_randomness_current_value: base64 encoded current shared
-    random value
+  :var dict recommended_client_protocols: recommended protocols for clients
+  :var dict recommended_relay_protocols: recommended protocols for relays
+  :var dict required_client_protocols: required protocols for clients
+  :var dict required_relay_protocols: required protocols for relays
 
   **\*** attribute is either required when we're parsed with validation or has
   a default value, others are left as None if undefined
@@ -824,6 +852,14 @@ class NetworkStatusDocumentV3(NetworkStatusDocument):
      shared_randomness_previous_value,
      shared_randomness_current_reveal_count, and
      shared_randomness_current_value attributes.
+
+  .. versionchanged:: 1.6.0
+     Added the recommended_client_protocols, recommended_relay_protocols,
+     required_client_protocols, and required_relay_protocols.
+
+  .. versionchanged:: 1.6.0
+     The shared randomness attributes were misdocumented in the tor spec and as
+     such never set. They're now an attribute of **directory_authorities**.
   """
 
   ATTRIBUTES = {
@@ -845,13 +881,13 @@ class NetworkStatusDocumentV3(NetworkStatusDocument):
     'packages': ([], _parse_package_line),
     'known_flags': ([], _parse_header_known_flags_line),
     'flag_thresholds': ({}, _parse_header_flag_thresholds_line),
-    'is_shared_randomness_participate': (False, _parse_shared_rand_participate_line),
-    'shared_randomness_commitments': ([], _parsed_shared_rand_commit),
-    'shared_randomness_previous_reveal_count': (None, _parse_shared_rand_previous_value),
-    'shared_randomness_previous_value': (None, _parse_shared_rand_previous_value),
-    'shared_randomness_current_reveal_count': (None, _parse_shared_rand_current_value),
-    'shared_randomness_current_value': (None, _parse_shared_rand_current_value),
+    'recommended_client_protocols': ({}, _parse_recommended_client_protocols_line),
+    'recommended_relay_protocols': ({}, _parse_recommended_relay_protocols_line),
+    'required_client_protocols': ({}, _parse_required_client_protocols_line),
+    'required_relay_protocols': ({}, _parse_required_relay_protocols_line),
     'params': ({}, _parse_header_parameters_line),
+    'shared_randomness_previous_value': (None, _parse_shared_rand_previous_value),
+    'shared_randomness_current_value': (None, _parse_shared_rand_current_value),
 
     'signatures': ([], _parse_footer_directory_signature_line),
     'bandwidth_weights': ({}, _parse_footer_bandwidth_weights_line),
@@ -872,11 +908,13 @@ class NetworkStatusDocumentV3(NetworkStatusDocument):
     'package': _parse_package_line,
     'known-flags': _parse_header_known_flags_line,
     'flag-thresholds': _parse_header_flag_thresholds_line,
-    'shared-rand-participate': _parse_shared_rand_participate_line,
-    'shared-rand-commit': _parsed_shared_rand_commit,
+    'recommended-client-protocols': _parse_recommended_client_protocols_line,
+    'recommended-relay-protocols': _parse_recommended_relay_protocols_line,
+    'required-client-protocols': _parse_required_client_protocols_line,
+    'required-relay-protocols': _parse_required_relay_protocols_line,
+    'params': _parse_header_parameters_line,
     'shared-rand-previous-value': _parse_shared_rand_previous_value,
     'shared-rand-current-value': _parse_shared_rand_current_value,
-    'params': _parse_header_parameters_line,
   }
 
   FOOTER_PARSER_FOR_LINE = {
@@ -885,6 +923,85 @@ class NetworkStatusDocumentV3(NetworkStatusDocument):
     'directory-signature': _parse_footer_directory_signature_line,
   }
 
+  @classmethod
+  def content(cls, attr = None, exclude = (), sign = False, authorities = None, routers = None):
+    if sign:
+      raise NotImplementedError('Signing of %s not implemented' % cls.__name__)
+
+    attr = {} if attr is None else dict(attr)
+    is_vote = attr.get('vote-status') == 'vote'
+
+    if is_vote:
+      extra_defaults = {'consensus-methods': '1 9', 'published': _random_date()}
+    else:
+      extra_defaults = {'consensus-method': '9'}
+
+    if is_vote and authorities is None:
+      authorities = [DirectoryAuthority.create(is_vote = is_vote)]
+
+    for k, v in extra_defaults.items():
+      if exclude and k in exclude:
+        continue  # explicitly excluding this field
+      elif k not in attr:
+        attr[k] = v
+
+    desc_content = _descriptor_content(attr, exclude, (
+      ('network-status-version', '3'),
+      ('vote-status', 'consensus'),
+      ('consensus-methods', None),
+      ('consensus-method', None),
+      ('published', None),
+      ('valid-after', _random_date()),
+      ('fresh-until', _random_date()),
+      ('valid-until', _random_date()),
+      ('voting-delay', '300 300'),
+      ('client-versions', None),
+      ('server-versions', None),
+      ('package', None),
+      ('known-flags', 'Authority BadExit Exit Fast Guard HSDir Named Running Stable Unnamed V2Dir Valid'),
+      ('params', None),
+    ), (
+      ('directory-footer', ''),
+      ('bandwidth-weights', None),
+      ('directory-signature', '%s %s%s' % (_random_fingerprint(), _random_fingerprint(), _random_crypto_blob('SIGNATURE'))),
+    ))
+
+    # inject the authorities and/or routers between the header and footer
+
+    if authorities:
+      if b'directory-footer' in desc_content:
+        footer_div = desc_content.find(b'\ndirectory-footer') + 1
+      elif b'directory-signature' in desc_content:
+        footer_div = desc_content.find(b'\ndirectory-signature') + 1
+      else:
+        if routers:
+          desc_content += b'\n'
+
+        footer_div = len(desc_content) + 1
+
+      authority_content = stem.util.str_tools._to_bytes('\n'.join([str(a) for a in authorities]) + '\n')
+      desc_content = desc_content[:footer_div] + authority_content + desc_content[footer_div:]
+
+    if routers:
+      if b'directory-footer' in desc_content:
+        footer_div = desc_content.find(b'\ndirectory-footer') + 1
+      elif b'directory-signature' in desc_content:
+        footer_div = desc_content.find(b'\ndirectory-signature') + 1
+      else:
+        if routers:
+          desc_content += b'\n'
+
+        footer_div = len(desc_content) + 1
+
+      router_content = stem.util.str_tools._to_bytes('\n'.join([str(r) for r in routers]) + '\n')
+      desc_content = desc_content[:footer_div] + router_content + desc_content[footer_div:]
+
+    return desc_content
+
+  @classmethod
+  def create(cls, attr = None, exclude = (), validate = True, sign = False, authorities = None, routers = None):
+    return cls(cls.content(attr, exclude, sign, authorities, routers), validate = validate)
+
   def __init__(self, raw_content, validate = False, default_params = True):
     """
     Parse a v3 network status document.
@@ -900,6 +1017,15 @@ class NetworkStatusDocumentV3(NetworkStatusDocument):
     super(NetworkStatusDocumentV3, self).__init__(raw_content, lazy_load = not validate)
     document_file = io.BytesIO(raw_content)
 
+    # TODO: Tor misdocumented these as being in the header rather than the
+    # authority section. As such these have never been set but we need the
+    # attributes for stem 1.5 compatability. Drop these in 2.0.
+
+    self.is_shared_randomness_participate = False
+    self.shared_randomness_commitments = []
+    self.shared_randomness_previous_reveal_count = None
+    self.shared_randomness_current_reveal_count = None
+
     self._default_params = default_params
     self._header(document_file, validate)
 
@@ -927,6 +1053,39 @@ class NetworkStatusDocumentV3(NetworkStatusDocument):
     self.routers = dict((desc.fingerprint, desc) for desc in router_iter)
     self._footer(document_file, validate)
 
+  def validate_signatures(self, key_certs):
+    """
+    Validates we're properly signed by the signing certificates.
+
+    .. versionadded:: 1.6.0
+
+    :param list key_certs: :class:`~stem.descriptor.networkstatus.KeyCertificates`
+      to validate the consensus against
+
+    :raises: **ValueError** if an insufficient number of valid signatures are present.
+    """
+
+    # sha1 hash of the body and header
+
+    local_digest = self._digest_for_content(b'network-status-version', b'directory-signature ')
+
+    valid_digests, total_digests = 0, 0
+    required_digests = len(self.signatures) / 2.0
+    signing_keys = dict([(cert.fingerprint, cert.signing_key) for cert in key_certs])
+
+    for sig in self.signatures:
+      if sig.identity not in signing_keys:
+        continue
+
+      signed_digest = self._digest_for_signature(signing_keys[sig.identity], sig.signature)
+      total_digests += 1
+
+      if signed_digest == local_digest:
+        valid_digests += 1
+
+    if valid_digests < required_digests:
+      raise ValueError('Network Status Document has %i valid signatures out of %i total, needed %i' % (valid_digests, total_digests, required_digests))
+
   def get_unrecognized_lines(self):
     if self._lazy_loading:
       self._parse(self._header_entries, False, parser_for_line = self.HEADER_PARSER_FOR_LINE)
@@ -961,13 +1120,14 @@ class NetworkStatusDocumentV3(NetworkStatusDocument):
 
   def _header(self, document_file, validate):
     content = bytes.join(b'', _read_until_keywords((AUTH_START, ROUTERS_START, FOOTER_START), document_file))
-    entries = _get_descriptor_components(content, validate)
+    entries = _descriptor_components(content, validate)
+    header_fields = [attr[0] for attr in HEADER_STATUS_DOCUMENT_FIELDS]
 
     if validate:
       # all known header fields can only appear once except
 
       for keyword, values in list(entries.items()):
-        if len(values) > 1 and keyword in HEADER_FIELDS and keyword != 'package' and keyword != 'shared-rand-commit':
+        if len(values) > 1 and keyword in header_fields and keyword != 'package' and keyword != 'shared-rand-commit':
           raise ValueError("Network status documents can only have a single '%s' line, got %i" % (keyword, len(values)))
 
       if self._default_params:
@@ -975,8 +1135,12 @@ class NetworkStatusDocumentV3(NetworkStatusDocument):
 
       self._parse(entries, validate, parser_for_line = self.HEADER_PARSER_FOR_LINE)
 
+      # should only appear in consensus-method 7 or later
+
+      if not self.meets_consensus_method(7) and 'params' in list(entries.keys()):
+        raise ValueError("A network status document's 'params' line should only appear in consensus-method 7 or later")
+
       _check_for_missing_and_disallowed_fields(self, entries, HEADER_STATUS_DOCUMENT_FIELDS)
-      _check_for_misordered_fields(entries, HEADER_FIELDS)
 
       # default consensus_method and consensus_methods based on if we're a consensus or vote
 
@@ -989,14 +1153,15 @@ class NetworkStatusDocumentV3(NetworkStatusDocument):
       self._entries.update(entries)
 
   def _footer(self, document_file, validate):
-    entries = _get_descriptor_components(document_file.read(), validate)
+    entries = _descriptor_components(document_file.read(), validate)
+    footer_fields = [attr[0] for attr in FOOTER_STATUS_DOCUMENT_FIELDS]
 
     if validate:
       for keyword, values in list(entries.items()):
         # all known footer fields can only appear once except...
         # * 'directory-signature' in a consensus
 
-        if len(values) > 1 and keyword in FOOTER_FIELDS:
+        if len(values) > 1 and keyword in footer_fields:
           if not (keyword == 'directory-signature' and self.is_consensus):
             raise ValueError("Network status documents can only have a single '%s' line, got %i" % (keyword, len(values)))
 
@@ -1015,7 +1180,6 @@ class NetworkStatusDocumentV3(NetworkStatusDocument):
             raise ValueError("Network status document's footer should start with a 'directory-signature' line prior to consensus-method 9")
 
         _check_for_missing_and_disallowed_fields(self, entries, FOOTER_STATUS_DOCUMENT_FIELDS)
-        _check_for_misordered_fields(entries, FOOTER_FIELDS)
     else:
       self._footer_entries = entries
       self._entries.update(entries)
@@ -1087,36 +1251,6 @@ def _check_for_missing_and_disallowed_fields(document, entries, fields):
     raise ValueError("Network status document has fields that shouldn't appear in this document type or version: %s" % ', '.join(disallowed_fields))
 
 
-def _check_for_misordered_fields(entries, expected):
-  """
-  To be valid a network status document's fiends need to appear in a specific
-  order. Checks that known fields appear in that order (unrecognized fields
-  are ignored).
-
-  :param dict entries: ordered keyword/value mappings of the header or footer
-  :param list expected: ordered list of expected fields (either
-    **HEADER_FIELDS** or **FOOTER_FIELDS**)
-
-  :raises: **ValueError** if entries aren't properly ordered
-  """
-
-  # Earlier validation has ensured that our fields either belong to our
-  # document type or are unknown. Remove the unknown fields since they
-  # reflect a spec change and can appear anywhere in the document.
-
-  actual = [field for field in entries.keys() if field in expected]
-
-  # Narrow the expected to just what we have. If the lists then match then the
-  # order's valid.
-
-  expected = [field for field in expected if field in actual]
-
-  if actual != expected:
-    actual_label = ', '.join(actual)
-    expected_label = ', '.join(expected)
-    raise ValueError("The fields in a section of the document are misordered. It should be '%s' but was '%s'" % (expected_label, actual_label))
-
-
 def _parse_int_mappings(keyword, value, validate):
   # Parse a series of 'key=value' entries, checking the following:
   # - values are integers
@@ -1221,11 +1355,31 @@ class DirectoryAuthority(Descriptor):
   :var stem.descriptor.networkstatus.KeyCertificate key_certificate: **\***
     authority's key certificate
 
+  :var bool is_shared_randomness_participate: **\*** **True** if this authority
+    participates in establishing a shared random value, **False** otherwise
+  :var list shared_randomness_commitments: **\*** list of
+    :data:`~stem.descriptor.networkstatus.SharedRandomnessCommitment` entries
+  :var int shared_randomness_previous_reveal_count: number of commitments
+    used to generate the last shared random value
+  :var str shared_randomness_previous_value: base64 encoded last shared random
+    value
+  :var int shared_randomness_current_reveal_count: number of commitments
+    used to generate the current shared random value
+  :var str shared_randomness_current_value: base64 encoded current shared
+    random value
+
   **\*** mandatory attribute
 
   .. versionchanged:: 1.4.0
      Renamed our 'fingerprint' attribute to 'v3ident' (prior attribute exists
      for backward compatability, but is deprecated).
+
+  .. versionchanged:: 1.6.0
+     Added the is_shared_randomness_participate, shared_randomness_commitments,
+     shared_randomness_previous_reveal_count,
+     shared_randomness_previous_value,
+     shared_randomness_current_reveal_count, and
+     shared_randomness_current_value attributes.
   """
 
   ATTRIBUTES = {
@@ -1239,6 +1393,12 @@ class DirectoryAuthority(Descriptor):
     'contact': (None, _parse_contact_line),
     'vote_digest': (None, _parse_vote_digest_line),
     'legacy_dir_key': (None, _parse_legacy_dir_key_line),
+    'is_shared_randomness_participate': (False, _parse_shared_rand_participate_line),
+    'shared_randomness_commitments': ([], _parsed_shared_rand_commit),
+    'shared_randomness_previous_reveal_count': (None, _parse_shared_rand_previous_value),
+    'shared_randomness_previous_value': (None, _parse_shared_rand_previous_value),
+    'shared_randomness_current_reveal_count': (None, _parse_shared_rand_current_value),
+    'shared_randomness_current_value': (None, _parse_shared_rand_current_value),
   }
 
   PARSER_FOR_LINE = {
@@ -1246,8 +1406,38 @@ class DirectoryAuthority(Descriptor):
     'contact': _parse_contact_line,
     'legacy-dir-key': _parse_legacy_dir_key_line,
     'vote-digest': _parse_vote_digest_line,
+    'shared-rand-participate': _parse_shared_rand_participate_line,
+    'shared-rand-commit': _parsed_shared_rand_commit,
+    'shared-rand-previous-value': _parse_shared_rand_previous_value,
+    'shared-rand-current-value': _parse_shared_rand_current_value,
   }
 
+  @classmethod
+  def content(cls, attr = None, exclude = (), sign = False, is_vote = False):
+    if sign:
+      raise NotImplementedError('Signing of %s not implemented' % cls.__name__)
+
+    attr = {} if attr is None else dict(attr)
+
+    # include mandatory 'vote-digest' if a consensus
+
+    if not is_vote and not ('vote-digest' in attr or (exclude and 'vote-digest' in exclude)):
+      attr['vote-digest'] = _random_fingerprint()
+
+    content = _descriptor_content(attr, exclude, (
+      ('dir-source', '%s %s no.place.com %s 9030 9090' % (_random_nickname(), _random_fingerprint(), _random_ipv4_address())),
+      ('contact', 'Mike Perry <email>'),
+    ))
+
+    if is_vote:
+      content += b'\n' + KeyCertificate.content()
+
+    return content
+
+  @classmethod
+  def create(cls, attr = None, exclude = (), validate = True, sign = False, is_vote = False):
+    return cls(cls.content(attr, exclude, sign, is_vote), validate = validate, is_vote = is_vote)
+
   def __init__(self, raw_content, validate = False, is_vote = False):
     """
     Parse a directory authority entry in a v3 network status document.
@@ -1272,7 +1462,7 @@ class DirectoryAuthority(Descriptor):
     else:
       self.key_certificate = None
 
-    entries = _get_descriptor_components(content, validate)
+    entries = _descriptor_components(content, validate)
 
     if validate and 'dir-source' != list(entries.keys())[0]:
       raise ValueError("Authority entries are expected to start with a 'dir-source' line:\n%s" % (content))
@@ -1422,9 +1612,25 @@ class KeyCertificate(Descriptor):
     'dir-key-certification': _parse_dir_key_certification_line,
   }
 
+  @classmethod
+  def content(cls, attr = None, exclude = (), sign = False):
+    if sign:
+      raise NotImplementedError('Signing of %s not implemented' % cls.__name__)
+
+    return _descriptor_content(attr, exclude, (
+      ('dir-key-certificate-version', '3'),
+      ('fingerprint', _random_fingerprint()),
+      ('dir-key-published', _random_date()),
+      ('dir-key-expires', _random_date()),
+      ('dir-identity-key', _random_crypto_blob('RSA PUBLIC KEY')),
+      ('dir-signing-key', _random_crypto_blob('RSA PUBLIC KEY')),
+    ), (
+      ('dir-key-certification', _random_crypto_blob('SIGNATURE')),
+    ))
+
   def __init__(self, raw_content, validate = False):
     super(KeyCertificate, self).__init__(raw_content, lazy_load = not validate)
-    entries = _get_descriptor_components(raw_content, validate)
+    entries = _descriptor_components(raw_content, validate)
 
     if validate:
       if 'dir-key-certificate-version' != list(entries.keys())[0]:
