@@ -1,4 +1,4 @@
-# Copyright 2012-2016, Damian Johnson and The Tor Project
+# Copyright 2012-2017, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -29,12 +29,19 @@ import stem.util.str_tools
 from stem.descriptor import (
   KEYWORD_LINE,
   Descriptor,
+  _descriptor_content,
   _value,
   _values,
-  _get_descriptor_components,
+  _descriptor_components,
+  _parse_protocol_line,
   _read_until_keywords,
+  _random_nickname,
+  _random_ipv4_address,
+  _random_date,
 )
 
+_parse_pr_line = _parse_protocol_line('pr', 'protocols')
+
 
 def _parse_file(document_file, validate, entry_class, entry_keyword = 'r', start_position = None, end_position = None, section_end_keywords = (), extra_args = ()):
   """
@@ -285,9 +292,8 @@ def _parse_id_line(descriptor, entries):
   value = _value('id', entries)
 
   if value:
-    if not (descriptor.document and descriptor.document.is_vote):
-      vote_status = 'vote' if descriptor.document else '<undefined document>'
-      raise ValueError("%s 'id' line should only appear in votes (appeared in a %s): id %s" % (descriptor._name(), vote_status, value))
+    if descriptor.document and not descriptor.document.is_vote:
+      raise ValueError("%s 'id' line should only appear in votes: id %s" % (descriptor._name(), value))
 
     value_comp = value.split()
 
@@ -365,7 +371,7 @@ def _base64_to_hex(identity, check_if_fingerprint = True):
   except (TypeError, binascii.Error):
     raise ValueError("Unable to decode identity string '%s'" % identity)
 
-  fingerprint = binascii.b2a_hex(identity_decoded).upper()
+  fingerprint = binascii.hexlify(identity_decoded).upper()
 
   if stem.prereq.is_python_3():
     fingerprint = stem.util.str_tools._to_unicode(fingerprint)
@@ -432,7 +438,7 @@ class RouterStatusEntry(Descriptor):
 
     super(RouterStatusEntry, self).__init__(content, lazy_load = not validate)
     self.document = document
-    entries = _get_descriptor_components(content, validate)
+    entries = _descriptor_components(content, validate)
 
     if validate:
       for keyword in self._required_fields():
@@ -508,6 +514,15 @@ class RouterStatusEntryV2(RouterStatusEntry):
     'digest': (None, _parse_r_line),
   })
 
+  @classmethod
+  def content(cls, attr = None, exclude = (), sign = False):
+    if sign:
+      raise NotImplementedError('Signing of %s not implemented' % cls.__name__)
+
+    return _descriptor_content(attr, exclude, (
+      ('r', '%s p1aag7VwarGxqctS7/fS0y5FU+s oQZFLYe9e4A7bOkWKR7TaNxb0JE %s %s 9001 0' % (_random_nickname(), _random_date(), _random_ipv4_address())),
+    ))
+
   def _name(self, is_plural = False):
     return 'Router status entries (v2)' if is_plural else 'Router status entry (v2)'
 
@@ -560,6 +575,7 @@ class RouterStatusEntryV3(RouterStatusEntry):
     information that isn't yet recognized
 
   :var stem.exit_policy.MicroExitPolicy exit_policy: router's exit policy
+  :var dict protocols: mapping of protocols to their supported versions
 
   :var list microdescriptor_hashes: **\*** tuples of two values, the list of
     consensus methods for generating a set of digests and the 'algorithm =>
@@ -570,6 +586,9 @@ class RouterStatusEntryV3(RouterStatusEntry):
 
   .. versionchanged:: 1.5.0
      Added the identifier and identifier_type attributes.
+
+  .. versionchanged:: 1.6.0
+     Added the protocols attribute.
   """
 
   ATTRIBUTES = dict(RouterStatusEntry.ATTRIBUTES, **{
@@ -584,6 +603,7 @@ class RouterStatusEntryV3(RouterStatusEntry):
     'unrecognized_bandwidth_entries': ([], _parse_w_line),
 
     'exit_policy': (None, _parse_p_line),
+    'protocols': ({}, _parse_pr_line),
     'microdescriptor_hashes': ([], _parse_m_line),
   })
 
@@ -591,10 +611,21 @@ class RouterStatusEntryV3(RouterStatusEntry):
     'a': _parse_a_line,
     'w': _parse_w_line,
     'p': _parse_p_line,
+    'pr': _parse_pr_line,
     'id': _parse_id_line,
     'm': _parse_m_line,
   })
 
+  @classmethod
+  def content(cls, attr = None, exclude = (), sign = False):
+    if sign:
+      raise NotImplementedError('Signing of %s not implemented' % cls.__name__)
+
+    return _descriptor_content(attr, exclude, (
+      ('r', '%s p1aag7VwarGxqctS7/fS0y5FU+s oQZFLYe9e4A7bOkWKR7TaNxb0JE %s %s 9001 0' % (_random_nickname(), _random_date(), _random_ipv4_address())),
+      ('s', 'Fast Named Running Stable Valid'),
+    ))
+
   def _name(self, is_plural = False):
     return 'Router status entries (v3)' if is_plural else 'Router status entry (v3)'
 
@@ -602,7 +633,7 @@ class RouterStatusEntryV3(RouterStatusEntry):
     return ('r', 's')
 
   def _single_fields(self):
-    return ('r', 's', 'v', 'w', 'p')
+    return ('r', 's', 'v', 'w', 'p', 'pr')
 
   def _compare(self, other, method):
     if not isinstance(other, RouterStatusEntryV3):
@@ -637,9 +668,13 @@ class RouterStatusEntryMicroV3(RouterStatusEntry):
     measurements
   :var list unrecognized_bandwidth_entries: **\*** bandwidth weighting
     information that isn't yet recognized
+  :var dict protocols: mapping of protocols to their supported versions
 
   :var str digest: **\*** router's hex encoded digest of our corresponding microdescriptor
 
+  .. versionchanged:: 1.6.0
+     Added the protocols attribute.
+
   **\*** attribute is either required when we're parsed with validation or has
   a default value, others are left as **None** if undefined
   """
@@ -649,6 +684,7 @@ class RouterStatusEntryMicroV3(RouterStatusEntry):
     'measured': (None, _parse_w_line),
     'is_unmeasured': (False, _parse_w_line),
     'unrecognized_bandwidth_entries': ([], _parse_w_line),
+    'protocols': ({}, _parse_pr_line),
 
     'digest': (None, _parse_microdescriptor_m_line),
   })
@@ -656,8 +692,20 @@ class RouterStatusEntryMicroV3(RouterStatusEntry):
   PARSER_FOR_LINE = dict(RouterStatusEntry.PARSER_FOR_LINE, **{
     'w': _parse_w_line,
     'm': _parse_microdescriptor_m_line,
+    'pr': _parse_pr_line,
   })
 
+  @classmethod
+  def content(cls, attr = None, exclude = (), sign = False):
+    if sign:
+      raise NotImplementedError('Signing of %s not implemented' % cls.__name__)
+
+    return _descriptor_content(attr, exclude, (
+      ('r', '%s ARIJF2zbqirB9IwsW0mQznccWww %s %s 9001 9030' % (_random_nickname(), _random_date(), _random_ipv4_address())),
+      ('m', 'aiUklwBrua82obG5AsTX+iEpkjQA2+AQHxZ7GwMfY70'),
+      ('s', 'Fast Guard HSDir Named Running Stable V2Dir Valid'),
+    ))
+
   def _name(self, is_plural = False):
     return 'Router status entries (micro v3)' if is_plural else 'Router status entry (micro v3)'
 
@@ -665,7 +713,7 @@ class RouterStatusEntryMicroV3(RouterStatusEntry):
     return ('r', 's', 'm')
 
   def _single_fields(self):
-    return ('r', 's', 'v', 'w', 'm')
+    return ('r', 's', 'v', 'w', 'm', 'pr')
 
   def _compare(self, other, method):
     if not isinstance(other, RouterStatusEntryMicroV3):
