@@ -1,4 +1,4 @@
-# Copyright 2012-2016, Damian Johnson and The Tor Project
+# Copyright 2012-2017, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -10,8 +10,7 @@ etc). This information is provided from a few sources...
 
 * The 'cached-descriptors' file in Tor's data directory.
 
-* Archived descriptors provided by CollecTor
-  (https://collector.torproject.org/).
+* Archived descriptors provided by `CollecTor <https://metrics.torproject.org/collector.html>`_.
 
 * Directory authorities and mirrors via their DirPort.
 
@@ -21,6 +20,7 @@ etc). This information is provided from a few sources...
 
   ServerDescriptor - Tor server descriptor.
     |- RelayDescriptor - Server descriptor for a relay.
+    |  +- make_router_status_entry - Creates a router status entry for this descriptor.
     |
     |- BridgeDescriptor - Scrubbed server descriptor for a bridge.
     |  |- is_scrubbed - checks if our content has been properly scrubbed
@@ -29,26 +29,49 @@ etc). This information is provided from a few sources...
     |- digest - calculates the upper-case hex digest value for our content
     |- get_annotations - dictionary of content prior to the descriptor entry
     +- get_annotation_lines - lines that provided the annotations
+
+.. data:: BridgeDistribution (enum)
+
+  Preferred method of distributing this relay if a bridge.
+
+  .. versionadded:: 1.6.0
+
+  ===================== ===========
+  BridgeDistribution    Description
+  ===================== ===========
+  **ANY**               No proference, BridgeDB will pick how the bridge is distributed.
+  **HTTPS**             Provided via the `web interface <https://bridges.torproject.org>`_.
+  **EMAIL**             Provided in response to emails to bridges@torproject.org.
+  **MOAT**              Provided in interactive menus within Tor Browser.
+  **HYPHAE**            Provided via a cryptographic invitation-based system.
+  ===================== ===========
 """
 
+import base64
+import binascii
 import functools
 import hashlib
 import re
 
+import stem.descriptor.certificate
 import stem.descriptor.extrainfo_descriptor
 import stem.exit_policy
 import stem.prereq
 import stem.util.connection
+import stem.util.enum
 import stem.util.str_tools
 import stem.util.tor_tools
 import stem.version
 
+from stem.descriptor.router_status_entry import RouterStatusEntryV3
 from stem.util import str_type
 
 from stem.descriptor import (
   PGP_BLOCK_END,
   Descriptor,
-  _get_descriptor_components,
+  create_signing_key,
+  _descriptor_content,
+  _descriptor_components,
   _read_until_keywords,
   _bytes_for_block,
   _value,
@@ -58,7 +81,13 @@ from stem.descriptor import (
   _parse_bytes_line,
   _parse_timestamp_line,
   _parse_forty_character_hex,
+  _parse_protocol_line,
   _parse_key_block,
+  _append_router_signature,
+  _random_nickname,
+  _random_ipv4_address,
+  _random_date,
+  _random_crypto_blob,
 )
 
 try:
@@ -89,6 +118,7 @@ SINGLE_FIELDS = (
   'read-history',
   'write-history',
   'eventdns',
+  'bridge-distribution-request',
   'family',
   'caches-extra-info',
   'extra-info-digest',
@@ -96,14 +126,28 @@ SINGLE_FIELDS = (
   'protocols',
   'allow-single-hop-exits',
   'tunnelled-dir-server',
+  'proto',
   'onion-key-crosscert',
   'ntor-onion-key',
   'ntor-onion-key-crosscert',
   'router-sig-ed25519',
 )
 
+BridgeDistribution = stem.util.enum.Enum(
+  ('ANY', 'any'),
+  ('HTTPS', 'https'),
+  ('EMAIL', 'email'),
+  ('MOAT', 'moat'),
+  ('HYPHAE', 'hyphae'),
+)
+
 DEFAULT_IPV6_EXIT_POLICY = stem.exit_policy.MicroExitPolicy('reject 1-65535')
 REJECT_ALL_POLICY = stem.exit_policy.ExitPolicy('reject *:*')
+DEFAULT_BRIDGE_DISTRIBUTION = 'any'
+
+
+def _truncated_b64encode(content):
+  return stem.util.str_tools._to_unicode(base64.b64encode(content).rstrip(b'='))
 
 
 def _parse_file(descriptor_file, is_bridge = False, validate = False, **kwargs):
@@ -274,12 +318,13 @@ def _parse_fingerprint_line(descriptor, entries):
 
 def _parse_extrainfo_digest_line(descriptor, entries):
   value = _value('extra-info-digest', entries)
-  value = value.split(' ')[0]  # lines have additional content from propsal 228, waiting for it to be documented: #16227
+  digest_comp = value.split(' ')
 
-  if not stem.util.tor_tools.is_hex_digits(value, 40):
-    raise ValueError('extra-info-digest should be 40 hex characters: %s' % value)
+  if not stem.util.tor_tools.is_hex_digits(digest_comp[0], 40):
+    raise ValueError('extra-info-digest should be 40 hex characters: %s' % digest_comp[0])
 
-  descriptor.extra_info_digest = value
+  descriptor.extra_info_digest = digest_comp[0]
+  descriptor.extra_info_sha256_digest = digest_comp[1] if len(digest_comp) >= 2 else None
 
 
 def _parse_hibernating_line(descriptor, entries):
@@ -384,7 +429,16 @@ def _parse_exit_policy(descriptor, entries):
     del descriptor._unparsed_exit_policy
 
 
-_parse_identity_ed25519_line = _parse_key_block('identity-ed25519', 'ed25519_certificate', 'ED25519 CERT')
+def _parse_identity_ed25519_line(descriptor, entries):
+  _parse_key_block('identity-ed25519', 'ed25519_certificate', 'ED25519 CERT')(descriptor, entries)
+
+  if descriptor.ed25519_certificate:
+    cert_lines = descriptor.ed25519_certificate.split('\n')
+
+    if cert_lines[0] == '-----BEGIN ED25519 CERT-----' and cert_lines[-1] == '-----END ED25519 CERT-----':
+      descriptor.certificate = stem.descriptor.certificate.Ed25519Certificate.parse(''.join(cert_lines[1:-1]))
+
+
 _parse_master_key_ed25519_line = _parse_simple_line('master-key-ed25519', 'ed25519_master_key')
 _parse_master_key_ed25519_for_hash_line = _parse_simple_line('master-key-ed25519', 'ed25519_certificate_hash')
 _parse_contact_line = _parse_bytes_line('contact', 'contact')
@@ -394,7 +448,9 @@ _parse_write_history_line = functools.partial(_parse_history_line, 'write-histor
 _parse_ipv6_policy_line = _parse_simple_line('ipv6-policy', 'exit_policy_v6', func = lambda v: stem.exit_policy.MicroExitPolicy(v))
 _parse_allow_single_hop_exits_line = _parse_if_present('allow-single-hop-exits', 'allow_single_hop_exits')
 _parse_tunneled_dir_server_line = _parse_if_present('tunnelled-dir-server', 'allow_tunneled_dir_requests')
+_parse_proto_line = _parse_protocol_line('proto', 'protocols')
 _parse_caches_extra_info_line = _parse_if_present('caches-extra-info', 'extra_info_cache')
+_parse_bridge_distribution_request_line = _parse_simple_line('bridge-distribution-request', 'bridge_distribution')
 _parse_family_line = _parse_simple_line('family', 'family', func = lambda v: set(v.split(' ')))
 _parse_eventdns_line = _parse_simple_line('eventdns', 'eventdns', func = lambda v: v == '1')
 _parse_onion_key_line = _parse_key_block('onion-key', 'onion_key', 'RSA PUBLIC KEY')
@@ -428,6 +484,8 @@ class ServerDescriptor(Descriptor):
   :var bytes contact: contact information
   :var stem.exit_policy.ExitPolicy exit_policy: **\*** stated exit policy
   :var stem.exit_policy.MicroExitPolicy exit_policy_v6: **\*** exit policy for IPv6
+  :var BridgeDistribution bridge_distribution: **\*** preferred method of providing this relay's
+    address if a bridge
   :var set family: **\*** nicknames or fingerprints of declared family
 
   :var int average_bandwidth: **\*** average rate it's willing to relay in bytes/s
@@ -442,11 +500,13 @@ class ServerDescriptor(Descriptor):
     requests are accepted
   :var bool extra_info_cache: **\*** flag if a mirror for extra-info documents
   :var str extra_info_digest: upper-case hex encoded digest of our extra-info document
+  :var str extra_info_sha256_digest: base64 encoded sha256 digest of our extra-info document
   :var bool eventdns: flag for evdns backend (**deprecated**, always unset)
   :var str ntor_onion_key: base64 key used to encrypt EXTEND in the ntor protocol
   :var list or_addresses: **\*** alternative for our address/or_port
     attributes, each entry is a tuple of the form (address (**str**), port
     (**int**), is_ipv6 (**bool**))
+  :var dict protocols: mapping of protocols to their supported versions
 
   **Deprecated**, moved to extra-info descriptor...
 
@@ -463,6 +523,10 @@ class ServerDescriptor(Descriptor):
 
   .. versionchanged:: 1.5.0
      Added the allow_tunneled_dir_requests attribute.
+
+  .. versionchanged:: 1.6.0
+     Added the extra_info_sha256_digest, protocols, and bridge_distribution
+     attributes.
   """
 
   ATTRIBUTES = {
@@ -482,6 +546,7 @@ class ServerDescriptor(Descriptor):
     'operating_system': (None, _parse_platform_line),
     'uptime': (None, _parse_uptime_line),
     'exit_policy_v6': (DEFAULT_IPV6_EXIT_POLICY, _parse_ipv6_policy_line),
+    'bridge_distribution': (DEFAULT_BRIDGE_DISTRIBUTION, _parse_bridge_distribution_request_line),
     'family': (set(), _parse_family_line),
 
     'average_bandwidth': (None, _parse_bandwidth_line),
@@ -493,8 +558,10 @@ class ServerDescriptor(Descriptor):
     'hibernating': (False, _parse_hibernating_line),
     'allow_single_hop_exits': (False, _parse_allow_single_hop_exits_line),
     'allow_tunneled_dir_requests': (False, _parse_tunneled_dir_server_line),
+    'protocols': ({}, _parse_proto_line),
     'extra_info_cache': (False, _parse_caches_extra_info_line),
     'extra_info_digest': (None, _parse_extrainfo_digest_line),
+    'extra_info_sha256_digest': (None, _parse_extrainfo_digest_line),
     'hidden_service_dir': (None, _parse_hidden_service_dir_line),
     'eventdns': (None, _parse_eventdns_line),
     'ntor_onion_key': (None, _parse_ntor_onion_key_line),
@@ -528,7 +595,9 @@ class ServerDescriptor(Descriptor):
     'ipv6-policy': _parse_ipv6_policy_line,
     'allow-single-hop-exits': _parse_allow_single_hop_exits_line,
     'tunnelled-dir-server': _parse_tunneled_dir_server_line,
+    'proto': _parse_proto_line,
     'caches-extra-info': _parse_caches_extra_info_line,
+    'bridge-distribution-request': _parse_bridge_distribution_request_line,
     'family': _parse_family_line,
     'eventdns': _parse_eventdns_line,
   }
@@ -562,7 +631,7 @@ class ServerDescriptor(Descriptor):
     # influences the resulting exit policy, but for everything else the order
     # does not matter so breaking it into key / value pairs.
 
-    entries, self._unparsed_exit_policy = _get_descriptor_components(stem.util.str_tools._to_unicode(raw_contents), validate, extra_keywords = ('accept', 'reject'), non_ascii_fields = ('contact', 'platform'))
+    entries, self._unparsed_exit_policy = _descriptor_components(stem.util.str_tools._to_unicode(raw_contents), validate, extra_keywords = ('accept', 'reject'), non_ascii_fields = ('contact', 'platform'))
 
     if validate:
       self._parse(entries, validate)
@@ -653,6 +722,12 @@ class ServerDescriptor(Descriptor):
     if expected_last_keyword and expected_last_keyword != list(entries.keys())[-1]:
       raise ValueError("Descriptor must end with a '%s' entry" % expected_last_keyword)
 
+    if 'identity-ed25519' in entries.keys():
+      if 'router-sig-ed25519' not in entries.keys():
+        raise ValueError('Descriptor must have router-sig-ed25519 entry to accompany identity-ed25519')
+      elif 'router-sig-ed25519' not in list(entries.keys())[-2:]:
+        raise ValueError("Descriptor must have 'router-sig-ed25519' as the next-to-last entry")
+
     if not self.exit_policy:
       raise ValueError("Descriptor must have at least one 'accept' or 'reject' entry")
 
@@ -677,6 +752,7 @@ class RelayDescriptor(ServerDescriptor):
   Server descriptor (`descriptor specification
   <https://gitweb.torproject.org/torspec.git/tree/dir-spec.txt>`_)
 
+  :var stem.certificate.Ed25519Certificate certificate: ed25519 certificate
   :var str ed25519_certificate: base64 encoded ed25519 certificate
   :var str ed25519_master_key: base64 encoded master key for our ed25519 certificate
   :var str ed25519_signature: signature of this document using ed25519
@@ -694,9 +770,26 @@ class RelayDescriptor(ServerDescriptor):
      Added the ed25519_certificate, ed25519_master_key, ed25519_signature,
      onion_key_crosscert, ntor_onion_key_crosscert, and
      ntor_onion_key_crosscert_sign attributes.
+
+  .. versionchanged:: 1.6.0
+     Moved from the deprecated `pycrypto
+     <https://www.dlitz.net/software/pycrypto/>`_ module to `cryptography
+     <https://pypi.python.org/pypi/cryptography>`_ for validating signatures.
+
+  .. versionchanged:: 1.6.0
+     Added the certificate attribute.
+
+  .. deprecated:: 1.6.0
+     Our **ed25519_certificate** is deprecated in favor of our new
+     **certificate** attribute. The base64 encoded certificate is available via
+     the certificate's **encoded** attribute.
+
+  .. versionchanged:: 1.6.0
+     Added the **skip_crypto_validation** constructor argument.
   """
 
   ATTRIBUTES = dict(ServerDescriptor.ATTRIBUTES, **{
+    'certificate': (None, _parse_identity_ed25519_line),
     'ed25519_certificate': (None, _parse_identity_ed25519_line),
     'ed25519_master_key': (None, _parse_master_key_ed25519_line),
     'ed25519_signature': (None, _parse_router_sig_ed25519_line),
@@ -720,7 +813,7 @@ class RelayDescriptor(ServerDescriptor):
     'router-signature': _parse_router_signature_line,
   })
 
-  def __init__(self, raw_contents, validate = False, annotations = None):
+  def __init__(self, raw_contents, validate = False, annotations = None, skip_crypto_validation = False):
     super(RelayDescriptor, self).__init__(raw_contents, validate, annotations)
 
     if validate:
@@ -730,12 +823,65 @@ class RelayDescriptor(ServerDescriptor):
         if key_hash != self.fingerprint.lower():
           raise ValueError('Fingerprint does not match the hash of our signing key (fingerprint: %s, signing key hash: %s)' % (self.fingerprint.lower(), key_hash))
 
-      if stem.prereq.is_crypto_available():
+      if not skip_crypto_validation and stem.prereq.is_crypto_available():
         signed_digest = self._digest_for_signature(self.signing_key, self.signature)
 
         if signed_digest != self.digest():
           raise ValueError('Decrypted digest does not match local digest (calculated: %s, local: %s)' % (signed_digest, self.digest()))
 
+        if self.onion_key_crosscert:
+          onion_key_crosscert_digest = self._digest_for_signature(self.onion_key, self.onion_key_crosscert)
+
+          if onion_key_crosscert_digest != self._onion_key_crosscert_digest():
+            raise ValueError('Decrypted onion-key-crosscert digest does not match local digest (calculated: %s, local: %s)' % (onion_key_crosscert_digest, self._onion_key_crosscert_digest()))
+
+      if stem.prereq._is_pynacl_available() and self.certificate:
+        self.certificate.validate(self)
+
+  @classmethod
+  def content(cls, attr = None, exclude = (), sign = False, signing_key = None):
+    if signing_key:
+      sign = True
+
+    if attr is None:
+      attr = {}
+
+    base_header = (
+      ('router', '%s %s 9001 0 0' % (_random_nickname(), _random_ipv4_address())),
+      ('published', _random_date()),
+      ('bandwidth', '153600 256000 104590'),
+      ('reject', '*:*'),
+      ('onion-key', _random_crypto_blob('RSA PUBLIC KEY')),
+      ('signing-key', _random_crypto_blob('RSA PUBLIC KEY')),
+    )
+
+    if sign:
+      if attr and 'signing-key' in attr:
+        raise ValueError('Cannot sign the descriptor if a signing-key has been provided')
+      elif attr and 'router-signature' in attr:
+        raise ValueError('Cannot sign the descriptor if a router-signature has been provided')
+
+      if signing_key is None:
+        signing_key = create_signing_key()
+
+      if 'fingerprint' not in attr:
+        fingerprint = hashlib.sha1(_bytes_for_block(stem.util.str_tools._to_unicode(signing_key.public_digest.strip()))).hexdigest().upper()
+        attr['fingerprint'] = ' '.join(stem.util.str_tools._split_by_length(fingerprint, 4))
+
+      attr['signing-key'] = signing_key.public_digest
+
+      content = _descriptor_content(attr, exclude, base_header) + b'\nrouter-signature\n'
+      return _append_router_signature(content, signing_key.private)
+    else:
+      return _descriptor_content(attr, exclude, base_header, (
+        ('router-sig-ed25519', None),
+        ('router-signature', _random_crypto_blob('SIGNATURE')),
+      ))
+
+  @classmethod
+  def create(cls, attr = None, exclude = (), validate = True, sign = False, signing_key = None):
+    return cls(cls.content(attr, exclude, sign, signing_key), validate = validate, skip_crypto_validation = not sign)
+
   @lru_cache()
   def digest(self):
     """
@@ -743,11 +889,64 @@ class RelayDescriptor(ServerDescriptor):
 
     :returns: the digest string encoded in uppercase hex
 
-    :raises: ValueError if the digest canot be calculated
+    :raises: ValueError if the digest cannot be calculated
     """
 
     return self._digest_for_content(b'router ', b'\nrouter-signature\n')
 
+  def make_router_status_entry(self):
+    """
+    Provides a RouterStatusEntryV3 for this descriptor content.
+
+    .. versionadded:: 1.6.0
+
+    :returns: :class:`~stem.descriptor.router_status_entry.RouterStatusEntryV3`
+      that would be in the consensus
+    """
+
+    if not self.fingerprint:
+      raise ValueError('Server descriptor lacks a fingerprint. This is an optional field, but required to make a router status entry.')
+
+    attr = {
+      'r': ' '.join([
+        self.nickname,
+        _truncated_b64encode(binascii.unhexlify(stem.util.str_tools._to_bytes(self.fingerprint))),
+        _truncated_b64encode(binascii.unhexlify(stem.util.str_tools._to_bytes(self.digest()))),
+        self.published.strftime('%Y-%m-%d %H:%M:%S'),
+        self.address,
+        str(self.or_port),
+        str(self.dir_port) if self.dir_port else '0',
+      ]),
+      'w': 'Bandwidth=%i' % self.average_bandwidth,
+      'p': self.exit_policy.summary().replace(', ', ','),
+    }
+
+    if self.tor_version:
+      attr['v'] = 'Tor %s' % self.tor_version
+
+    if self.or_addresses:
+      attr['a'] = ['%s:%s' % (addr, port) for addr, port, _ in self.or_addresses]
+
+    if self.certificate:
+      attr['id'] = 'ed25519 %s' % _truncated_b64encode(self.certificate.key)
+
+    return RouterStatusEntryV3.create(attr)
+
+  @lru_cache()
+  def _onion_key_crosscert_digest(self):
+    """
+    Provides the digest of the onion-key-crosscert data. This consists of the
+    RSA identity key sha1 and ed25519 identity key.
+
+    :returns: **unicode** digest encoded in uppercase hex
+
+    :raises: ValueError if the digest cannot be calculated
+    """
+
+    signing_key_digest = hashlib.sha1(_bytes_for_block(self.signing_key)).digest()
+    data = signing_key_digest + base64.b64decode(stem.util.str_tools._to_bytes(self.ed25519_master_key) + b'=')
+    return stem.util.str_tools._to_unicode(binascii.hexlify(data).upper())
+
   def _compare(self, other, method):
     if not isinstance(other, RelayDescriptor):
       return False
@@ -782,7 +981,7 @@ class RelayDescriptor(ServerDescriptor):
 class BridgeDescriptor(ServerDescriptor):
   """
   Bridge descriptor (`bridge descriptor specification
-  <https://collector.torproject.org/formats.html#bridge-descriptors>`_)
+  <https://metrics.torproject.org/collector.html#bridge-descriptors>`_)
 
   :var str ed25519_certificate_hash: sha256 hash of the original identity-ed25519
   :var str router_digest_sha256: sha256 digest of this document
@@ -805,6 +1004,19 @@ class BridgeDescriptor(ServerDescriptor):
     'router-digest': _parse_router_digest_line,
   })
 
+  @classmethod
+  def content(cls, attr = None, exclude = (), sign = False):
+    if sign:
+      raise NotImplementedError('Signing of %s not implemented' % cls.__name__)
+
+    return _descriptor_content(attr, exclude, (
+      ('router', '%s %s 9001 0 0' % (_random_nickname(), _random_ipv4_address())),
+      ('router-digest', '006FD96BA35E7785A6A3B8B75FE2E2435A13BDB4'),
+      ('published', _random_date()),
+      ('bandwidth', '409600 819200 5120'),
+      ('reject', '*:*'),
+    ))
+
   def digest(self):
     return self._digest
 
@@ -812,7 +1024,7 @@ class BridgeDescriptor(ServerDescriptor):
     """
     Checks if we've been properly scrubbed in accordance with the `bridge
     descriptor specification
-    <https://collector.torproject.org/formats.html#bridge-descriptors>`_.
+    <https://metrics.torproject.org/collector.html#bridge-descriptors>`_.
     Validation is a moving target so this may not be fully up to date.
 
     :returns: **True** if we're scrubbed, **False** otherwise
