@@ -1,4 +1,4 @@
-# Copyright 2012-2016, Damian Johnson and The Tor Project
+# Copyright 2012-2017, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -30,11 +30,13 @@ Parses replies from the control socket.
     +- pop_mapping - removes and returns the next entry as a KEY=VALUE mapping
 """
 
+import codecs
 import io
 import re
 import threading
 
 import stem.socket
+import stem.util.str_tools
 
 __all__ = [
   'add_onion',
@@ -51,16 +53,6 @@ __all__ = [
 
 KEY_ARG = re.compile('^(\S+)=')
 
-# Escape sequences from the 'esc_for_log' function of tor's 'common/util.c'.
-# It's hard to tell what controller functions use this in practice, but direct
-# users are...
-# - 'COOKIEFILE' field of PROTOCOLINFO responses
-# - logged messages about bugs
-# - the 'getinfo_helper_listeners' function of control.c
-
-CONTROL_ESCAPES = {r'\\': '\\', r'\"': '\"', r'\'': '\'',
-                   r'\r': '\r', r'\n': '\n', r'\t': '\t'}
-
 
 def convert(response_type, message, **kwargs):
   """
@@ -140,20 +132,31 @@ class ControlMessage(object):
   """
 
   @staticmethod
-  def from_str(content, msg_type = None, **kwargs):
+  def from_str(content, msg_type = None, normalize = False, **kwargs):
     """
     Provides a ControlMessage for the given content.
 
     .. versionadded:: 1.1.0
 
+    .. versionchanged:: 1.6.0
+       Added the normalize argument.
+
     :param str content: message to construct the message from
     :param str msg_type: type of tor reply to parse the content as
+    :param bool normalize: ensures expected carriage return and ending newline
+      are present
     :param kwargs: optional keyword arguments to be passed to the parser method
 
     :returns: stem.response.ControlMessage instance
     """
 
-    msg = stem.socket.recv_message(io.StringIO(stem.util.str_tools._to_unicode(content)))
+    if normalize:
+      if not content.endswith('\n'):
+        content += '\n'
+
+      content = re.sub('([\r]?)\n', '\r\n', content)
+
+    msg = stem.socket.recv_message(io.BytesIO(stem.util.str_tools._to_bytes(content)))
 
     if msg_type is not None:
       convert(msg_type, msg, **kwargs)
@@ -166,6 +169,7 @@ class ControlMessage(object):
 
     self._parsed_content = parsed_content
     self._raw_content = raw_content
+    self._str = None
 
   def is_ok(self):
     """
@@ -242,7 +246,10 @@ class ControlMessage(object):
     formatting.
     """
 
-    return '\n'.join(list(self))
+    if self._str is None:
+      self._str = '\n'.join(list(self))
+
+    return self._str
 
   def __iter__(self):
     """
@@ -333,7 +340,7 @@ class ControlLine(str):
     """
     Checks if our next entry is a quoted value or not.
 
-    :param bool escaped: unescapes the CONTROL_ESCAPES escape sequences
+    :param bool escaped: unescapes the string
 
     :returns: **True** if the next entry can be parsed as a quoted value, **False** otherwise
     """
@@ -347,7 +354,7 @@ class ControlLine(str):
 
     :param str key: checks that the key matches this value, skipping the check if **None**
     :param bool quoted: checks that the mapping is to a quoted value
-    :param bool escaped: unescapes the CONTROL_ESCAPES escape sequences
+    :param bool escaped: unescapes the string
 
     :returns: **True** if the next entry can be parsed as a key=value mapping,
       **False** otherwise
@@ -405,7 +412,7 @@ class ControlLine(str):
         "this has a \\" and \\\\ in it"
 
     :param bool quoted: parses the next entry as a quoted value, removing the quotes
-    :param bool escaped: unescapes the CONTROL_ESCAPES escape sequences
+    :param bool escaped: unescapes the string
 
     :returns: **str** of the next space separated entry
 
@@ -415,17 +422,21 @@ class ControlLine(str):
     """
 
     with self._remainder_lock:
-      next_entry, remainder = _parse_entry(self._remainder, quoted, escaped)
+      next_entry, remainder = _parse_entry(self._remainder, quoted, escaped, False)
       self._remainder = remainder
       return next_entry
 
-  def pop_mapping(self, quoted = False, escaped = False):
+  def pop_mapping(self, quoted = False, escaped = False, get_bytes = False):
     """
     Parses the next space separated entry as a KEY=VALUE mapping, removing it
     and the space from our remaining content.
 
+    .. versionchanged:: 1.6.0
+       Added the get_bytes argument.
+
     :param bool quoted: parses the value as being quoted, removing the quotes
-    :param bool escaped: unescapes the CONTROL_ESCAPES escape sequences
+    :param bool escaped: unescapes the string
+    :param bool get_bytes: provides **bytes** for the **value** rather than a **str**
 
     :returns: **tuple** of the form (key, value)
 
@@ -447,18 +458,18 @@ class ControlLine(str):
       key = key_match.groups()[0]
       remainder = self._remainder[key_match.end():]
 
-      next_entry, remainder = _parse_entry(remainder, quoted, escaped)
+      next_entry, remainder = _parse_entry(remainder, quoted, escaped, get_bytes)
       self._remainder = remainder
       return (key, next_entry)
 
 
-def _parse_entry(line, quoted, escaped):
+def _parse_entry(line, quoted, escaped, get_bytes):
   """
   Parses the next entry from the given space separated content.
 
   :param str line: content to be parsed
   :param bool quoted: parses the next entry as a quoted value, removing the quotes
-  :param bool escaped: unescapes the CONTROL_ESCAPES escape sequences
+  :param bool escaped: unescapes the string
 
   :returns: **tuple** of the form (entry, remainder)
 
@@ -488,7 +499,26 @@ def _parse_entry(line, quoted, escaped):
       next_entry, remainder = remainder, ''
 
   if escaped:
-    next_entry = _unescape(next_entry)
+    # Tor does escaping in its 'esc_for_log' function of 'common/util.c'. It's
+    # hard to tell what controller functions use this in practice, but direct
+    # users are...
+    #
+    #   * 'COOKIEFILE' field of PROTOCOLINFO responses
+    #   * logged messages about bugs
+    #   * the 'getinfo_helper_listeners' function of control.c
+    #
+    # Ideally we'd use "next_entry.decode('string_escape')" but it was removed
+    # in python 3.x and 'unicode_escape' isn't quite the same...
+    #
+    #   https://stackoverflow.com/questions/14820429/how-do-i-decodestring-escape-in-python3
+
+    next_entry = codecs.escape_decode(next_entry)[0]
+
+    if stem.prereq.is_python_3() and not get_bytes:
+      next_entry = stem.util.str_tools._to_unicode(next_entry)  # normalize back to str
+
+  if get_bytes:
+    next_entry = stem.util.str_tools._to_bytes(next_entry)
 
   return (next_entry, remainder.lstrip())
 
@@ -498,7 +528,7 @@ def _get_quote_indices(line, escaped):
   Provides the indices of the next two quotes in the given content.
 
   :param str line: content to be parsed
-  :param bool escaped: unescapes the CONTROL_ESCAPES escape sequences
+  :param bool escaped: unescapes the string
 
   :returns: **tuple** of two ints, indices being -1 if a quote doesn't exist
   """
@@ -519,34 +549,6 @@ def _get_quote_indices(line, escaped):
   return tuple(indices)
 
 
-def _unescape(entry):
-  # Unescapes the given string with the mappings in CONTROL_ESCAPES.
-  #
-  # This can't be a simple series of str.replace() calls because replacements
-  # need to be excluded from consideration for further unescaping. For
-  # instance, '\\t' should be converted to '\t' rather than a tab.
-
-  def _pop_with_unescape(entry):
-    # Pop either the first character or the escape sequence conversion the
-    # entry starts with. This provides a tuple of...
-    #
-    #   (unescaped prefix, remaining entry)
-
-    for esc_sequence, replacement in CONTROL_ESCAPES.items():
-      if entry.startswith(esc_sequence):
-        return (replacement, entry[len(esc_sequence):])
-
-    return (entry[0], entry[1:])
-
-  result = []
-
-  while entry:
-    prefix, entry = _pop_with_unescape(entry)
-    result.append(prefix)
-
-  return ''.join(result)
-
-
 class SingleLineResponse(ControlMessage):
   """
   Reply to a request that performs an action rather than querying data. These
