@@ -1,4 +1,4 @@
-# Copyright 2012-2016, Damian Johnson and The Tor Project
+# Copyright 2012-2017, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 import io
@@ -20,6 +20,7 @@ from stem.util import str_type, int_type, connection, log, str_tools, tor_tools
 KW_ARG = re.compile('^(.*) ([A-Za-z0-9_]+)=(\S*)$')
 QUOTED_KW_ARG = re.compile('^(.*) ([A-Za-z0-9_]+)="(.*)"$')
 CELL_TYPE = re.compile('^[a-z0-9_]+$')
+PARSE_NEWCONSENSUS_EVENTS = True
 
 
 class Event(stem.response.ControlMessage):
@@ -125,6 +126,25 @@ class Event(stem.response.ControlMessage):
     for controller_attr_name, attr_name in self._KEYWORD_ARGS.items():
       setattr(self, attr_name, self.keyword_args.get(controller_attr_name))
 
+  def _iso_timestamp(self, timestamp):
+    """
+    Parses an iso timestamp (ISOTime2Frac in the control-spec).
+
+    :param str timestamp: timestamp to parse
+
+    :returns: **datetime** with the parsed timestamp
+
+    :raises: :class:`stem.ProtocolError` if timestamp is malformed
+    """
+
+    if timestamp is None:
+      return None
+
+    try:
+      return str_tools._parse_iso_timestamp(timestamp)
+    except ValueError as exc:
+      raise stem.ProtocolError('Unable to parse timestamp (%s): %s' % (exc, self))
+
   # method overwritten by our subclasses for special handling that they do
   def _parse(self):
     pass
@@ -211,7 +231,11 @@ class AuthDirNewDescEvent(Event):
   descriptors. The descriptor type contained within this event is unspecified
   so the descriptor contents are left unparsed.
 
-  The AUTHDIR_NEWDESCS event was introduced in tor version 0.1.1.10-alpha.
+  The AUTHDIR_NEWDESCS event was introduced in tor version 0.1.1.10-alpha and
+  removed in 0.3.2.1-alpha. (:spec:`6e887ba`)
+
+  .. deprecated:: 1.6.0
+     Tor dropped this event as of version 0.3.2.1. (:spec:`6e887ba`)
 
   :var stem.AuthDescriptorAction action: what is being done with the descriptor
   :var str message: explanation of why we chose this action
@@ -364,16 +388,11 @@ class CircuitEvent(Event):
 
   def _parse(self):
     self.path = tuple(stem.control._parse_circ_path(self.path))
+    self.created = self._iso_timestamp(self.created)
 
     if self.build_flags is not None:
       self.build_flags = tuple(self.build_flags.split(','))
 
-    if self.created is not None:
-      try:
-        self.created = str_tools._parse_iso_timestamp(self.created)
-      except ValueError as exc:
-        raise stem.ProtocolError('Unable to parse create date (%s): %s' % (exc, self))
-
     if not tor_tools.is_valid_circuit_id(self.id):
       raise stem.ProtocolError("Circuit IDs must be one to sixteen alphanumeric characters, got '%s': %s" % (self.id, self))
 
@@ -463,16 +482,11 @@ class CircMinorEvent(Event):
 
   def _parse(self):
     self.path = tuple(stem.control._parse_circ_path(self.path))
+    self.created = self._iso_timestamp(self.created)
 
     if self.build_flags is not None:
       self.build_flags = tuple(self.build_flags.split(','))
 
-    if self.created is not None:
-      try:
-        self.created = str_tools._parse_iso_timestamp(self.created)
-      except ValueError as exc:
-        raise stem.ProtocolError('Unable to parse create date (%s): %s' % (exc, self))
-
     if not tor_tools.is_valid_circuit_id(self.id):
       raise stem.ProtocolError("Circuit IDs must be one to sixteen alphanumeric characters, got '%s': %s" % (self.id, self))
 
@@ -657,11 +671,12 @@ class HSDescEvent(Event):
     self.directory_fingerprint = None
     self.directory_nickname = None
 
-    try:
-      self.directory_fingerprint, self.directory_nickname = \
-        stem.control._parse_circ_entry(self.directory)
-    except stem.ProtocolError:
-      raise stem.ProtocolError("HS_DESC's directory doesn't match a ServerSpec: %s" % self)
+    if self.directory != 'UNKNOWN':
+      try:
+        self.directory_fingerprint, self.directory_nickname = \
+          stem.control._parse_circ_entry(self.directory)
+      except stem.ProtocolError:
+        raise stem.ProtocolError("HS_DESC's directory doesn't match a ServerSpec: %s" % self)
 
     if self.replica is not None:
       if not self.replica.isdigit():
@@ -759,7 +774,7 @@ class NetworkStatusEvent(Event):
 
     self.desc = list(stem.descriptor.router_status_entry._parse_file(
       io.BytesIO(str_tools._to_bytes(content)),
-      True,
+      False,
       entry_class = stem.descriptor.router_status_entry.RouterStatusEntryV3,
     ))
 
@@ -789,6 +804,19 @@ class NewConsensusEvent(Event):
 
   The NEWCONSENSUS event was introduced in tor version 0.2.1.13-alpha.
 
+  .. versionchanged:: 1.6.0
+     Added the consensus_content attribute.
+
+  .. deprecated:: 1.6.0
+     In Stem 2.0 we'll remove the desc attribute, so this event only provides
+     the unparsed consensus. Callers can then parse it if they'd like. To drop
+     parsing before then you can set...
+
+     ::
+
+       stem.response.events.PARSE_NEWCONSENSUS_EVENTS = False
+
+  :var str consensus_content: consensus content
   :var list desc: :class:`~stem.descriptor.router_status_entry.RouterStatusEntryV3` for the changed descriptors
   """
 
@@ -796,16 +824,19 @@ class NewConsensusEvent(Event):
   _VERSION_ADDED = stem.version.Requirement.EVENT_NEWCONSENSUS
 
   def _parse(self):
-    content = str(self).lstrip('NEWCONSENSUS\n').rstrip('\nOK')
+    self.consensus_content = str(self).lstrip('NEWCONSENSUS\n').rstrip('\nOK')
 
     # TODO: For stem 2.0.0 consider changing 'desc' to 'descriptors' to match
     # our other events.
 
-    self.desc = list(stem.descriptor.router_status_entry._parse_file(
-      io.BytesIO(str_tools._to_bytes(content)),
-      True,
-      entry_class = stem.descriptor.router_status_entry.RouterStatusEntryV3,
-    ))
+    if PARSE_NEWCONSENSUS_EVENTS:
+      self.desc = list(stem.descriptor.router_status_entry._parse_file(
+        io.BytesIO(str_tools._to_bytes(self.consensus_content)),
+        False,
+        entry_class = stem.descriptor.router_status_entry.RouterStatusEntryV3,
+      ))
+    else:
+      self.desc = None
 
 
 class NewDescEvent(Event):
@@ -1049,12 +1080,16 @@ class StreamBwEvent(Event):
 
   The STREAM_BW event was introduced in tor version 0.1.2.8-beta.
 
+  .. versionchanged:: 1.6.0
+     Added the time attribute.
+
   :var str id: stream identifier
   :var long written: bytes sent by the application
   :var long read: bytes received by the application
+  :var datetime time: time when the measurement was recorded
   """
 
-  _POSITIONAL_ARGS = ('id', 'written', 'read')
+  _POSITIONAL_ARGS = ('id', 'written', 'read', 'time')
   _VERSION_ADDED = stem.version.Requirement.EVENT_STREAM_BW
 
   def _parse(self):
@@ -1069,6 +1104,7 @@ class StreamBwEvent(Event):
 
     self.read = int_type(self.read)
     self.written = int_type(self.written)
+    self.time = self._iso_timestamp(self.time)
 
 
 class TransportLaunchedEvent(Event):
@@ -1112,15 +1148,19 @@ class ConnectionBandwidthEvent(Event):
 
   .. versionadded:: 1.2.0
 
+  .. versionchanged:: 1.6.0
+     Renamed 'type' attribute to 'conn_type' so it wouldn't be override parent
+     class attribute with the same name.
+
   :var str id: connection identifier
-  :var stem.ConnectionType type: connection type
+  :var stem.ConnectionType conn_type: connection type
   :var long read: bytes received by tor that second
   :var long written: bytes sent by tor that second
   """
 
   _KEYWORD_ARGS = {
     'ID': 'id',
-    'TYPE': 'type',
+    'TYPE': 'conn_type',
     'READ': 'read',
     'WRITTEN': 'written',
   }
@@ -1130,8 +1170,8 @@ class ConnectionBandwidthEvent(Event):
   def _parse(self):
     if not self.id:
       raise stem.ProtocolError('CONN_BW event is missing its id')
-    elif not self.type:
-      raise stem.ProtocolError('CONN_BW event is missing its type')
+    elif not self.conn_type:
+      raise stem.ProtocolError('CONN_BW event is missing its connection type')
     elif not self.read:
       raise stem.ProtocolError('CONN_BW event is missing its read value')
     elif not self.written:
@@ -1144,7 +1184,7 @@ class ConnectionBandwidthEvent(Event):
     self.read = int_type(self.read)
     self.written = int_type(self.written)
 
-    self._log_if_unrecognized('type', stem.ConnectionType)
+    self._log_if_unrecognized('conn_type', stem.ConnectionType)
 
 
 class CircuitBandwidthEvent(Event):
@@ -1156,15 +1196,20 @@ class CircuitBandwidthEvent(Event):
 
   .. versionadded:: 1.2.0
 
+  .. versionchanged:: 1.6.0
+     Added the time attribute.
+
   :var str id: circuit identifier
   :var long read: bytes received by tor that second
   :var long written: bytes sent by tor that second
+  :var datetime time: time when the measurement was recorded
   """
 
   _KEYWORD_ARGS = {
     'ID': 'id',
     'READ': 'read',
     'WRITTEN': 'written',
+    'TIME': 'time',
   }
 
   _VERSION_ADDED = stem.version.Requirement.EVENT_CIRC_BW
@@ -1183,6 +1228,7 @@ class CircuitBandwidthEvent(Event):
 
     self.read = int_type(self.read)
     self.written = int_type(self.written)
+    self.time = self._iso_timestamp(self.time)
 
 
 class CellStatsEvent(Event):
