@@ -1,4 +1,4 @@
-# Copyright 2011-2016, Damian Johnson and The Tor Project
+# Copyright 2011-2017, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -79,6 +79,9 @@ try:
 except AttributeError:
   CLOCK_TICKS = None
 
+IS_LITTLE_ENDIAN = sys.byteorder == 'little'
+ENCODED_ADDR = {}  # cache of encoded ips to their decoded version
+
 Stat = stem.util.enum.Enum(
   ('COMMAND', 'command'), ('CPU_UTIME', 'utime'),
   ('CPU_STIME', 'stime'), ('START_TIME', 'start time')
@@ -368,34 +371,80 @@ def connections(pid = None, user = None):
     if not IS_PWD_AVAILABLE:
       raise IOError("This requires python's pwd module, which is unavailable on Windows.")
 
-    inodes = _inodes_for_sockets(pid) if pid else []
-    process_uid = pwd.getpwnam(user).pw_uid if user else None
+    inodes = _inodes_for_sockets(pid) if pid else set()
+    process_uid = stem.util.str_tools._to_bytes(str(pwd.getpwnam(user).pw_uid)) if user else None
 
     for proc_file_path in ('/proc/net/tcp', '/proc/net/tcp6', '/proc/net/udp', '/proc/net/udp6'):
       if proc_file_path.endswith('6') and not os.path.exists(proc_file_path):
         continue  # ipv6 proc contents are optional
 
+      protocol = proc_file_path[10:].rstrip('6')  # 'tcp' or 'udp'
+      is_tcp, is_ipv6 = protocol == 'tcp', proc_file_path.endswith('6')
+      title = ''
+
       try:
         with open(proc_file_path, 'rb') as proc_file:
-          proc_file.readline()  # skip the first line
+          title = proc_file.readline()
+
+          if b'local_address' in title:
+            laddr_start = title.index(b'local_address')
+            laddr_end = laddr_start + (8 if not is_ipv6 else 32)
+
+            lport_start = laddr_end + 1
+            lport_end = lport_start + 4
+          else:
+            raise IOError("title line missing 'local_address', %s" % title)
+
+          if b'rem_address' in title or b'remote_address' in title:
+            raddr_start = title.index(b'rem_address') if b'rem_address' in title else title.index(b'remote_address')
+            raddr_end = raddr_start + (8 if not is_ipv6 else 32)
+
+            rport_start = raddr_end + 1
+            rport_end = rport_start + 4
+          else:
+            raise IOError("title line missing 'remote_address', %s" % title)
+
+          if b'st' in title:
+            status_start = title.index(b'st')
+            status_end = status_start + 2
+          else:
+            raise IOError("title line missing 'st', %s" % title)
+
+          if b'retrnsmt' in title and b'uid' in title:
+            # unlike the above fields uid is right aligned
+            uid_start = title.index(b'retrnsmt') + 9
+            uid_end = title.index(b'uid') + 3
+          elif b'retrnsmt' not in title:
+            raise IOError("title line missing 'retrnsmt', %s" % title)
+          else:
+            raise IOError("title line missing 'uid', %s" % title)
+
+          if b'timeout' in title:
+            # inodes can lack a header, and are a dynamic size
+            inode_start = title.index(b'timeout') + 8
+          else:
+            raise IOError("title line missing 'timeout', %s" % title)
 
           for line in proc_file:
-            _, l_addr, f_addr, status, _, _, _, uid, _, inode = line.split()[:10]
-            protocol = proc_file_path[10:].rstrip('6')  # 'tcp' or 'udp'
-            is_ipv6 = proc_file_path.endswith('6')
-
-            if inodes and inode not in inodes:
+            if inodes and line[inode_start:].split(b' ', 1)[0] not in inodes:
               continue
-            elif process_uid and int(uid) != process_uid:
+            elif process_uid and line[uid_start:uid_end].strip() != process_uid:
               continue
-            elif protocol == 'tcp' and status != b'01':
+            elif is_tcp and line[status_start:status_end] != b'01':
               continue  # skip tcp connections that aren't yet established
-            elif protocol == 'udp' and (f_addr == '00000000:0000' or f_addr == '00000000000000000000000000000000:0000'):
-              continue  # skip udp connections with a blank destination
 
-            local_ip, local_port = _decode_proc_address_encoding(l_addr, is_ipv6)
-            foreign_ip, foreign_port = _decode_proc_address_encoding(f_addr, is_ipv6)
-            conn.append(stem.util.connection.Connection(local_ip, local_port, foreign_ip, foreign_port, protocol, is_ipv6))
+            l_addr = _unpack_addr(line[laddr_start:laddr_end])
+            l_port = int(line[lport_start:lport_end], 16)
+
+            r_addr = _unpack_addr(line[raddr_start:raddr_end])
+            r_port = int(line[rport_start:rport_end], 16)
+
+            if r_addr == '0.0.0.0' or r_addr == '0000:0000:0000:0000:0000:0000':
+              continue  # no address
+            elif l_port == 0 or r_port == 0:
+              continue  # no port
+
+            conn.append(stem.util.connection.Connection(l_addr, l_port, r_addr, r_port, protocol, is_ipv6))
       except IOError as exc:
         raise IOError("unable to read '%s': %s" % (proc_file_path, exc))
       except Exception as exc:
@@ -414,12 +463,12 @@ def _inodes_for_sockets(pid):
 
   :param int pid: process id of the process to be queried
 
-  :returns: **list** with inodes for its sockets
+  :returns: **set** with inodes for its sockets
 
   :raises: **IOError** if it can't be determined
   """
 
-  inodes = []
+  inodes = set()
 
   try:
     fd_contents = os.listdir('/proc/%s/fd' % pid)
@@ -435,7 +484,7 @@ def _inodes_for_sockets(pid):
       fd_name = os.readlink(fd_path)
 
       if fd_name.startswith('socket:['):
-        inodes.append(stem.util.str_tools._to_bytes(fd_name[8:-1]))
+        inodes.add(stem.util.str_tools._to_bytes(fd_name[8:-1]))
     except OSError as exc:
       if not os.path.exists(fd_path):
         continue  # descriptors may shift while we're in the middle of iterating over them
@@ -446,7 +495,7 @@ def _inodes_for_sockets(pid):
   return inodes
 
 
-def _decode_proc_address_encoding(addr, is_ipv6):
+def _unpack_addr(addr):
   """
   Translates an address entry in the /proc/net/* contents to a human readable
   form (`reference <http://linuxdevcenter.com/pub/a/linux/2000/11/16/LinuxAdmin.html>`_,
@@ -454,39 +503,40 @@ def _decode_proc_address_encoding(addr, is_ipv6):
 
   ::
 
-    "0500000A:0016" -> ("10.0.0.5", 22)
-    "F804012A4A5190010000000002000000:01BB" -> ("2a01:4f8:190:514a::2", 443)
+    "0500000A" -> "10.0.0.5"
+    "F804012A4A5190010000000002000000" -> "2a01:4f8:190:514a::2"
 
   :param str addr: proc address entry to be decoded
-  :param bool is_ipv6: if we should treat the address as ipv6
 
-  :returns: **tuple** of the form **(addr, port)**, with addr as a string and port an int
+  :returns: **str** of the decoded address
   """
 
-  ip, port = addr.rsplit(b':', 1)
+  if addr not in ENCODED_ADDR:
+    if len(addr) == 8:
+      # IPv4 address
+      decoded = base64.b16decode(addr)[::-1] if IS_LITTLE_ENDIAN else base64.b16decode(addr)
+      ENCODED_ADDR[addr] = socket.inet_ntop(socket.AF_INET, decoded)
+    else:
+      # IPv6 address
 
-  port = int(port, 16)  # the port is represented as a two-byte hexadecimal number
+      if IS_LITTLE_ENDIAN:
+        # Group into eight characters, then invert in pairs...
+        #
+        #   https://trac.torproject.org/projects/tor/ticket/18079#comment:24
 
-  if not is_ipv6:
-    ip_encoded = base64.b16decode(ip)[::-1] if sys.byteorder == 'little' else base64.b16decode(ip)
-    ip = socket.inet_ntop(socket.AF_INET, ip_encoded)
-  else:
-    if sys.byteorder == 'little':
-      # Group into eight characters, then invert in pairs...
-      #
-      #   https://trac.torproject.org/projects/tor/ticket/18079#comment:24
+        inverted = []
 
-      inverted = []
+        for i in range(4):
+          grouping = addr[8 * i:8 * (i + 1)]
+          inverted += [grouping[2 * i:2 * (i + 1)] for i in range(4)][::-1]
 
-      for i in range(4):
-        grouping = ip[8 * i:8 * (i + 1)]
-        inverted += [grouping[2 * i:2 * (i + 1)] for i in range(4)][::-1]
+        encoded = b''.join(inverted)
+      else:
+        encoded = addr
 
-      ip = b''.join(inverted)
+      ENCODED_ADDR[addr] = stem.util.connection.expand_ipv6_address(socket.inet_ntop(socket.AF_INET6, base64.b16decode(encoded)))
 
-    ip = stem.util.connection.expand_ipv6_address(socket.inet_ntop(socket.AF_INET6, base64.b16decode(ip)))
-
-  return (ip, port)
+  return ENCODED_ADDR[addr]
 
 
 def _is_float(*value):
@@ -570,6 +620,7 @@ def _log_failure(parameter, exc):
 
   log.debug('proc call failed (%s): %s' % (parameter, exc))
 
+
 # TODO: drop with stem 2.x
 # We renamed our methods to drop a redundant 'get_*' prefix, so alias the old
 # names for backward compatability.
