@@ -2,7 +2,6 @@
 Integration tests for the stem.control.Controller class.
 """
 
-import hashlib
 import os
 import shutil
 import socket
@@ -19,7 +18,9 @@ import stem.response.protocolinfo
 import stem.socket
 import stem.util.str_tools
 import stem.version
+import test
 import test.network
+import test.require
 import test.runner
 
 from stem import Flag, Signal
@@ -27,32 +28,15 @@ from stem.control import EventType, Listener, State
 from stem.exit_policy import ExitPolicy
 from stem.version import Requirement
 
-from test.util import register_new_capability
-
-from test.runner import (
-  require_controller,
-  require_version,
-  require_online,
-  only_run_once,
-)
-
 # Router status entry for a relay with a nickname other than 'Unnamed'. This is
 # used for a few tests that need to look up a relay.
 
 TEST_ROUTER_STATUS_ENTRY = None
 
 
-def random_fingerprint():
-  """
-  Provides a random 40 character hex string.
-  """
-
-  return hashlib.sha1(os.urandom(20)).hexdigest().upper()
-
-
 class TestController(unittest.TestCase):
-  @only_run_once
-  @require_controller
+  @test.require.only_run_once
+  @test.require.controller
   def test_missing_capabilities(self):
     """
     Check to see if tor supports any events, signals, or features that we
@@ -62,17 +46,17 @@ class TestController(unittest.TestCase):
     with test.runner.get_runner().get_tor_controller() as controller:
       for event in controller.get_info('events/names').split():
         if event not in EventType:
-          register_new_capability('Event', event)
+          test.register_new_capability('Event', event)
 
       for signal in controller.get_info('signal/names').split():
         if signal not in Signal:
-          register_new_capability('Signal', signal)
+          test.register_new_capability('Signal', signal)
 
       # new features should simply be added to enable_feature()'s docs
 
       for feature in controller.get_info('features/names').split():
         if feature not in ('EXTENDED_EVENTS', 'VERBOSE_NAMES'):
-          register_new_capability('Feature', feature)
+          test.register_new_capability('Feature', feature)
 
   def test_from_port(self):
     """
@@ -96,8 +80,8 @@ class TestController(unittest.TestCase):
     else:
       self.assertRaises(stem.SocketError, stem.control.Controller.from_socket_file, test.runner.CONTROL_SOCKET_PATH)
 
-  @require_controller
-  @require_version(Requirement.EVENT_SIGNAL)
+  @test.require.controller
+  @test.require.version(Requirement.EVENT_SIGNAL)
   def test_reset_notification(self):
     """
     Checks that a notificiation listener is... well, notified of SIGHUPs.
@@ -119,7 +103,7 @@ class TestController(unittest.TestCase):
         if (time.time() - before) > 2:
           self.fail("We've waited a couple seconds for SIGHUP to generate an event, but it didn't come")
 
-        time.sleep(0.1)
+        time.sleep(0.001)
 
       after = time.time()
 
@@ -133,7 +117,7 @@ class TestController(unittest.TestCase):
 
       controller.reset_conf('__OwningControllerProcess')
 
-  @require_controller
+  @test.require.controller
   def test_event_handling(self):
     """
     Add a couple listeners for various events and make sure that they receive
@@ -158,7 +142,7 @@ class TestController(unittest.TestCase):
       controller.add_event_listener(listener2, EventType.CONF_CHANGED, EventType.DEBUG)
 
       # The NodeFamily is a harmless option we can toggle
-      controller.set_conf('NodeFamily', random_fingerprint())
+      controller.set_conf('NodeFamily', 'FD4CC275C5AA4D27A487C6CA29097900F85E2C33')
 
       # Wait for the event. Assert that we get it within 10 seconds
       event_notice1.wait(10)
@@ -175,7 +159,7 @@ class TestController(unittest.TestCase):
 
       buffer2_size = len(event_buffer2)
 
-      controller.set_conf('NodeFamily', random_fingerprint())
+      controller.set_conf('NodeFamily', 'A82F7EFDB570F6BC801805D0328D30A99403C401')
       event_notice1.wait(10)
       self.assertEqual(len(event_buffer1), 2)
       event_notice1.clear()
@@ -191,7 +175,7 @@ class TestController(unittest.TestCase):
 
       controller.reset_conf('NodeFamily')
 
-  @require_controller
+  @test.require.controller
   def test_reattaching_listeners(self):
     """
     Checks that event listeners are re-attached when a controller disconnects
@@ -208,68 +192,31 @@ class TestController(unittest.TestCase):
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller() as controller:
-      controller.add_event_listener(listener, EventType.BW)
+      controller.add_event_listener(listener, EventType.CONF_CHANGED)
 
-      # Get a BW event or two. These should be emitted each second but under
-      # heavy system load that's not always the case.
+      # trigger an event
 
+      controller.set_conf('NodeFamily', 'FD4CC275C5AA4D27A487C6CA29097900F85E2C33')
       event_notice.wait(4)
       self.assertTrue(len(event_buffer) >= 1)
 
-      # disconnect and check that we stop getting events
+      # disconnect, then reconnect and check that we get events again
 
       controller.close()
       event_notice.clear()
       event_buffer = []
 
-      event_notice.wait(2)
-      self.assertTrue(len(event_buffer) == 0)
-
-      # reconnect and check that we get events again
-
       controller.connect()
       controller.authenticate(password = test.runner.CONTROL_PASSWORD)
+      self.assertTrue(len(event_buffer) == 0)
+      controller.set_conf('NodeFamily', 'A82F7EFDB570F6BC801805D0328D30A99403C401')
 
       event_notice.wait(4)
       self.assertTrue(len(event_buffer) >= 1)
 
-      # disconnect
+      controller.reset_conf('NodeFamily')
 
-      controller.close()
-      event_notice.clear()
-      event_buffer = []
-
-      # reconnect and check that we get events again
-
-      controller.connect()
-      stem.connection.authenticate(controller, password = test.runner.CONTROL_PASSWORD)
-
-      event_notice.wait(4)
-      self.assertTrue(len(event_buffer) >= 1)
-
-      # disconnect
-
-      controller.close()
-      event_notice.clear()
-      event_buffer = []
-
-      # Reconnect and check that we get events again. This is being done by
-      # calling AUTHENTICATE manually so skipping cookie auth.
-
-      tor_options = test.runner.get_runner().get_options()
-
-      if test.runner.Torrc.COOKIE not in tor_options:
-        controller.connect()
-
-        if test.runner.Torrc.PASSWORD in tor_options:
-          controller.msg('AUTHENTICATE "%s"' % test.runner.CONTROL_PASSWORD)
-        else:
-          controller.msg('AUTHENTICATE')
-
-        event_notice.wait(4)
-        self.assertTrue(len(event_buffer) >= 1)
-
-  @require_controller
+  @test.require.controller
   def test_getinfo(self):
     """
     Exercises GETINFO with valid and invalid queries.
@@ -307,7 +254,7 @@ class TestController(unittest.TestCase):
       self.assertEqual({}, controller.get_info([]))
       self.assertEqual({}, controller.get_info([], {}))
 
-  @require_controller
+  @test.require.controller
   def test_get_version(self):
     """
     Test that the convenient method get_version() works.
@@ -318,9 +265,9 @@ class TestController(unittest.TestCase):
     with runner.get_tor_controller() as controller:
       version = controller.get_version()
       self.assertTrue(isinstance(version, stem.version.Version))
-      self.assertEqual(version, runner.get_tor_version())
+      self.assertEqual(version, test.tor_version())
 
-  @require_controller
+  @test.require.controller
   def test_get_exit_policy(self):
     """
     Sanity test for get_exit_policy(). We have the default policy (no
@@ -363,7 +310,7 @@ class TestController(unittest.TestCase):
       policy_str = policy_str[:public_addr_start] + policy_str[public_addr_end:]
       self.assertEqual(str(expected), policy_str)
 
-  @require_controller
+  @test.require.controller
   def test_authenticate(self):
     """
     Test that the convenient method authenticate() works.
@@ -375,7 +322,7 @@ class TestController(unittest.TestCase):
       controller.authenticate(test.runner.CONTROL_PASSWORD)
       test.runner.exercise_controller(self, controller)
 
-  @require_controller
+  @test.require.controller
   def test_protocolinfo(self):
     """
     Test that the convenient method protocolinfo() works.
@@ -389,13 +336,12 @@ class TestController(unittest.TestCase):
 
       # Doing a sanity test on the ProtocolInfoResponse instance returned.
       tor_options = runner.get_options()
-      tor_version = runner.get_tor_version()
       auth_methods = []
 
       if test.runner.Torrc.COOKIE in tor_options:
         auth_methods.append(stem.response.protocolinfo.AuthMethod.COOKIE)
 
-        if tor_version >= stem.version.Requirement.AUTH_SAFECOOKIE:
+        if test.tor_version() >= stem.version.Requirement.AUTH_SAFECOOKIE:
           auth_methods.append(stem.response.protocolinfo.AuthMethod.SAFECOOKIE)
 
       if test.runner.Torrc.PASSWORD in tor_options:
@@ -406,7 +352,7 @@ class TestController(unittest.TestCase):
 
       self.assertEqual(tuple(auth_methods), protocolinfo.auth_methods)
 
-  @require_controller
+  @test.require.controller
   def test_getconf(self):
     """
     Exercises GETCONF with valid and invalid queries.
@@ -469,7 +415,7 @@ class TestController(unittest.TestCase):
       self.assertEqual({}, controller.get_conf_map('', 'la-di-dah'))
       self.assertEqual({}, controller.get_conf_map([], 'la-di-dah'))
 
-  @require_controller
+  @test.require.controller
   def test_is_set(self):
     """
     Exercises our is_set() method.
@@ -481,10 +427,10 @@ class TestController(unittest.TestCase):
       custom_options = controller._get_custom_options()
       self.assertTrue('ControlPort' in custom_options or 'ControlSocket' in custom_options)
       self.assertEqual('1', custom_options['DownloadExtraInfo'])
-      self.assertEqual('127.0.0.1:1112', custom_options['SocksListenAddress'])
+      self.assertEqual('1112', custom_options['SocksPort'])
 
       self.assertTrue(controller.is_set('DownloadExtraInfo'))
-      self.assertTrue(controller.is_set('SocksListenAddress'))
+      self.assertTrue(controller.is_set('SocksPort'))
       self.assertFalse(controller.is_set('CellStatistics'))
       self.assertFalse(controller.is_set('ConnLimit'))
 
@@ -496,7 +442,7 @@ class TestController(unittest.TestCase):
       controller.reset_conf('ConnLimit')
       self.assertFalse(controller.is_set('ConnLimit'))
 
-  @require_controller
+  @test.require.controller
   def test_hidden_services_conf(self):
     """
     Exercises the hidden service family of methods (get_hidden_service_conf,
@@ -510,7 +456,6 @@ class TestController(unittest.TestCase):
     service2_path = os.path.join(test_dir, 'test_hidden_service2')
     service3_path = os.path.join(test_dir, 'test_hidden_service3')
     service4_path = os.path.join(test_dir, 'test_hidden_service4')
-    empty_service_path = os.path.join(test_dir, 'test_hidden_service_empty')
 
     with runner.get_tor_controller() as controller:
       try:
@@ -541,9 +486,6 @@ class TestController(unittest.TestCase):
               (8032, '127.0.0.1', 8032),
             ]
           },
-          empty_service_path: {
-            'HiddenServicePort': []
-          }
         }
 
         controller.set_hidden_service_conf(initialconf)
@@ -565,18 +507,18 @@ class TestController(unittest.TestCase):
         self.assertTrue(hs_address1.endswith('.onion'))
 
         conf = controller.get_hidden_service_conf()
-        self.assertEqual(4, len(conf))
+        self.assertEqual(3, len(conf))
         self.assertEqual(2, len(conf[hs_path]['HiddenServicePort']))
 
         # remove a hidden service, the service dir should still be there
 
         controller.remove_hidden_service(hs_path, 8888)
-        self.assertEqual(4, len(controller.get_hidden_service_conf()))
+        self.assertEqual(3, len(controller.get_hidden_service_conf()))
 
         # remove a service completely, it should now be gone
 
         controller.remove_hidden_service(hs_path, 8989)
-        self.assertEqual(3, len(controller.get_hidden_service_conf()))
+        self.assertEqual(2, len(controller.get_hidden_service_conf()))
 
         # add a new service, this time with client authentication
 
@@ -589,13 +531,13 @@ class TestController(unittest.TestCase):
         self.assertTrue(hs_attributes.hostname_for_client['c2'].endswith('.onion'))
 
         conf = controller.get_hidden_service_conf()
-        self.assertEqual(4, len(conf))
+        self.assertEqual(3, len(conf))
         self.assertEqual(1, len(conf[hs_path]['HiddenServicePort']))
 
         # remove a hidden service
 
         controller.remove_hidden_service(hs_path, 8888)
-        self.assertEqual(3, len(controller.get_hidden_service_conf()))
+        self.assertEqual(2, len(controller.get_hidden_service_conf()))
       finally:
         controller.set_hidden_service_conf({})  # drop hidden services created during the test
 
@@ -607,8 +549,8 @@ class TestController(unittest.TestCase):
           except:
             pass
 
-  @require_controller
-  @require_version(Requirement.ADD_ONION)
+  @test.require.controller
+  @test.require.version(Requirement.ADD_ONION)
   def test_without_ephemeral_hidden_services(self):
     """
     Exercises ephemeral hidden service methods when none are present.
@@ -619,8 +561,8 @@ class TestController(unittest.TestCase):
       self.assertEqual([], controller.list_ephemeral_hidden_services(detached = True))
       self.assertEqual(False, controller.remove_ephemeral_hidden_service('gfzprpioee3hoppz'))
 
-  @require_controller
-  @require_version(Requirement.ADD_ONION)
+  @test.require.controller
+  @test.require.version(Requirement.ADD_ONION)
   def test_with_ephemeral_hidden_services(self):
     """
     Exercises creating ephemeral hidden services and methods when they're
@@ -630,13 +572,11 @@ class TestController(unittest.TestCase):
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller() as controller:
+      # try creating a service with an invalid ports
+
       for ports in (4567890, [4567, 4567890], {4567: '-:4567'}):
-        try:
-          # try creating a service with an invalid port
-          response = controller.create_ephemeral_hidden_service(ports)
-          self.fail("we should've raised a stem.ProtocolError")
-        except stem.ProtocolError as exc:
-          self.assertEqual("ADD_ONION response didn't have an OK status: Invalid VIRTPORT/TARGET", str(exc))
+        exc_msg = "ADD_ONION response didn't have an OK status: Invalid VIRTPORT/TARGET"
+        self.assertRaisesRegexp(stem.ProtocolError, exc_msg, controller.create_ephemeral_hidden_service, ports)
 
       response = controller.create_ephemeral_hidden_service(4567)
       self.assertEqual([response.service_id], controller.list_ephemeral_hidden_services())
@@ -672,8 +612,8 @@ class TestController(unittest.TestCase):
         self.assertEqual(2, len(controller.list_ephemeral_hidden_services()))
         self.assertEqual(0, len(second_controller.list_ephemeral_hidden_services()))
 
-  @require_controller
-  @require_version(Requirement.ADD_ONION_BASIC_AUTH)
+  @test.require.controller
+  @test.require.version(Requirement.ADD_ONION_BASIC_AUTH)
   def test_with_ephemeral_hidden_services_basic_auth(self):
     """
     Exercises creating ephemeral hidden services that uses basic authentication.
@@ -692,9 +632,9 @@ class TestController(unittest.TestCase):
       self.assertEqual(True, controller.remove_ephemeral_hidden_service(response.service_id))
       self.assertEqual([], controller.list_ephemeral_hidden_services())
 
-  @require_controller
-  @require_version(Requirement.ADD_ONION_BASIC_AUTH)
-  def test_with_ephemeral_hidden_services_basic_auth_without_credentials(self):
+  @test.require.controller
+  @test.require.version(Requirement.ADD_ONION_BASIC_AUTH)
+  def test_with_ephemeral_hidden_services_basic_auth_no_credentials(self):
     """
     Exercises creating ephemeral hidden services when attempting to use basic
     auth but not including any credentials.
@@ -703,14 +643,11 @@ class TestController(unittest.TestCase):
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller() as controller:
-      try:
-        controller.create_ephemeral_hidden_service(4567, basic_auth = {})
-        self.fail('ADD_ONION should fail when using basic auth without any clients')
-      except stem.ProtocolError as exc:
-        self.assertEqual("ADD_ONION response didn't have an OK status: No auth clients specified", str(exc))
+      exc_msg = "ADD_ONION response didn't have an OK status: No auth clients specified"
+      self.assertRaisesRegexp(stem.ProtocolError, exc_msg, controller.create_ephemeral_hidden_service, 4567, basic_auth = {})
 
-  @require_controller
-  @require_version(Requirement.ADD_ONION)
+  @test.require.controller
+  @test.require.version(Requirement.ADD_ONION)
   def test_with_detached_ephemeral_hidden_services(self):
     """
     Exercises creating detached ephemeral hidden services and methods when
@@ -745,7 +682,20 @@ class TestController(unittest.TestCase):
       self.assertEqual([response.service_id], controller.list_ephemeral_hidden_services(detached = True))
       controller.remove_ephemeral_hidden_service(response.service_id)
 
-  @require_controller
+  @test.require.controller
+  @test.require.version(Requirement.ADD_ONION)
+  def test_rejecting_unanonymous_hidden_services_creation(self):
+    """
+    Attempt to create a non-anonymous hidden service despite not setting
+    HiddenServiceSingleHopMode and HiddenServiceNonAnonymousMode.
+    """
+
+    runner = test.runner.get_runner()
+
+    with runner.get_tor_controller() as controller:
+      self.assertEqual('Tor is in anonymous hidden service mode', str(controller.msg('ADD_ONION NEW:BEST Flags=NonAnonymous Port=4567')))
+
+  @test.require.controller
   def test_set_conf(self):
     """
     Exercises set_conf(), reset_conf(), and set_options() methods with valid
@@ -818,8 +768,18 @@ class TestController(unittest.TestCase):
 
         shutil.rmtree(tmpdir)
 
-  @require_controller
-  @require_version(Requirement.LOADCONF)
+  @test.require.controller
+  def test_set_conf_when_immutable(self):
+    """
+    Issue a SETCONF for tor options that cannot be changed while running.
+    """
+
+    with test.runner.get_runner().get_tor_controller() as controller:
+      self.assertRaisesRegexp(stem.InvalidArguments, "DisableAllSwap cannot be changed while tor's running", controller.set_conf, 'DisableAllSwap', '1')
+      self.assertRaisesRegexp(stem.InvalidArguments, "DisableAllSwap, User cannot be changed while tor's running", controller.set_options, {'User': 'atagar', 'DisableAllSwap': '1'})
+
+  @test.require.controller
+  @test.require.version(Requirement.LOADCONF)
   def test_loadconf(self):
     """
     Exercises Controller.load_conf with valid and invalid requests.
@@ -856,7 +816,7 @@ class TestController(unittest.TestCase):
         controller.load_conf(oldconf)
         controller.reset_conf('__OwningControllerProcess')
 
-  @require_controller
+  @test.require.controller
   def test_saveconf(self):
     runner = test.runner.get_runner()
 
@@ -876,7 +836,7 @@ class TestController(unittest.TestCase):
         controller.save_conf()
         controller.reset_conf('__OwningControllerProcess')
 
-  @require_controller
+  @test.require.controller
   def test_get_ports(self):
     """
     Test Controller.get_ports against a running tor instance.
@@ -897,7 +857,7 @@ class TestController(unittest.TestCase):
       else:
         self.assertEqual([], controller.get_ports(Listener.CONTROL))
 
-  @require_controller
+  @test.require.controller
   def test_get_listeners(self):
     """
     Test Controller.get_listeners against a running tor instance.
@@ -918,7 +878,7 @@ class TestController(unittest.TestCase):
       else:
         self.assertEqual([], controller.get_listeners(Listener.CONTROL))
 
-  @require_controller
+  @test.require.controller
   def test_get_socks_listeners(self):
     """
     Test Controller.get_socks_listeners against a running tor instance.
@@ -927,9 +887,9 @@ class TestController(unittest.TestCase):
     with test.runner.get_runner().get_tor_controller() as controller:
       self.assertEqual([('127.0.0.1', 1112)], controller.get_socks_listeners())
 
-  @require_controller
-  @require_online
-  @require_version(stem.version.Version('0.1.2.2-alpha'))
+  @test.require.controller
+  @test.require.online
+  @test.require.version(stem.version.Version('0.1.2.2-alpha'))
   def test_enable_feature(self):
     """
     Test Controller.enable_feature with valid and invalid inputs.
@@ -950,7 +910,7 @@ class TestController(unittest.TestCase):
       else:
         self.fail()
 
-  @require_controller
+  @test.require.controller
   def test_signal(self):
     """
     Test controller.signal with valid and invalid signals.
@@ -963,7 +923,7 @@ class TestController(unittest.TestCase):
       # invalid signals
       self.assertRaises(stem.InvalidArguments, controller.signal, 'FOOBAR')
 
-  @require_controller
+  @test.require.controller
   def test_newnym_availability(self):
     """
     Test the is_newnym_available and get_newnym_wait methods.
@@ -978,9 +938,9 @@ class TestController(unittest.TestCase):
       self.assertEqual(False, controller.is_newnym_available())
       self.assertTrue(controller.get_newnym_wait() > 9.0)
 
-  @require_controller
-  @require_online
-  @require_version(Requirement.EXTENDCIRCUIT_PATH_OPTIONAL)
+  @test.require.controller
+  @test.require.online
+  @test.require.version(Requirement.EXTENDCIRCUIT_PATH_OPTIONAL)
   def test_extendcircuit(self):
     with test.runner.get_runner().get_tor_controller() as controller:
       circuit_id = controller.extend_circuit('0')
@@ -994,9 +954,9 @@ class TestController(unittest.TestCase):
       self.assertRaises(stem.InvalidRequest, controller.extend_circuit, '0', 'thisroutershouldntexistbecausestemexists!@##$%#')
       self.assertRaises(stem.InvalidRequest, controller.extend_circuit, '0', 'thisroutershouldntexistbecausestemexists!@##$%#', 'foo')
 
-  @require_controller
-  @require_online
-  @require_version(Requirement.EXTENDCIRCUIT_PATH_OPTIONAL)
+  @test.require.controller
+  @test.require.online
+  @test.require.version(Requirement.EXTENDCIRCUIT_PATH_OPTIONAL)
   def test_repurpose_circuit(self):
     """
     Tests Controller.repurpose_circuit with valid and invalid input.
@@ -1017,9 +977,9 @@ class TestController(unittest.TestCase):
       self.assertRaises(stem.InvalidRequest, controller.repurpose_circuit, 'f934h9f3h4', 'fooo')
       self.assertRaises(stem.InvalidRequest, controller.repurpose_circuit, '4', 'fooo')
 
-  @require_controller
-  @require_online
-  @require_version(Requirement.EXTENDCIRCUIT_PATH_OPTIONAL)
+  @test.require.controller
+  @test.require.online
+  @test.require.version(Requirement.EXTENDCIRCUIT_PATH_OPTIONAL)
   def test_close_circuit(self):
     """
     Tests Controller.close_circuit with valid and invalid input.
@@ -1044,8 +1004,8 @@ class TestController(unittest.TestCase):
       self.assertRaises(stem.InvalidArguments, controller.close_circuit, circuit_id + '1024')
       self.assertRaises(stem.InvalidRequest, controller.close_circuit, '')
 
-  @require_controller
-  @require_online
+  @test.require.controller
+  @test.require.online
   def test_get_streams(self):
     """
     Tests Controller.get_streams().
@@ -1069,8 +1029,8 @@ class TestController(unittest.TestCase):
 
     self.assertTrue('%s:%s' % (host, port) in [stream.target for stream in streams])
 
-  @require_controller
-  @require_online
+  @test.require.controller
+  @test.require.online
   def test_close_stream(self):
     """
     Tests Controller.close_stream with valid and invalid input.
@@ -1108,8 +1068,8 @@ class TestController(unittest.TestCase):
 
       self.assertRaises(stem.InvalidArguments, controller.close_stream, 'blarg')
 
-  @require_controller
-  @require_online
+  @test.require.controller
+  @test.require.online
   def test_mapaddress(self):
     runner = test.runner.get_runner()
 
@@ -1125,7 +1085,7 @@ class TestController(unittest.TestCase):
         try:
           s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
           s.settimeout(30)
-          s.connect(('127.0.0.1', int(controller.get_conf('SocksListenAddress').rsplit(':', 1)[1])))
+          s.connect(('127.0.0.1', int(controller.get_conf('SocksPort'))))
           test.network.negotiate_socks(s, '1.2.1.2', 80)
           s.sendall(stem.util.str_tools._to_bytes(test.network.ip_request))  # make the http request for the ip address
           response = s.recv(1000)
@@ -1146,9 +1106,9 @@ class TestController(unittest.TestCase):
       ip_addr = response[response.find(b'\r\n\r\n'):].strip()
       self.assertTrue(stem.util.connection.is_valid_ipv4_address(stem.util.str_tools._to_unicode(ip_addr)), "'%s' isn't an address" % ip_addr)
 
-  @require_controller
-  @require_online
-  @require_version(Requirement.MICRODESCRIPTOR_IS_DEFAULT)
+  @test.require.controller
+  @test.require.online
+  @test.require.version(Requirement.MICRODESCRIPTOR_IS_DEFAULT)
   def test_get_microdescriptor(self):
     """
     Basic checks for get_microdescriptor().
@@ -1171,7 +1131,7 @@ class TestController(unittest.TestCase):
 
       self.assertEqual(md_by_fingerprint, md_by_nickname)
 
-  @require_controller
+  @test.require.controller
   def test_get_microdescriptors(self):
     """
     Fetches a few descriptors via the get_microdescriptors() method.
@@ -1180,7 +1140,7 @@ class TestController(unittest.TestCase):
     runner = test.runner.get_runner()
 
     if not os.path.exists(runner.get_test_dir('cached-descriptors')):
-      test.runner.skip(self, '(no cached microdescriptors)')
+      self.skipTest('(no cached microdescriptors)')
       return
 
     with runner.get_tor_controller() as controller:
@@ -1193,7 +1153,7 @@ class TestController(unittest.TestCase):
         if count > 10:
           break
 
-  @require_controller
+  @test.require.controller
   def test_get_server_descriptor(self):
     """
     Basic checks for get_server_descriptor().
@@ -1201,8 +1161,8 @@ class TestController(unittest.TestCase):
 
     runner = test.runner.get_runner()
 
-    if runner.get_tor_version() >= Requirement.MICRODESCRIPTOR_IS_DEFAULT:
-      test.runner.skip(self, '(requires server descriptors)')
+    if test.tor_version() >= Requirement.MICRODESCRIPTOR_IS_DEFAULT:
+      self.skipTest('(requires server descriptors)')
       return
 
     with runner.get_tor_controller() as controller:
@@ -1223,7 +1183,8 @@ class TestController(unittest.TestCase):
 
       self.assertEqual(desc_by_fingerprint, desc_by_nickname)
 
-  @require_controller
+  @test.require.controller
+  @test.require.online
   def test_get_server_descriptors(self):
     """
     Fetches a few descriptors via the get_server_descriptors() method.
@@ -1231,8 +1192,8 @@ class TestController(unittest.TestCase):
 
     runner = test.runner.get_runner()
 
-    if runner.get_tor_version() >= Requirement.MICRODESCRIPTOR_IS_DEFAULT:
-      test.runner.skip(self, '(requires server descriptors)')
+    if test.tor_version() >= Requirement.MICRODESCRIPTOR_IS_DEFAULT:
+      self.skipTest('(requires server descriptors)')
       return
 
     with runner.get_tor_controller() as controller:
@@ -1247,11 +1208,12 @@ class TestController(unittest.TestCase):
         # while to do so).
 
         count += 1
+
         if count > 10:
           break
 
-  @require_controller
-  @require_online
+  @test.require.controller
+  @test.require.online
   def test_get_network_status(self):
     """
     Basic checks for get_network_status().
@@ -1274,8 +1236,8 @@ class TestController(unittest.TestCase):
 
       self.assertEqual(desc_by_fingerprint, desc_by_nickname)
 
-  @require_controller
-  @require_online
+  @test.require.controller
+  @test.require.online
   def test_get_network_statuses(self):
     """
     Fetches a few descriptors via the get_network_statuses() method.
@@ -1291,15 +1253,15 @@ class TestController(unittest.TestCase):
         self.assertTrue(desc.nickname is not None)
 
         for line in desc.get_unrecognized_lines():
-          register_new_capability('Consensus Line', line)
+          test.register_new_capability('Consensus Line', line)
 
         count += 1
         if count > 10:
           break
 
-  @require_controller
-  @require_online
-  @require_version(Requirement.HSFETCH)
+  @test.require.controller
+  @test.require.online
+  @test.require.version(Requirement.HSFETCH)
   def test_get_hidden_service_descriptor(self):
     """
     Fetches a few descriptors via the get_hidden_service_descriptor() method.
@@ -1315,20 +1277,17 @@ class TestController(unittest.TestCase):
 
       # try to fetch something that doesn't exist
 
-      try:
-        desc = controller.get_hidden_service_descriptor('m4cfuk6qp4lpu2g3')
-        self.fail("Didn't expect m4cfuk6qp4lpu2g3.onion to exist, but provided: %s" % desc)
-      except stem.DescriptorUnavailable as exc:
-        self.assertEqual('No running hidden service at m4cfuk6qp4lpu2g3.onion', str(exc))
+      exc_msg = 'No running hidden service at m4cfuk6qp4lpu2g3.onion'
+      self.assertRaisesRegexp(stem.DescriptorUnavailable, exc_msg, controller.get_hidden_service_descriptor, 'm4cfuk6qp4lpu2g3')
 
       # ... but shouldn't fail if we have a default argument or aren't awaiting the descriptor
 
       self.assertEqual('pop goes the weasel', controller.get_hidden_service_descriptor('m4cfuk6qp4lpu2g5', 'pop goes the weasel'))
       self.assertEqual(None, controller.get_hidden_service_descriptor('m4cfuk6qp4lpu2g5', await_result = False))
 
-  @require_controller
-  @require_online
-  @require_version(Requirement.EXTENDCIRCUIT_PATH_OPTIONAL)
+  @test.require.controller
+  @test.require.online
+  @test.require.version(Requirement.EXTENDCIRCUIT_PATH_OPTIONAL)
   def test_attachstream(self):
     host = socket.gethostbyname('www.torproject.org')
     port = 80
@@ -1367,9 +1326,9 @@ class TestController(unittest.TestCase):
 
     self.assertEqual(our_stream.circ_id, circuit_id)
 
-  @require_controller
-  @require_online
-  @require_version(Requirement.EXTENDCIRCUIT_PATH_OPTIONAL)
+  @test.require.controller
+  @test.require.online
+  @test.require.version(Requirement.EXTENDCIRCUIT_PATH_OPTIONAL)
   def test_get_circuits(self):
     """
     Fetches circuits via the get_circuits() method.
@@ -1380,7 +1339,8 @@ class TestController(unittest.TestCase):
       circuits = controller.get_circuits()
       self.assertTrue(new_circ in [circ.id for circ in circuits])
 
-  @require_controller
+  @test.require.online
+  @test.require.controller
   def test_transition_to_relay(self):
     """
     Transitions Tor to turn into a relay, then back to a client. This helps to
@@ -1415,7 +1375,7 @@ class TestController(unittest.TestCase):
 
       if TEST_ROUTER_STATUS_ENTRY is None:
         # this is only likely to occure if we can't get descriptors
-        test.runner.skip(self, '(no named relays)')
+        self.skipTest('(no named relays)')
         return
 
     return TEST_ROUTER_STATUS_ENTRY
