@@ -10,8 +10,11 @@ import unittest
 
 import stem.util.proc
 import stem.util.system
+import test.require
 import test.runner
 
+from stem.util.system import State, DaemonTask
+
 try:
   # added in python 3.3
   from unittest.mock import Mock, patch
@@ -36,6 +39,23 @@ def filter_system_call(prefixes):
   return _filter_system_call
 
 
+def _is_single_tor_running():
+  if stem.util.system.is_windows():
+    return True  # TODO: not sure how to check for this on windows
+  elif not stem.util.system.is_bsd():
+    tor_cmd = test.runner.get_runner().get_tor_command(True)
+    pgrep_results = stem.util.system.call(stem.util.system.GET_PID_BY_NAME_PGREP % tor_cmd)
+    return len(pgrep_results) == 1
+  else:
+    ps_results = stem.util.system.call(stem.util.system.GET_PID_BY_NAME_PS_BSD)
+    results = [r for r in ps_results if r.endswith(' tor')]
+    return len(results) == 1
+
+
+def _is_linux():
+  return not stem.util.system.is_bsd() and not stem.util.system.is_windows()
+
+
 def _has_port():
   """
   True if our test runner has a control port, False otherwise.
@@ -44,7 +64,48 @@ def _has_port():
   return test.runner.Torrc.PORT in test.runner.get_runner().get_options()
 
 
+require_single_tor_instance = test.require.needs(_is_single_tor_running, 'multiple tor instances')
+require_control_port = test.require.needs(_has_port, 'test instance has no port')
+require_linux = test.require.needs(_is_linux, 'linux only')
+require_bsd = test.require.needs(stem.util.system.is_bsd, 'bsd only')
+require_path = test.require.needs(lambda: 'PATH' in os.environ, 'requires PATH')
+
+
 class TestSystem(unittest.TestCase):
+  def test_daemon_task_when_successful(self):
+    """
+    Checks a simple, successfully DaemonTask that simply echos a value.
+    """
+
+    task = DaemonTask(lambda arg: arg, ('hello world',))
+
+    self.assertEqual(None, task.result)
+    self.assertEqual(State.PENDING, task.status)
+
+    task.run()
+    self.assertEqual('hello world', task.join())
+    self.assertEqual(State.DONE, task.status)
+    self.assertTrue(0 < task.runtime < 1.0)
+
+  def test_daemon_task_on_failure(self):
+    """
+    Checks DaemonTask when an exception is raised.
+    """
+
+    def _test_task(arg):
+      raise RuntimeError(arg)
+
+    task = DaemonTask(_test_task, ('hello world',))
+
+    self.assertEqual(None, task.result)
+    self.assertEqual(State.PENDING, task.status)
+
+    task.run()
+    self.assertRaisesRegexp(RuntimeError, 'hello world', task.join)
+    self.assertEqual(State.FAILED, task.status)
+    self.assertTrue(0 < task.runtime < 1.0)
+
+  @require_path
   def test_is_available(self):
     """
     Checks the stem.util.system.is_available function.
@@ -61,14 +122,19 @@ class TestSystem(unittest.TestCase):
 
     self.assertFalse(stem.util.system.is_available('blarg_and_stuff'))
 
-  def test_is_running(self):
+  def test_is_running_by_pid(self):
     """
-    Checks the stem.util.system.is_running function.
+    Checks the stem.util.system.is_running function with a pid.
     """
 
-    if not stem.util.system.is_available('ps'):
-      test.runner.skip(self, '(ps unavailable)')
-      return
+    self.assertTrue(stem.util.system.is_running(test.runner.get_runner().get_pid()))
+    self.assertFalse(stem.util.system.is_running(528955))
+
+  @test.require.command('ps')
+  def test_is_running_by_name(self):
+    """
+    Checks the stem.util.system.is_running function with a process name.
+    """
 
     # Check to see if the command we started tor with is running. The process
     # might be running under another name so need to check for 'tor.real' too
@@ -78,35 +144,27 @@ class TestSystem(unittest.TestCase):
     self.assertTrue(stem.util.system.is_running(tor_cmd) or stem.util.system.is_running('tor.real'))
     self.assertFalse(stem.util.system.is_running('blarg_and_stuff'))
 
+  @require_path
+  @require_single_tor_instance
   def test_pid_by_name(self):
     """
     Checks general usage of the stem.util.system.pid_by_name function. This
     will fail if there's other tor instances running.
     """
 
-    if self._is_extra_tor_running():
-      test.runner.skip(self, '(multiple tor instances)')
-      return
-
     tor_pid = test.runner.get_runner().get_pid()
     tor_cmd = test.runner.get_runner().get_tor_command(True)
     self.assertEqual(tor_pid, stem.util.system.pid_by_name(tor_cmd))
     self.assertEqual(None, stem.util.system.pid_by_name('blarg_and_stuff'))
 
+  @require_single_tor_instance
+  @test.require.command('pgrep')
   def test_pid_by_name_pgrep(self):
     """
     Tests the pid_by_name function with a pgrep response.
     """
 
-    if self._is_extra_tor_running():
-      test.runner.skip(self, '(multiple tor instances)')
-      return
-    elif not stem.util.system.is_available('pgrep'):
-      test.runner.skip(self, '(pgrep unavailable)')
-      return
-
     pgrep_prefix = stem.util.system.GET_PID_BY_NAME_PGREP % ''
-
     call_replacement = filter_system_call([pgrep_prefix])
 
     with patch('stem.util.system.call') as call_mock:
@@ -116,20 +174,14 @@ class TestSystem(unittest.TestCase):
       tor_cmd = test.runner.get_runner().get_tor_command(True)
       self.assertEqual(tor_pid, stem.util.system.pid_by_name(tor_cmd))
 
+  @require_single_tor_instance
+  @test.require.command('pidof')
   def test_pid_by_name_pidof(self):
     """
     Tests the pid_by_name function with a pidof response.
     """
 
-    if self._is_extra_tor_running():
-      test.runner.skip(self, '(multiple tor instances)')
-      return
-    elif not stem.util.system.is_available('pidof'):
-      test.runner.skip(self, '(pidof unavailable)')
-      return
-
     pidof_prefix = stem.util.system.GET_PID_BY_NAME_PIDOF % ''
-
     call_replacement = filter_system_call([pidof_prefix])
 
     with patch('stem.util.system.call') as call_mock:
@@ -139,23 +191,15 @@ class TestSystem(unittest.TestCase):
       tor_cmd = test.runner.get_runner().get_tor_command()
       self.assertEqual(tor_pid, stem.util.system.pid_by_name(tor_cmd))
 
+  @require_linux
+  @require_single_tor_instance
+  @test.require.command('ps')
   def test_pid_by_name_ps_linux(self):
     """
     Tests the pid_by_name function with the linux variant of ps.
     """
 
-    if self._is_extra_tor_running():
-      test.runner.skip(self, '(multiple tor instances)')
-      return
-    elif not stem.util.system.is_available('ps'):
-      test.runner.skip(self, '(ps unavailable)')
-      return
-    elif stem.util.system.is_bsd():
-      test.runner.skip(self, '(linux only)')
-      return
-
     ps_prefix = stem.util.system.GET_PID_BY_NAME_PS_LINUX % ''
-
     call_replacement = filter_system_call([ps_prefix])
 
     with patch('stem.util.system.call') as call_mock:
@@ -165,23 +209,15 @@ class TestSystem(unittest.TestCase):
       tor_cmd = test.runner.get_runner().get_tor_command(True)
       self.assertEqual(tor_pid, stem.util.system.pid_by_name(tor_cmd))
 
+  @require_bsd
+  @require_single_tor_instance
+  @test.require.command('ps')
   def test_pid_by_name_ps_bsd(self):
     """
     Tests the pid_by_name function with the bsd variant of ps.
     """
 
-    if self._is_extra_tor_running():
-      test.runner.skip(self, '(multiple tor instances)')
-      return
-    elif not stem.util.system.is_available('ps'):
-      test.runner.skip(self, '(ps unavailable)')
-      return
-    elif not stem.util.system.is_bsd():
-      test.runner.skip(self, '(bsd only)')
-      return
-
     ps_prefix = stem.util.system.GET_PID_BY_NAME_PS_BSD
-
     call_replacement = filter_system_call([ps_prefix])
 
     with patch('stem.util.system.call') as call_mock:
@@ -191,24 +227,15 @@ class TestSystem(unittest.TestCase):
       tor_cmd = test.runner.get_runner().get_tor_command(True)
       self.assertEqual(tor_pid, stem.util.system.pid_by_name(tor_cmd))
 
+  @require_single_tor_instance
+  @test.require.ptrace
+  @test.require.command('lsof')
   def test_pid_by_name_lsof(self):
     """
     Tests the pid_by_name function with a lsof response.
     """
 
-    runner = test.runner.get_runner()
-    if self._is_extra_tor_running():
-      test.runner.skip(self, '(multiple tor instances)')
-      return
-    elif not stem.util.system.is_available('lsof'):
-      test.runner.skip(self, '(lsof unavailable)')
-      return
-    elif not runner.is_ptraceable():
-      test.runner.skip(self, '(DisableDebuggerAttachment is set)')
-      return
-
     lsof_prefix = stem.util.system.GET_PID_BY_NAME_LSOF % ''
-
     call_replacement = filter_system_call([lsof_prefix])
 
     with patch('stem.util.system.call') as call_mock:
@@ -221,68 +248,51 @@ class TestSystem(unittest.TestCase):
       if len(all_tor_pids) == 1:
         self.assertEqual(our_tor_pid, all_tor_pids[0])
 
+  @require_single_tor_instance
+  @test.require.command('tasklist')
   def test_pid_by_name_tasklist(self):
     """
     Tests the pid_by_name function with a tasklist response.
     """
 
-    if self._is_extra_tor_running():
-      test.runner.skip(self, '(multiple tor instances)')
-      return
-    elif not stem.util.system.is_available('tasklist'):
-      test.runner.skip(self, '(tasklist unavailable)')
-      return
-
     runner = test.runner.get_runner()
     self.assertEqual(runner.get_pid(), stem.util.system.pid_by_name(runner.get_tor_command(True)))
 
+  @require_control_port
+  @test.require.ptrace
   def test_pid_by_port(self):
     """
     Checks general usage of the stem.util.system.pid_by_port function.
     """
-    runner = test.runner.get_runner()
+
     if stem.util.system.is_windows():
-      test.runner.skip(self, '(unavailable on windows)')
-      return
-    elif not _has_port():
-      test.runner.skip(self, '(test instance has no port)')
+      self.skipTest('(unavailable on windows)')
       return
     elif stem.util.system.is_mac() or stem.util.system.is_gentoo():
-      test.runner.skip(self, '(resolvers unavailable)')
-      return
-    elif not runner.is_ptraceable():
-      test.runner.skip(self, '(DisableDebuggerAttachment is set)')
+      self.skipTest('(resolvers unavailable)')
       return
     elif not (stem.util.system.is_available('netstat') or
               stem.util.system.is_available('sockstat') or
               stem.util.system.is_available('lsof')):
-      test.runner.skip(self, '(connection resolvers unavailable)')
+      self.skipTest('(connection resolvers unavailable)')
       return
 
+    runner = test.runner.get_runner()
     tor_pid, tor_port = runner.get_pid(), test.runner.CONTROL_PORT
     self.assertEqual(tor_pid, stem.util.system.pid_by_port(tor_port))
     self.assertEqual(None, stem.util.system.pid_by_port(99999))
 
+  @require_linux
+  @require_control_port
+  @test.require.ptrace
+  @test.require.command('netstat')
   def test_pid_by_port_netstat(self):
     """
     Tests the pid_by_port function with a netstat response.
     """
 
-    runner = test.runner.get_runner()
-    if not _has_port():
-      test.runner.skip(self, '(test instance has no port)')
-      return
-    elif not stem.util.system.is_available('netstat'):
-      test.runner.skip(self, '(netstat unavailable)')
-      return
-    elif stem.util.system.is_bsd() or stem.util.system.is_windows():
-      test.runner.skip(self, '(linux only)')
-      return
-    elif stem.util.system.is_gentoo():
-      test.runner.skip(self, '(unavailable on gentoo)')
-      return
-    elif not runner.is_ptraceable():
-      test.runner.skip(self, '(DisableDebuggerAttachment is set)')
+    if stem.util.system.is_gentoo():
+      self.skipTest('(unavailable on gentoo)')
       return
 
     netstat_prefix = stem.util.system.GET_PID_BY_PORT_NETSTAT
@@ -295,27 +305,16 @@ class TestSystem(unittest.TestCase):
       tor_pid = test.runner.get_runner().get_pid()
       self.assertEqual(tor_pid, stem.util.system.pid_by_port(test.runner.CONTROL_PORT))
 
+  @require_bsd
+  @require_control_port
+  @test.require.ptrace
+  @test.require.command('sockstat')
   def test_pid_by_port_sockstat(self):
     """
     Tests the pid_by_port function with a sockstat response.
     """
 
-    runner = test.runner.get_runner()
-    if not _has_port():
-      test.runner.skip(self, '(test instance has no port)')
-      return
-    elif not stem.util.system.is_available('sockstat'):
-      test.runner.skip(self, '(sockstat unavailable)')
-      return
-    elif not stem.util.system.is_bsd():
-      test.runner.skip(self, '(bsd only)')
-      return
-    elif not runner.is_ptraceable():
-      test.runner.skip(self, '(DisableDebuggerAttachment is set)')
-      return
-
     sockstat_prefix = stem.util.system.GET_PID_BY_PORT_SOCKSTAT % ''
-
     call_replacement = filter_system_call([sockstat_prefix])
 
     with patch('stem.util.system.call') as call_mock:
@@ -324,23 +323,16 @@ class TestSystem(unittest.TestCase):
       tor_pid = test.runner.get_runner().get_pid()
       self.assertEqual(tor_pid, stem.util.system.pid_by_port(test.runner.CONTROL_PORT))
 
+  @require_control_port
+  @test.require.ptrace
+  @test.require.command('lsof')
   def test_pid_by_port_lsof(self):
     """
     Tests the pid_by_port function with a lsof response.
     """
 
-    runner = test.runner.get_runner()
-    if not _has_port():
-      test.runner.skip(self, '(test instance has no port)')
-      return
-    elif not stem.util.system.is_available('lsof'):
-      test.runner.skip(self, '(lsof unavailable)')
-      return
-    elif stem.util.system.is_mac() or stem.util.system.is_gentoo():
-      test.runner.skip(self, '(resolvers unavailable)')
-      return
-    elif not runner.is_ptraceable():
-      test.runner.skip(self, '(DisableDebuggerAttachment is set)')
+    if stem.util.system.is_mac() or stem.util.system.is_gentoo():
+      self.skipTest('(resolvers unavailable)')
       return
 
     lsof_prefix = stem.util.system.GET_PID_BY_PORT_LSOF
@@ -366,6 +358,7 @@ class TestSystem(unittest.TestCase):
     os.rmdir(tmpdir)
     self.assertEqual(None, stem.util.system.pid_by_open_file(tmpdir))
 
+  @require_path
   def test_pids_by_user(self):
     """
     Checks the stem.util.system.pids_by_user function.
@@ -376,38 +369,28 @@ class TestSystem(unittest.TestCase):
     pids = stem.util.system.pids_by_user(getpass.getuser())
     self.assertTrue(os.getpid() in pids)
 
+  @test.require.ptrace
   def test_cwd(self):
     """
     Checks general usage of the stem.util.system.cwd function.
     """
 
-    runner = test.runner.get_runner()
-
     if stem.util.system.is_windows():
-      test.runner.skip(self, '(unavailable on windows)')
-      return
-    elif not runner.is_ptraceable():
-      test.runner.skip(self, '(DisableDebuggerAttachment is set)')
+      self.skipTest('(unavailable on windows)')
       return
 
+    runner = test.runner.get_runner()
     runner_pid, tor_cwd = runner.get_pid(), runner.get_tor_cwd()
     self.assertEqual(tor_cwd, stem.util.system.cwd(runner_pid))
     self.assertEqual(None, stem.util.system.cwd(99999))
 
+  @test.require.ptrace
+  @test.require.command('pwdx')
   def test_cwd_pwdx(self):
     """
     Tests the pid_by_cwd function with a pwdx response.
     """
 
-    runner = test.runner.get_runner()
-
-    if not stem.util.system.is_available('pwdx'):
-      test.runner.skip(self, '(pwdx unavailable)')
-      return
-    elif not runner.is_ptraceable():
-      test.runner.skip(self, '(DisableDebuggerAttachment is set)')
-      return
-
     # filter the call function to only allow this command
 
     pwdx_prefix = stem.util.system.GET_CWD_PWDX % ''
@@ -417,23 +400,17 @@ class TestSystem(unittest.TestCase):
     with patch('stem.util.system.call') as call_mock:
       call_mock.side_effect = call_replacement
 
+      runner = test.runner.get_runner()
       runner_pid, tor_cwd = runner.get_pid(), runner.get_tor_cwd()
       self.assertEqual(tor_cwd, stem.util.system.cwd(runner_pid))
 
+  @test.require.ptrace
+  @test.require.command('lsof')
   def test_cwd_lsof(self):
     """
     Tests the pid_by_cwd function with a lsof response.
     """
 
-    runner = test.runner.get_runner()
-
-    if not stem.util.system.is_available('lsof'):
-      test.runner.skip(self, '(lsof unavailable)')
-      return
-    elif not runner.is_ptraceable():
-      test.runner.skip(self, '(DisableDebuggerAttachment is set)')
-      return
-
     # filter the call function to only allow this command
 
     lsof_prefix = 'lsof -a -p '
@@ -443,6 +420,7 @@ class TestSystem(unittest.TestCase):
     with patch('stem.util.system.call') as call_mock:
       call_mock.side_effect = call_replacement
 
+      runner = test.runner.get_runner()
       runner_pid, tor_cwd = runner.get_pid(), runner.get_tor_cwd()
       self.assertEqual(tor_cwd, stem.util.system.cwd(runner_pid))
 
@@ -455,15 +433,12 @@ class TestSystem(unittest.TestCase):
     self.assertEqual(None, stem.util.system.user(-5))
     self.assertEqual(None, stem.util.system.start_time(98765))
 
+  @test.require.proc
   def test_user_proc(self):
     """
     Tests the user function with a proc response.
     """
 
-    if not stem.util.proc.is_available():
-      test.runner.skip(self, '(proc unavailable)')
-      return
-
     call_replacement = filter_system_call(['ps '])
 
     with patch('stem.util.system.call') as call_mock:
@@ -474,16 +449,13 @@ class TestSystem(unittest.TestCase):
       pid = test.runner.get_runner().get_pid()
       self.assertTrue(getpass.getuser(), stem.util.system.user(pid))
 
+  @test.require.command('ps')
   @patch('stem.util.proc.is_available', Mock(return_value = False))
   def test_user_ps(self):
     """
     Tests the user function with a ps response.
     """
 
-    if not stem.util.system.is_available('ps'):
-      test.runner.skip(self, '(ps unavailable)')
-      return
-
     pid = test.runner.get_runner().get_pid()
     self.assertTrue(getpass.getuser(), stem.util.system.user(pid))
 
@@ -496,15 +468,12 @@ class TestSystem(unittest.TestCase):
     self.assertEqual(None, stem.util.system.start_time(-5))
     self.assertEqual(None, stem.util.system.start_time(98765))
 
+  @test.require.proc
   def test_start_time_proc(self):
     """
     Tests the start_time function with a proc response.
     """
 
-    if not stem.util.proc.is_available():
-      test.runner.skip(self, '(proc unavailable)')
-      return
-
     call_replacement = filter_system_call(['ps '])
 
     with patch('stem.util.system.call') as call_mock:
@@ -513,16 +482,13 @@ class TestSystem(unittest.TestCase):
       pid = test.runner.get_runner().get_pid()
       self.assertTrue(stem.util.system.start_time(pid) >= 0)
 
+  @test.require.command('ps')
   @patch('stem.util.proc.is_available', Mock(return_value = False))
   def test_start_time_ps(self):
     """
     Tests the start_time function with a ps response.
     """
 
-    if not stem.util.system.is_available('ps'):
-      test.runner.skip(self, '(ps unavailable)')
-      return
-
     pid = test.runner.get_runner().get_pid()
     self.assertTrue(stem.util.system.start_time(pid) >= 0)
 
@@ -550,7 +516,7 @@ class TestSystem(unittest.TestCase):
     #   '/root'
 
     if getpass.getuser() == 'root':
-      test.runner.skip(self, '(running as root)')
+      self.skipTest('(running as root)')
       return
 
     self.assertEqual(os.getcwd(), stem.util.system.expand_path('.'))
@@ -563,14 +529,17 @@ class TestSystem(unittest.TestCase):
     self.assertEqual(home_dir, stem.util.system.expand_path('~%s' % username))
     self.assertEqual(os.path.join(home_dir, 'foo'), stem.util.system.expand_path('~%s/foo' % username))
 
+  def test_call_timeout(self):
+    self.assertRaisesRegexp(stem.util.system.CallTimeoutError, "Process didn't finish after 0.0 seconds", stem.util.system.call, 'sleep 1', timeout = 0.001)
+
   def test_call_time_tracked(self):
     """
     Check that time taken in the call() function is tracked by SYSTEM_CALL_TIME.
     """
 
     initial = stem.util.system.SYSTEM_CALL_TIME
-    stem.util.system.call('sleep 0.5')
-    self.assertTrue(stem.util.system.SYSTEM_CALL_TIME - initial > 0.4)
+    stem.util.system.call('sleep 0.005')
+    self.assertTrue(stem.util.system.SYSTEM_CALL_TIME - initial > 0.004)
 
   def test_set_process_name(self):
     """
@@ -585,21 +554,3 @@ class TestSystem(unittest.TestCase):
       self.assertEqual('stem_integ', stem.util.system.get_process_name())
     finally:
       stem.util.system.set_process_name(initial_name)
-
-  def _is_extra_tor_running(self):
-    # Try to figure out if there's more than one tor instance running. This
-    # check will fail if pgrep is unavailable (for instance on bsd) but this
-    # isn't the end of the world. It's just used to skip tests if they should
-    # legitemately fail.
-
-    if stem.util.system.is_windows():
-      # TODO: not sure how to check for this on windows
-      return False
-    elif not stem.util.system.is_bsd():
-      tor_cmd = test.runner.get_runner().get_tor_command(True)
-      pgrep_results = stem.util.system.call(stem.util.system.GET_PID_BY_NAME_PGREP % tor_cmd)
-      return len(pgrep_results) > 1
-    else:
-      ps_results = stem.util.system.call(stem.util.system.GET_PID_BY_NAME_PS_BSD)
-      results = [r for r in ps_results if r.endswith(' tor')]
-      return len(results) > 1
