@@ -1,4 +1,4 @@
-# Copyright 2011-2016, Damian Johnson and The Tor Project
+# Copyright 2011-2017, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -11,20 +11,14 @@ about the tor test instance they're running against.
   RunnerStopped - Runner doesn't have an active tor instance
   TorInaccessable - Tor can't be queried for the information
 
-  skip - skips the current test if we can
-  require_controller - skips the test unless tor provides a controller endpoint
-  require_version - skips the test unless we meet a tor version requirement
-  require_online - skips unless targets allow for online tests
-  only_run_once - skip the test if it has been ran before
   exercise_controller - basic sanity check that a controller connection can be used
-
   get_runner - Singleton for fetching our runtime context.
+
   Runner - Runtime context for our integration tests.
     |- start - prepares and starts a tor instance for our tests to run against
     |- stop - stops our tor instance and cleans up any temporary files
     |- is_running - checks if our tor test instance is running
     |- is_accessible - checks if our tor instance can be connected to
-    |- is_ptraceable - checks if DisableDebuggerAttachment is set
     |- get_options - custom torrc options used for our test instance
     |- get_test_dir - testing directory path
     |- get_torrc_path - path to our tor instance's torrc
@@ -35,7 +29,6 @@ about the tor test instance they're running against.
     |- get_pid - process id of our tor process
     |- get_tor_socket - provides a socket to our test instance
     |- get_tor_controller - provides a controller for our test instance
-    |- get_tor_version - provides the version of tor we're running against
     +- get_tor_command - provides the command used to start tor
 """
 
@@ -54,26 +47,25 @@ import stem.process
 import stem.socket
 import stem.util.conf
 import stem.util.enum
-import stem.version
+import test
 
 from test.output import println, STATUS, ERROR, SUBSTATUS, NO_NL
-from test.util import Target, STEM_BASE
 
 CONFIG = stem.util.conf.config_dict('test', {
   'integ.test_directory': './test/data',
   'integ.log': './test/data/log',
+  'target.torrc': {},
 })
 
-SOCKS_HOST = '127.0.0.1'
 SOCKS_PORT = 1112
 
 BASE_TORRC = """# configuration for stem integration tests
 DataDirectory %%s
-SocksListenAddress %s:%i
+SocksPort %i
 DownloadExtraInfo 1
 Log notice stdout
 Log notice file %%s/tor_log
-""" % (SOCKS_HOST, SOCKS_PORT)
+""" % SOCKS_PORT
 
 # singleton Runner instance
 INTEG_RUNNER = None
@@ -91,9 +83,6 @@ Torrc = stem.util.enum.Enum(
   ('PTRACE', 'DisableDebuggerAttachment 0'),
 )
 
-# (test_instance, test_name) tuples that we've registered as having been ran
-RAN_TESTS = []
-
 
 class RunnerStopped(Exception):
   "Raised when we try to use a Runner that doesn't have an active tor instance"
@@ -103,85 +92,6 @@ class TorInaccessable(Exception):
   'Raised when information is needed from tor but the instance we have is inaccessible'
 
 
-def skip(test_case, message):
-  """
-  Skips the test if we can. The capability for skipping tests was added in
-  python 2.7 so callers should return after this, so they report 'success' if
-  this method is unavailable.
-
-  :param unittest.TestCase test_case: test being ran
-  :param str message: message to skip the test with
-  """
-
-  if not stem.prereq._is_python_26():
-    test_case.skipTest(message)
-
-
-def require_controller(func):
-  """
-  Skips the test unless tor provides an endpoint for controllers to attach to.
-  """
-
-  def wrapped(self, *args, **kwargs):
-    if get_runner().is_accessible():
-      return func(self, *args, **kwargs)
-    else:
-      skip(self, '(no connection)')
-
-  return wrapped
-
-
-def require_version(req_version):
-  """
-  Skips the test unless we meet the required version.
-
-  :param stem.version.Version req_version: required tor version for the test
-  """
-
-  def decorator(func):
-    def wrapped(self, *args, **kwargs):
-      if get_runner().get_tor_version() >= req_version:
-        return func(self, *args, **kwargs)
-      else:
-        skip(self, '(requires %s)' % req_version)
-
-    return wrapped
-
-  return decorator
-
-
-def require_online(func):
-  """
-  Skips the test if we weren't started with the ONLINE target, which indicates
-  that tests requiring network connectivity should run.
-  """
-
-  def wrapped(self, *args, **kwargs):
-    if Target.ONLINE in get_runner().attribute_targets:
-      return func(self, *args, **kwargs)
-    else:
-      skip(self, '(requires online target)')
-
-  return wrapped
-
-
-def only_run_once(func):
-  """
-  Skips the test if it has ran before. If it hasn't then flags it as being ran.
-  This is useful to prevent lengthy tests that are independent of integ targets
-  from being run repeatedly with ``RUN_ALL``.
-  """
-
-  def wrapped(self, *args, **kwargs):
-    if (self, self.id()) not in RAN_TESTS:
-      RAN_TESTS.append((self, self.id()))
-      return func(self, *args, **kwargs)
-    else:
-      skip(self, '(already ran)')
-
-  return wrapped
-
-
 def exercise_controller(test_case, controller):
   """
   Checks that we can now use the socket by issuing a 'GETINFO config-file'
@@ -236,7 +146,6 @@ class _MockChrootFile(object):
 
 class Runner(object):
   def __init__(self):
-    self.run_target = None
     self.attribute_targets = []
 
     self._runner_lock = threading.RLock()
@@ -255,21 +164,25 @@ class Runner(object):
 
     self._original_recv_message = None
 
-  def start(self, run_target, attribute_targets, tor_cmd, extra_torrc_opts):
+    # The first controller to attach takes ownership so tor will promptly
+    # terminate if the tests do. As such we need to ensure that first
+    # connection is our runner's.
+
+    self._owner_controller = None
+
+  def start(self, config_target, attribute_targets, tor_cmd):
     """
     Makes temporary testing resources and starts tor, blocking until it
     completes.
 
-    :param Target run_target: configuration we're running with
+    :param str config_target: **Target** for this test run's torrc settings
     :param list attribute_targets: **Targets** for our non-configuration attributes
     :param str tor_cmd: command to start tor with
-    :param list extra_torrc_opts: additional torrc options for our test instance
 
     :raises: OSError if unable to run test preparations or start tor
     """
 
     with self._runner_lock:
-      self.run_target = run_target
       self.attribute_targets = attribute_targets
 
       # if we're holding on to a tor process (running or not) then clean up after
@@ -286,13 +199,14 @@ class Runner(object):
       config_test_dir = CONFIG['integ.test_directory']
 
       if config_test_dir:
-        self._test_dir = stem.util.system.expand_path(config_test_dir, STEM_BASE)
+        self._test_dir = stem.util.system.expand_path(config_test_dir, test.STEM_BASE)
       else:
         self._test_dir = tempfile.mktemp('-stem-integ')
 
       original_cwd, data_dir_path = os.getcwd(), self._test_dir
+      self._tor_cmd = stem.util.system.expand_path(tor_cmd) if os.path.sep in tor_cmd else tor_cmd
 
-      if Target.RELATIVE in self.attribute_targets:
+      if test.Target.RELATIVE in self.attribute_targets:
         tor_cwd = os.path.dirname(self._test_dir)
 
         if not os.path.exists(tor_cwd):
@@ -301,7 +215,18 @@ class Runner(object):
         os.chdir(tor_cwd)
         data_dir_path = './%s' % os.path.basename(self._test_dir)
 
-      self._tor_cmd = tor_cmd
+      config_csv = CONFIG['target.torrc'].get(config_target)
+      extra_torrc_opts = []
+
+      if config_csv:
+        for opt in config_csv.split(','):
+          opt = opt.strip()
+
+          if opt in Torrc.keys():
+            extra_torrc_opts.append(Torrc[opt])
+          else:
+            raise ValueError("'%s' isn't a test.runner.Torrc enumeration" % opt)
+
       self._custom_opts = extra_torrc_opts
       self._torrc_contents = BASE_TORRC % (data_dir_path, data_dir_path)
 
@@ -311,12 +236,12 @@ class Runner(object):
       try:
         self._tor_cwd = os.getcwd()
         self._run_setup()
-        self._start_tor(tor_cmd)
+        self._start_tor(self._tor_cmd)
 
         # strip the testing directory from recv_message responses if we're
         # simulating a chroot setup
 
-        if Target.CHROOT in self.attribute_targets and not self._original_recv_message:
+        if test.Target.CHROOT in self.attribute_targets and not self._original_recv_message:
           # TODO: when we have a function for telling stem the chroot we'll
           # need to set that too
 
@@ -328,9 +253,11 @@ class Runner(object):
 
           stem.socket.recv_message = _chroot_recv_message
 
-        # revert our cwd back to normal
-        if Target.RELATIVE in self.attribute_targets:
-          os.chdir(original_cwd)
+        if self.is_accessible():
+          self._owner_controller = self.get_tor_controller(True)
+
+        if test.Target.RELATIVE in self.attribute_targets:
+          os.chdir(original_cwd)  # revert our cwd back to normal
       except OSError as exc:
         raise exc
 
@@ -342,6 +269,10 @@ class Runner(object):
     with self._runner_lock:
       println('Shutting down tor... ', STATUS, NO_NL)
 
+      if self._owner_controller:
+        self._owner_controller.close()
+        self._owner_controller = None
+
       if self._tor_process:
         # if the tor process has stopped on its own then the following raises
         # an OSError ([Errno 3] No such process)
@@ -404,21 +335,6 @@ class Runner(object):
 
     return Torrc.PORT in self._custom_opts or Torrc.SOCKET in self._custom_opts
 
-  def is_ptraceable(self):
-    """
-    Checks if tor's 'DisableDebuggerAttachment' option is set. This feature has
-    a lot of adverse side effects (:trac:`3313`).
-
-    :returns: True if debugger attachment is allowed, False otherwise
-    """
-
-    # If we're running a tor version where ptrace is disabled and we didn't
-    # set 'DisableDebuggerAttachment=1' then we can infer that it's disabled.
-
-    tor_version = self.get_tor_version()
-    has_option = tor_version >= stem.version.Requirement.TORRC_DISABLE_DEBUGGER_ATTACHMENT
-    return not has_option or Torrc.PTRACE in self.get_options()
-
   def get_options(self):
     """
     Provides the custom torrc options our tor instance is running with.
@@ -559,33 +475,6 @@ class Runner(object):
 
     return controller
 
-  def get_tor_version(self):
-    """
-    Queries our test instance for tor's version.
-
-    :returns: :class:`stem.version.Version` for our test instance
-    """
-
-    try:
-      # TODO: replace with higher level functions when we've completed a basic
-      # controller class
-
-      control_socket = self.get_tor_socket()
-
-      control_socket.send('GETINFO version')
-      version_response = control_socket.recv()
-      control_socket.close()
-
-      tor_version = list(version_response)[0]
-      tor_version = tor_version[8:]
-
-      if ' ' in tor_version:
-        tor_version = tor_version.split(' ', 1)[0]
-
-      return stem.version.Version(tor_version)
-    except TorInaccessable:
-      return stem.version.get_system_tor_version(self.get_tor_command())
-
   def get_tor_command(self, base_cmd = False):
     """
     Provides the command used to run our tor instance.
@@ -659,7 +548,7 @@ class Runner(object):
     logging_path = CONFIG['integ.log']
 
     if logging_path:
-      logging_path = stem.util.system.expand_path(logging_path, STEM_BASE)
+      logging_path = stem.util.system.expand_path(logging_path, test.STEM_BASE)
       println('  configuring logger (%s)... ' % logging_path, STATUS, NO_NL)
 
       # delete the old log
@@ -711,16 +600,13 @@ class Runner(object):
     start_time = time.time()
 
     try:
-      # wait to fully complete if we're running tests with network activity,
-      # otherwise finish after local bootstraping
-
-      complete_percent = 100 if Target.ONLINE in self.attribute_targets else 5
-
-      def print_init_line(line):
-        println('  %s' % line, SUBSTATUS)
-
-      torrc_dst = os.path.join(self._test_dir, 'torrc')
-      self._tor_process = stem.process.launch_tor(tor_cmd, None, torrc_dst, complete_percent, print_init_line, take_ownership = True)
+      self._tor_process = stem.process.launch_tor(
+        tor_cmd = tor_cmd,
+        torrc_path = os.path.join(self._test_dir, 'torrc'),
+        completion_percent = 100 if test.Target.ONLINE in self.attribute_targets else 5,
+        init_msg_handler = lambda line: println('  %s' % line, SUBSTATUS),
+        take_ownership = True,
+      )
 
       runtime = time.time() - start_time
       println('  done (%i seconds)\n' % runtime, STATUS)
