@@ -5,6 +5,7 @@ integ tests, but a few bits lend themselves to unit testing.
 
 import datetime
 import io
+import time
 import unittest
 
 import stem.descriptor.router_status_entry
@@ -15,8 +16,8 @@ import stem.version
 
 from stem import ControllerError, DescriptorUnavailable, InvalidArguments, InvalidRequest, ProtocolError, UnsatisfiableRequest
 from stem.control import _parse_circ_path, Listener, Controller, EventType
+from stem.response import ControlMessage
 from stem.exit_policy import ExitPolicy
-from test import mocking
 
 try:
   # added in python 3.3
@@ -45,6 +46,57 @@ class TestControl(unittest.TestCase):
     for event in stem.control.EventType:
       self.assertTrue(stem.control.event_description(event) is not None)
 
+  @patch('stem.control.Controller.msg')
+  def test_get_info(self, msg_mock):
+    msg_mock.return_value = ControlMessage.from_str('250-hello=hi right back!\r\n250 OK\r\n', 'GETINFO')
+    self.assertEqual('hi right back!', self.controller.get_info('hello'))
+
+  @patch('stem.control.Controller.msg')
+  def test_get_info_address_caching(self, msg_mock):
+    test_start = time.time()
+    msg_mock.return_value = ControlMessage.from_str('551 Address unknown\r\n')
+
+    self.assertEqual(0, self.controller._address_cached_at)
+    self.assertEqual(None, self.controller._last_address_exc)
+    self.assertRaisesRegexp(stem.ProtocolError, 'Address unknown', self.controller.get_info, 'address')
+    self.assertTrue(test_start <= self.controller._address_cached_at <= time.time())
+    self.assertEqual("GETINFO response didn't have an OK status:\nAddress unknown", str(self.controller._last_address_exc))
+    self.assertEqual(1, msg_mock.call_count)
+
+    # now that we have a cached failure we should provide that back
+
+    self.assertRaisesRegexp(stem.ProtocolError, 'Address unknown', self.controller.get_info, 'address')
+    self.assertEqual(1, msg_mock.call_count)
+
+    # pretend it's a minute later and try again, now with an address
+
+    msg_mock.return_value = ControlMessage.from_str('250-address=17.2.89.80\r\n250 OK\r\n', 'GETINFO')
+    self.assertRaisesRegexp(stem.ProtocolError, 'Address unknown', self.controller.get_info, 'address')
+    self.controller._address_cached_at -= 60
+    self.assertEqual('17.2.89.80', self.controller.get_info('address'))
+
+  @patch('stem.control.Controller.msg')
+  @patch('stem.control.Controller.get_conf')
+  def test_get_info_without_fingerprint(self, get_conf_mock, msg_mock):
+    msg_mock.return_value = ControlMessage.from_str('551 Not running in server mode\r\n')
+    get_conf_mock.return_value = None
+
+    self.assertEqual(None, self.controller._last_fingerprint_exc)
+    self.assertRaisesRegexp(stem.ProtocolError, 'Not running in server mode', self.controller.get_info, 'fingerprint')
+    self.assertEqual("GETINFO response didn't have an OK status:\nNot running in server mode", str(self.controller._last_fingerprint_exc))
+    self.assertEqual(1, msg_mock.call_count)
+
+    # now that we have a cached failure we should provide that back
+
+    self.assertRaisesRegexp(stem.ProtocolError, 'Not running in server mode', self.controller.get_info, 'fingerprint')
+    self.assertEqual(1, msg_mock.call_count)
+
+    # ... but if we become a relay we'll call it again
+
+    get_conf_mock.return_value = '443'
+    self.assertRaisesRegexp(stem.ProtocolError, 'Not running in server mode', self.controller.get_info, 'fingerprint')
+    self.assertEqual(2, msg_mock.call_count)
+
   @patch('stem.control.Controller.get_info')
   def test_get_version(self, get_info_mock):
     """
@@ -73,6 +125,11 @@ class TestControl(unittest.TestCase):
       # Return a version without caching, so it will be the new version.
       self.assertEqual(version_2_2_object, self.controller.get_version())
 
+      # Spec says the getinfo response may optionally be prefixed by 'Tor '. In
+      # practice it doesn't but we should accept that.
+      get_info_mock.return_value = 'Tor 0.2.1.32'
+      self.assertEqual(version_2_1_object, self.controller.get_version())
+
       # Raise an exception in the get_info() call.
       get_info_mock.side_effect = InvalidArguments
 
@@ -157,6 +214,7 @@ class TestControl(unittest.TestCase):
 
     self.assertEqual([('127.0.0.1', 9050)], self.controller.get_listeners(Listener.CONTROL))
     self.assertEqual([9050], self.controller.get_ports(Listener.CONTROL))
+    self.controller.clear_cache()
 
     # non-local addresss
 
@@ -167,8 +225,9 @@ class TestControl(unittest.TestCase):
 
     self.assertEqual([('27.4.4.1', 9050)], self.controller.get_listeners(Listener.CONTROL))
     self.assertEqual([], self.controller.get_ports(Listener.CONTROL))
+    self.controller.clear_cache()
 
-    # Exercise via the GETINFO option.
+    # exercise via the GETINFO option
 
     get_info_mock.side_effect = None
     get_info_mock.return_value = '"127.0.0.1:1112" "127.0.0.1:1114"'
@@ -179,6 +238,15 @@ class TestControl(unittest.TestCase):
     )
 
     self.assertEqual([1112, 1114], self.controller.get_ports(Listener.CONTROL))
+    self.controller.clear_cache()
+
+    # with all localhost addresses, including a couple that aren't
+
+    get_info_mock.side_effect = None
+    get_info_mock.return_value = '"27.4.4.1:1113" "127.0.0.5:1114" "0.0.0.0:1115" "[::]:1116" "[::1]:1117" "[10::]:1118"'
+
+    self.assertEqual([1114, 1115, 1116, 1117], self.controller.get_ports(Listener.OR))
+    self.controller.clear_cache()
 
     # IPv6 address
 
@@ -191,6 +259,7 @@ class TestControl(unittest.TestCase):
 
     # unix socket file
 
+    self.controller.clear_cache()
     get_info_mock.return_value = '"unix:/tmp/tor/socket"'
 
     self.assertEqual([], self.controller.get_listeners(Listener.CONTROL))
@@ -215,6 +284,7 @@ class TestControl(unittest.TestCase):
     }[param]
 
     self.assertEqual([('127.0.0.1', 9050)], self.controller.get_socks_listeners())
+    self.controller.clear_cache()
 
     # Again, an old tor, but SocksListenAddress overrides the port number.
 
@@ -224,6 +294,7 @@ class TestControl(unittest.TestCase):
     }[param]
 
     self.assertEqual([('127.0.0.1', 1112)], self.controller.get_socks_listeners())
+    self.controller.clear_cache()
 
     # Again, an old tor, but multiple listeners
 
@@ -233,6 +304,7 @@ class TestControl(unittest.TestCase):
     }[param]
 
     self.assertEqual([('127.0.0.1', 1112), ('127.0.0.1', 1114)], self.controller.get_socks_listeners())
+    self.controller.clear_cache()
 
     # Again, an old tor, but no SOCKS listeners
 
@@ -242,6 +314,7 @@ class TestControl(unittest.TestCase):
     }[param]
 
     self.assertEqual([], self.controller.get_socks_listeners())
+    self.controller.clear_cache()
 
     # Where tor provides invalid ports or addresses
 
@@ -284,6 +357,7 @@ class TestControl(unittest.TestCase):
 
     # no SOCKS listeners
 
+    self.controller.clear_cache()
     get_info_mock.return_value = ''
     self.assertEqual([], self.controller.get_socks_listeners())
 
@@ -297,6 +371,7 @@ class TestControl(unittest.TestCase):
     )
 
     for response in invalid_responses:
+      self.controller.clear_cache()
       get_info_mock.return_value = response
       self.assertRaises(stem.ProtocolError, self.controller.get_socks_listeners)
 
@@ -338,13 +413,14 @@ class TestControl(unittest.TestCase):
 
     # use the handy mocked protocolinfo response
 
-    get_protocolinfo_mock.return_value = mocking.get_protocolinfo_response()
+    protocolinfo_msg = ControlMessage.from_str('250-PROTOCOLINFO 1\r\n250 OK\r\n', 'PROTOCOLINFO')
+    get_protocolinfo_mock.return_value = protocolinfo_msg
 
     # compare the str representation of these object, because the class
     # does not have, nor need, a direct comparison operator
 
     self.assertEqual(
-      str(mocking.get_protocolinfo_response()),
+      str(protocolinfo_msg),
       str(self.controller.get_protocolinfo())
     )
 
@@ -442,12 +518,8 @@ class TestControl(unittest.TestCase):
 
     get_info_mock.side_effect = ControllerError('nope, too bad')
 
-    try:
-      self.controller.get_network_status()
-      self.fail("We should've raised an exception")
-    except ControllerError as exc:
-      self.assertEqual('Unable to determine our own fingerprint: nope, too bad', str(exc))
-
+    exc_msg = 'Unable to determine our own fingerprint: nope, too bad'
+    self.assertRaisesRegexp(ControllerError, exc_msg, self.controller.get_network_status)
     self.assertEqual('boom', self.controller.get_network_status(default = 'boom'))
 
     # successful request
@@ -462,18 +534,15 @@ class TestControl(unittest.TestCase):
     self.assertEqual(stem.descriptor.router_status_entry.RouterStatusEntryV3(desc), self.controller.get_network_status())
 
   @patch('stem.control.Controller.get_info')
-  def test_get_network_status_when_raising_descriptor_unavailable(self, get_info_mock):
+  def test_get_network_status_when_unavailable(self, get_info_mock):
     """
     Exercises the get_network_status() method.
     """
 
     get_info_mock.side_effect = InvalidArguments(None, 'GETINFO request contained unrecognized keywords: ns/id/5AC9C5AA75BA1F18D8459B326B4B8111A856D290')
 
-    try:
-      self.controller.get_network_status('5AC9C5AA75BA1F18D8459B326B4B8111A856D290')
-      self.fail("We should've raised an exception")
-    except DescriptorUnavailable as exc:
-      self.assertEqual("Tor was unable to provide the descriptor for '5AC9C5AA75BA1F18D8459B326B4B8111A856D290'", str(exc))
+    exc_msg = "Tor was unable to provide the descriptor for '5AC9C5AA75BA1F18D8459B326B4B8111A856D290'"
+    self.assertRaisesRegexp(DescriptorUnavailable, exc_msg, self.controller.get_network_status, '5AC9C5AA75BA1F18D8459B326B4B8111A856D290')
 
   @patch('stem.control.Controller.get_info')
   def test_get_network_status(self, get_info_mock):
