@@ -14,7 +14,6 @@ import time
 import unittest
 
 import stem.descriptor.reader
-import test.runner
 import test.unit.descriptor
 
 from stem.util import str_type, system
@@ -191,7 +190,7 @@ class TestDescriptorReader(unittest.TestCase):
     # test relies on being unable to read a file
 
     if getpass.getuser() == 'root':
-      test.runner.skip(self, '(running as root)')
+      self.skipTest('(running as root)')
       return
 
     # Skip the test on windows, since you can only set the file's
@@ -199,7 +198,7 @@ class TestDescriptorReader(unittest.TestCase):
     # http://docs.python.org/library/os.html#os.chmod
 
     if system.is_windows():
-      test.runner.skip(self, '(chmod not functional)')
+      self.skipTest('(chmod not functional)')
 
     test_listing_path = self._make_processed_files_listing(BASIC_LISTING)
     os.chmod(test_listing_path, 0o077)  # remove read permissions
@@ -315,7 +314,7 @@ class TestDescriptorReader(unittest.TestCase):
 
     with reader:
       self.assertTrue(reader.get_buffered_descriptor_count() <= 2)
-      time.sleep(0.01)
+      time.sleep(0.001)
       self.assertTrue(reader.get_buffered_descriptor_count() <= 2)
 
   def test_persistence_path(self):
@@ -397,6 +396,13 @@ class TestDescriptorReader(unittest.TestCase):
     Checks that we can read descriptors from an bzipped archive.
     """
 
+    # when python's compiled it only optionally has bz2 support
+
+    try:
+      import bz2
+    except ImportError:
+      self.skipTest('(bz2 unsupported}')
+
     expected_results = _get_raw_tar_descriptors()
     test_path = os.path.join(DESCRIPTOR_TEST_DATA, 'descriptor_archive.tar.bz2')
 
@@ -413,7 +419,7 @@ class TestDescriptorReader(unittest.TestCase):
     # Skip on windows since SIGALRM is unavailable
 
     if system.is_windows():
-      test.runner.skip(self, '(SIGALRM unavailable)')
+      self.skipTest('(SIGALRM unavailable)')
 
     is_test_running = True
     reader = stem.descriptor.reader.DescriptorReader('/usr')
@@ -430,7 +436,7 @@ class TestDescriptorReader(unittest.TestCase):
     signal.alarm(2)
 
     reader.start()
-    time.sleep(0.1)
+    time.sleep(0.001)
     reader.stop()
     is_test_running = False
 
@@ -440,20 +446,15 @@ class TestDescriptorReader(unittest.TestCase):
     iterating over our test data.
     """
 
-    expected_results = {}
+    desc_path = os.path.join(DESCRIPTOR_TEST_DATA, 'example_descriptor')
+    last_modified = int(os.stat(desc_path).st_mtime)
 
-    for root, _, files in os.walk(DESCRIPTOR_TEST_DATA):
-      for filename in files:
-        path = os.path.join(root, filename)
-        last_modified = int(os.stat(path).st_mtime)
-        expected_results[path] = last_modified
-
-    reader = stem.descriptor.reader.DescriptorReader(DESCRIPTOR_TEST_DATA)
+    reader = stem.descriptor.reader.DescriptorReader(desc_path)
 
     with reader:
       list(reader)  # iterates over all of the descriptors
 
-    self.assertEqual(expected_results, reader.get_processed_files())
+    self.assertEqual({desc_path: last_modified}, reader.get_processed_files())
 
   def test_skip_nondescriptor_contents(self):
     """
@@ -462,7 +463,7 @@ class TestDescriptorReader(unittest.TestCase):
     """
 
     skip_listener = SkipListener()
-    reader = stem.descriptor.reader.DescriptorReader(DESCRIPTOR_TEST_DATA)
+    reader = stem.descriptor.reader.DescriptorReader(os.path.join(DESCRIPTOR_TEST_DATA, 'unparseable'))
     reader.register_skip_listener(skip_listener.listener)
 
     expected_skip_files = ('riddle', 'tiny.png', 'vote', 'new_metrics_type', 'cached-microdesc-consensus_with_carriage_returns', 'extrainfo_nonascii_v3_reqs')
@@ -555,10 +556,10 @@ class TestDescriptorReader(unittest.TestCase):
     # test relies on being unable to read a file
 
     if getpass.getuser() == 'root':
-      test.runner.skip(self, '(running as root)')
+      self.skipTest('(running as root)')
       return
     elif system.is_windows():
-      test.runner.skip(self, '(chmod not functional)')
+      self.skipTest('(chmod not functional)')
       return
 
     test_path = os.path.join(self.temp_directory, 'secret_file')
@@ -612,7 +613,7 @@ class TestDescriptorReader(unittest.TestCase):
     Parses a file that has a valid metrics header, but an unrecognized type.
     """
 
-    test_path = test.unit.descriptor.get_resource('new_metrics_type')
+    test_path = test.unit.descriptor.get_resource('unparseable/new_metrics_type')
 
     skip_listener = SkipListener()
     reader = stem.descriptor.reader.DescriptorReader(test_path)
