@@ -3,22 +3,27 @@ Unit tests for stem.descriptor.router_status_entry.
 """
 
 import datetime
+import functools
 import unittest
 
 import stem.descriptor
+import test.require
 
 from stem import Flag
-from stem.descriptor.router_status_entry import RouterStatusEntryV3, _base64_to_hex
 from stem.exit_policy import MicroExitPolicy
 from stem.version import Version
 
-from test.unit.descriptor import get_resource
+from test.unit.descriptor import (
+  get_resource,
+  base_expect_invalid_attr,
+  base_expect_invalid_attr_for_text,
+)
 
-from test.mocking import (
-  get_router_status_entry_v2,
-  get_router_status_entry_v3,
-  get_router_status_entry_micro_v3,
-  ROUTER_STATUS_ENTRY_V3_HEADER,
+from stem.descriptor.router_status_entry import (
+  RouterStatusEntryV2,
+  RouterStatusEntryV3,
+  RouterStatusEntryMicroV3,
+  _base64_to_hex,
 )
 
 ENTRY_WITHOUT_ED25519 = """\
@@ -47,6 +52,9 @@ m 18,19,20 sha256=AkZH3gIvz3wunsroqh5izBJizdYuR7kn2oVbsvqgML8
 m 21 sha256=AVp41YVxKEJCaoEf0+77Cdvyw5YgpyDXdob0+LSv/pE
 """
 
+expect_invalid_attr = functools.partial(base_expect_invalid_attr, RouterStatusEntryV3, 'nickname', 'Unnamed')
+expect_invalid_attr_for_text = functools.partial(base_expect_invalid_attr_for_text, RouterStatusEntryV3, 'nickname', 'Unnamed')
+
 
 def vote_document():
   class Stub(object):
@@ -86,14 +94,11 @@ class TestRouterStatusEntry(unittest.TestCase):
     Parses a minimal v2 router status entry.
     """
 
-    entry = get_router_status_entry_v2()
+    entry = RouterStatusEntryV2.create()
 
     self.assertEqual(None, entry.document)
-    self.assertEqual('caerSidi', entry.nickname)
-    self.assertEqual('A7569A83B5706AB1B1A9CB52EFF7D2D32E4553EB', entry.fingerprint)
+    self.assertTrue(entry.nickname.startswith('Unnamed'))
     self.assertEqual('A106452D87BD7B803B6CE916291ED368DC5BD091', entry.digest)
-    self.assertEqual(datetime.datetime(2012, 8, 6, 11, 19, 31), entry.published)
-    self.assertEqual('71.35.150.29', entry.address)
     self.assertEqual(9001, entry.or_port)
     self.assertEqual(None, entry.dir_port)
     self.assertEqual(None, entry.flags)
@@ -106,15 +111,12 @@ class TestRouterStatusEntry(unittest.TestCase):
     Parses a minimal v3 router status entry.
     """
 
-    entry = get_router_status_entry_v3()
+    entry = RouterStatusEntryV3.create()
 
     expected_flags = set([Flag.FAST, Flag.NAMED, Flag.RUNNING, Flag.STABLE, Flag.VALID])
     self.assertEqual(None, entry.document)
-    self.assertEqual('caerSidi', entry.nickname)
-    self.assertEqual('A7569A83B5706AB1B1A9CB52EFF7D2D32E4553EB', entry.fingerprint)
+    self.assertTrue(entry.nickname.startswith('Unnamed'))
     self.assertEqual('A106452D87BD7B803B6CE916291ED368DC5BD091', entry.digest)
-    self.assertEqual(datetime.datetime(2012, 8, 6, 11, 19, 31), entry.published)
-    self.assertEqual('71.35.150.29', entry.address)
     self.assertEqual(9001, entry.or_port)
     self.assertEqual(None, entry.dir_port)
     self.assertEqual(expected_flags, set(entry.flags))
@@ -135,14 +137,11 @@ class TestRouterStatusEntry(unittest.TestCase):
     Parses a minimal microdescriptor v3 router status entry.
     """
 
-    entry = get_router_status_entry_micro_v3()
+    entry = RouterStatusEntryMicroV3.create()
 
     expected_flags = set([Flag.FAST, Flag.GUARD, Flag.HSDIR, Flag.NAMED, Flag.RUNNING, Flag.STABLE, Flag.V2DIR, Flag.VALID])
     self.assertEqual(None, entry.document)
-    self.assertEqual('Konata', entry.nickname)
-    self.assertEqual('011209176CDBAA2AC1F48C2C5B4990CE771C5B0C', entry.fingerprint)
-    self.assertEqual(datetime.datetime(2012, 9, 24, 13, 40, 40), entry.published)
-    self.assertEqual('69.64.48.168', entry.address)
+    self.assertTrue(entry.nickname.startswith('Unnamed'))
     self.assertEqual(9001, entry.or_port)
     self.assertEqual(9030, entry.dir_port)
     self.assertEqual(expected_flags, set(entry.flags))
@@ -151,6 +150,12 @@ class TestRouterStatusEntry(unittest.TestCase):
     self.assertEqual('6A252497006BB9AF36A1B1B902C4D7FA2129923400DBE0101F167B1B031F63BD', entry.digest)
     self.assertEqual([], entry.get_unrecognized_lines())
 
+  @test.require.cryptography
+  def test_descriptor_signing(self):
+    self.assertRaisesRegexp(NotImplementedError, 'Signing of RouterStatusEntryV2 not implemented', RouterStatusEntryV2.create, sign = True)
+    self.assertRaisesRegexp(NotImplementedError, 'Signing of RouterStatusEntryV3 not implemented', RouterStatusEntryV3.create, sign = True)
+    self.assertRaisesRegexp(NotImplementedError, 'Signing of RouterStatusEntryMicroV3 not implemented', RouterStatusEntryMicroV3.create, sign = True)
+
   def test_without_ed25519(self):
     """
     Parses a router status entry without a ed25519 value.
@@ -222,21 +227,16 @@ class TestRouterStatusEntry(unittest.TestCase):
     Parses a router status entry that's missing fields.
     """
 
-    content = get_router_status_entry_v3(exclude = ('r', 's'), content = True)
-    self._expect_invalid_attr(content, 'address')
-
-    content = get_router_status_entry_v3(exclude = ('r',), content = True)
-    self._expect_invalid_attr(content, 'address')
-
-    content = get_router_status_entry_v3(exclude = ('s',), content = True)
-    self._expect_invalid_attr(content, 'flags')
+    expect_invalid_attr_for_text(self, RouterStatusEntryV3.content(exclude = ('r', 's')), 'address')
+    expect_invalid_attr_for_text(self, RouterStatusEntryV3.content(exclude = ('r',)), 'address')
+    expect_invalid_attr_for_text(self, RouterStatusEntryV3.content(exclude = ('s',)), 'flags')
 
   def test_unrecognized_lines(self):
     """
     Parses a router status entry with new keywords.
     """
 
-    entry = get_router_status_entry_v3({'z': 'New tor feature: sparkly unicorns!'})
+    entry = RouterStatusEntryV3.create({'z': 'New tor feature: sparkly unicorns!'})
     self.assertEqual(['z New tor feature: sparkly unicorns!'], entry.get_unrecognized_lines())
 
   def test_proceeding_line(self):
@@ -244,7 +244,7 @@ class TestRouterStatusEntry(unittest.TestCase):
     Includes content prior to the 'r' line.
     """
 
-    content = b'z some stuff\n' + get_router_status_entry_v3(content = True)
+    content = b'z some stuff\n' + RouterStatusEntryV3.content()
     self.assertRaises(ValueError, RouterStatusEntryV3, content, True)
     self.assertEqual(['z some stuff'], RouterStatusEntryV3(content, False).get_unrecognized_lines())
 
@@ -253,7 +253,7 @@ class TestRouterStatusEntry(unittest.TestCase):
     Includes blank lines, which should be ignored.
     """
 
-    content = get_router_status_entry_v3(content = True) + b'\n\nv Tor 0.2.2.35\n\n'
+    content = RouterStatusEntryV3.content() + b'\n\nv Tor 0.2.2.35\n\n'
     entry = RouterStatusEntryV3(content)
     self.assertEqual('Tor 0.2.2.35', entry.version_line)
 
@@ -262,14 +262,14 @@ class TestRouterStatusEntry(unittest.TestCase):
     Duplicates linesin the entry.
     """
 
-    lines = get_router_status_entry_v3(content = True).split(b'\n')
+    lines = RouterStatusEntryV3.content().split(b'\n')
 
     for index, duplicate_line in enumerate(lines):
       content = b'\n'.join(lines[:index] + [duplicate_line] + lines[index:])
       self.assertRaises(ValueError, RouterStatusEntryV3, content, True)
 
       entry = RouterStatusEntryV3(content, False)
-      self.assertEqual('caerSidi', entry.nickname)
+      self.assertTrue(entry.nickname.startswith('Unnamed'))
 
   def test_missing_r_field(self):
     """
@@ -291,9 +291,7 @@ class TestRouterStatusEntry(unittest.TestCase):
       test_components = [comp[1] for comp in components]
       test_components.remove(value)
       r_line = ' '.join(test_components)
-
-      content = get_router_status_entry_v3({'r': r_line}, content = True)
-      self._expect_invalid_attr(content, attr)
+      expect_invalid_attr(self, {'r': r_line}, attr)
 
   def test_malformed_nickname(self):
     """
@@ -307,8 +305,7 @@ class TestRouterStatusEntry(unittest.TestCase):
     )
 
     for value in test_values:
-      r_line = ROUTER_STATUS_ENTRY_V3_HEADER[0][1].replace('caerSidi', value)
-      content = get_router_status_entry_v3({'r': r_line}, content = True)
+      r_line = '%s p1aag7VwarGxqctS7/fS0y5FU+s oQZFLYe9e4A7bOkWKR7TaNxb0JE 2012-08-06 11:19:31 71.35.150.29 9001 0' % value
 
       # TODO: Initial whitespace is consumed as part of the keyword/value
       # divider. This is a bug in the case of V3 router status entries, but
@@ -323,7 +320,7 @@ class TestRouterStatusEntry(unittest.TestCase):
       if value == '':
         value = None
 
-      self._expect_invalid_attr(content, 'nickname')
+      expect_invalid_attr(self, {'r': r_line}, 'nickname')
 
   def test_malformed_fingerprint(self):
     """
@@ -337,9 +334,8 @@ class TestRouterStatusEntry(unittest.TestCase):
     )
 
     for value in test_values:
-      r_line = ROUTER_STATUS_ENTRY_V3_HEADER[0][1].replace('p1aag7VwarGxqctS7/fS0y5FU+s', value)
-      content = get_router_status_entry_v3({'r': r_line}, content = True)
-      self._expect_invalid_attr(content, 'fingerprint')
+      r_line = 'caerSidi %s oQZFLYe9e4A7bOkWKR7TaNxb0JE 2012-08-06 11:19:31 71.35.150.29 9001 0' % value
+      expect_invalid_attr(self, {'r': r_line}, 'fingerprint')
 
   def test_malformed_published_date(self):
     """
@@ -363,9 +359,8 @@ class TestRouterStatusEntry(unittest.TestCase):
     )
 
     for value in test_values:
-      r_line = ROUTER_STATUS_ENTRY_V3_HEADER[0][1].replace('2012-08-06 11:19:31', value)
-      content = get_router_status_entry_v3({'r': r_line}, content = True)
-      self._expect_invalid_attr(content, 'published')
+      r_line = 'caerSidi p1aag7VwarGxqctS7/fS0y5FU+s oQZFLYe9e4A7bOkWKR7TaNxb0JE %s 71.35.150.29 9001 0' % value
+      expect_invalid_attr(self, {'r': r_line}, 'published')
 
   def test_malformed_address(self):
     """
@@ -381,9 +376,8 @@ class TestRouterStatusEntry(unittest.TestCase):
     )
 
     for value in test_values:
-      r_line = ROUTER_STATUS_ENTRY_V3_HEADER[0][1].replace('71.35.150.29', value)
-      content = get_router_status_entry_v3({'r': r_line}, content = True)
-      self._expect_invalid_attr(content, 'address')
+      r_line = 'caerSidi p1aag7VwarGxqctS7/fS0y5FU+s oQZFLYe9e4A7bOkWKR7TaNxb0JE 2012-08-06 11:19:31 %s 9001 0' % value
+      expect_invalid_attr(self, {'r': r_line}, 'address')
 
   def test_malformed_port(self):
     """
@@ -403,7 +397,7 @@ class TestRouterStatusEntry(unittest.TestCase):
           if not include_or_port and not include_dir_port:
             continue
 
-          r_line = ROUTER_STATUS_ENTRY_V3_HEADER[0][1]
+          r_line = 'caerSidi p1aag7VwarGxqctS7/fS0y5FU+s oQZFLYe9e4A7bOkWKR7TaNxb0JE 2012-08-06 11:19:31 71.35.150.29 9001 0'
 
           if include_or_port:
             r_line = r_line.replace(' 9001 ', ' %s ' % value)
@@ -412,9 +406,7 @@ class TestRouterStatusEntry(unittest.TestCase):
             r_line = r_line[:-1] + value
 
           attr = 'or_port' if include_or_port else 'dir_port'
-
-          content = get_router_status_entry_v3({'r': r_line}, content = True)
-          self._expect_invalid_attr(content, attr)
+          expect_invalid_attr(self, {'r': r_line}, attr)
 
   def test_ipv6_addresses(self):
     """
@@ -427,12 +419,12 @@ class TestRouterStatusEntry(unittest.TestCase):
     }
 
     for a_line, expected in test_values.items():
-      entry = get_router_status_entry_v3({'a': a_line})
+      entry = RouterStatusEntryV3.create({'a': a_line})
       self.assertEqual(expected, entry.or_addresses)
 
     # includes multiple 'a' lines
 
-    content = get_router_status_entry_v3(content = True)
+    content = RouterStatusEntryV3.content()
     content += b'\na [2607:fcd0:daaa:101::602c:bd62]:443'
     content += b'\na [1148:fcd0:daaa:101::602c:bd62]:80'
 
@@ -452,8 +444,7 @@ class TestRouterStatusEntry(unittest.TestCase):
     )
 
     for a_line in test_values:
-      content = get_router_status_entry_v3({'a': a_line}, content = True)
-      self._expect_invalid_attr(content, expected_value = {})
+      expect_invalid_attr(self, {'a': a_line}, expected_value = {})
 
   def test_flags(self):
     """
@@ -468,7 +459,7 @@ class TestRouterStatusEntry(unittest.TestCase):
     }
 
     for s_line, expected in test_values.items():
-      entry = get_router_status_entry_v3({'s': s_line})
+      entry = RouterStatusEntryV3.create({'s': s_line})
       self.assertEqual(expected, entry.flags)
 
     # tries some invalid inputs
@@ -479,8 +470,11 @@ class TestRouterStatusEntry(unittest.TestCase):
     }
 
     for s_line, expected in test_values.items():
-      content = get_router_status_entry_v3({'s': s_line}, content = True)
-      self._expect_invalid_attr(content, 'flags', expected)
+      expect_invalid_attr(self, {'s': s_line}, 'flags', expected)
+
+  def test_protocols(self):
+    desc = RouterStatusEntryV3.create({'pr': 'Cons=1 Desc=1 DirCache=1 HSDir=1 HSIntro=3 HSRend=1 Link=1-4 LinkAuth=1 Microdesc=1 Relay=1-2'})
+    self.assertEqual(10, len(desc.protocols))
 
   def test_versions(self):
     """
@@ -495,13 +489,12 @@ class TestRouterStatusEntry(unittest.TestCase):
     }
 
     for v_line, expected in test_values.items():
-      entry = get_router_status_entry_v3({'v': v_line})
+      entry = RouterStatusEntryV3.create({'v': v_line})
       self.assertEqual(expected, entry.version)
       self.assertEqual(v_line, entry.version_line)
 
     # tries an invalid input
-    content = get_router_status_entry_v3({'v': 'Tor ugabuga'}, content = True)
-    self._expect_invalid_attr(content, 'version')
+    expect_invalid_attr(self, {'v': 'Tor ugabuga'}, 'version')
 
   def test_bandwidth(self):
     """
@@ -517,7 +510,7 @@ class TestRouterStatusEntry(unittest.TestCase):
     }
 
     for w_line, expected in test_values.items():
-      entry = get_router_status_entry_v3({'w': w_line})
+      entry = RouterStatusEntryV3.create({'w': w_line})
       self.assertEqual(expected[0], entry.bandwidth)
       self.assertEqual(expected[1], entry.measured)
       self.assertEqual(expected[2], entry.is_unmeasured)
@@ -543,8 +536,7 @@ class TestRouterStatusEntry(unittest.TestCase):
     )
 
     for w_line in test_values:
-      content = get_router_status_entry_v3({'w': w_line}, content = True)
-      self._expect_invalid_attr(content)
+      expect_invalid_attr(self, {'w': w_line})
 
   def test_exit_policy(self):
     """
@@ -557,7 +549,7 @@ class TestRouterStatusEntry(unittest.TestCase):
     }
 
     for p_line, expected in test_values.items():
-      entry = get_router_status_entry_v3({'p': p_line})
+      entry = RouterStatusEntryV3.create({'p': p_line})
       self.assertEqual(expected, entry.exit_policy)
 
     # tries some invalid inputs
@@ -569,8 +561,7 @@ class TestRouterStatusEntry(unittest.TestCase):
     )
 
     for p_line in test_values:
-      content = get_router_status_entry_v3({'p': p_line}, content = True)
-      self._expect_invalid_attr(content, 'exit_policy')
+      expect_invalid_attr(self, {'p': p_line}, 'exit_policy')
 
   def test_microdescriptor_hashes(self):
     """
@@ -587,13 +578,13 @@ class TestRouterStatusEntry(unittest.TestCase):
     }
 
     for m_line, expected in test_values.items():
-      content = get_router_status_entry_v3({'m': m_line}, content = True)
+      content = RouterStatusEntryV3.content({'m': m_line})
       entry = RouterStatusEntryV3(content, document = vote_document())
       self.assertEqual(expected, entry.microdescriptor_hashes)
 
     # try with multiple 'm' lines
 
-    content = get_router_status_entry_v3(content = True)
+    content = RouterStatusEntryV3.content()
     content += b'\nm 11,12 sha256=g1vx9si329muxV3tquWIXXySNOIwRGMeAESKs/v4DWs'
     content += b'\nm 31,32 sha512=g1vx9si329muxV3tquWIXXySNOIwRGMeAESKs/v4DWs'
 
@@ -606,8 +597,7 @@ class TestRouterStatusEntry(unittest.TestCase):
     self.assertEqual(expected, entry.microdescriptor_hashes)
 
     # try without a document
-    content = get_router_status_entry_v3({'m': '8,9,10,11,12'}, content = True)
-    self._expect_invalid_attr(content, 'microdescriptor_hashes', expected_value = [])
+    expect_invalid_attr(self, {'m': '8,9,10,11,12'}, 'microdescriptor_hashes', expected_value = [])
 
     # tries some invalid inputs
     test_values = (
@@ -617,7 +607,7 @@ class TestRouterStatusEntry(unittest.TestCase):
     )
 
     for m_line in test_values:
-      content = get_router_status_entry_v3({'m': m_line}, content = True)
+      content = RouterStatusEntryV3.content({'m': m_line})
       self.assertRaises(ValueError, RouterStatusEntryV3, content, True, vote_document())
 
   def test_with_carriage_returns(self):
@@ -625,7 +615,7 @@ class TestRouterStatusEntry(unittest.TestCase):
     Read a descriptor file with windows newlines (CRLF).
     """
 
-    descriptor_path = get_resource('cached-microdesc-consensus_with_carriage_returns')
+    descriptor_path = get_resource('unparseable/cached-microdesc-consensus_with_carriage_returns')
 
     with open(descriptor_path, 'rb') as descriptor_file:
       descriptors = stem.descriptor.parse_file(descriptor_file, 'network-status-microdesc-consensus-3 1.0', normalize_newlines = True)
@@ -634,18 +624,3 @@ class TestRouterStatusEntry(unittest.TestCase):
 
       router = next(descriptors)
       self.assertEqual([Flag.FAST, Flag.RUNNING, Flag.STABLE, Flag.VALID], router.flags)
-
-  def _expect_invalid_attr(self, content, attr = None, expected_value = None):
-    """
-    Asserts that construction will fail due to content having a malformed
-    attribute. If an attr is provided then we check that it matches an expected
-    value when we're constructed without validation.
-    """
-
-    self.assertRaises(ValueError, RouterStatusEntryV3, content, True)
-    entry = RouterStatusEntryV3(content, False)
-
-    if attr:
-      self.assertEqual(expected_value, getattr(entry, attr))
-    else:
-      self.assertEqual('caerSidi', entry.nickname)
