@@ -3,27 +3,37 @@ Unit tests for stem.descriptor.server_descriptor.
 """
 
 import datetime
+import functools
 import io
 import pickle
 import tarfile
+import time
 import unittest
 
+import stem.descriptor
+import stem.descriptor.router_status_entry
 import stem.descriptor.server_descriptor
 import stem.exit_policy
 import stem.prereq
 import stem.version
 import stem.util.str_tools
+import test.require
 
 from stem.util import str_type
-from stem.descriptor.server_descriptor import RelayDescriptor, BridgeDescriptor
+from stem.descriptor.certificate import CertType, ExtensionType
+from stem.descriptor.server_descriptor import BridgeDistribution, RelayDescriptor, BridgeDescriptor
 
-from test.mocking import (
-  get_relay_server_descriptor,
-  get_bridge_server_descriptor,
-  CRYPTO_BLOB,
+from test.unit.descriptor import (
+  get_resource,
+  base_expect_invalid_attr,
+  base_expect_invalid_attr_for_text,
 )
 
-from test.unit.descriptor import get_resource
+try:
+  # Added in 2.7
+  from collections import OrderedDict
+except ImportError:
+  from stem.util.ordereddict import OrderedDict
 
 try:
   # added in python 3.3
@@ -37,6 +47,9 @@ TARFILE_FINGERPRINTS = set([
   str_type('1F43EE37A0670301AD9CB555D94AFEC2C89FDE86'),
 ])
 
+expect_invalid_attr = functools.partial(base_expect_invalid_attr, RelayDescriptor, 'nickname', 'Unnamed')
+expect_invalid_attr_for_text = functools.partial(base_expect_invalid_attr_for_text, RelayDescriptor, 'nickname', 'Unnamed')
+
 
 class TestServerDescriptor(unittest.TestCase):
   def test_with_tarfile_path(self):
@@ -109,6 +122,7 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     self.assertEqual(9001, desc.or_port)
     self.assertEqual(None, desc.socks_port)
     self.assertEqual(None, desc.dir_port)
+    self.assertEqual(None, desc.certificate)
     self.assertEqual(None, desc.ed25519_certificate)
     self.assertEqual(None, desc.ed25519_master_key)
     self.assertEqual(None, desc.ed25519_signature)
@@ -125,7 +139,9 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     self.assertEqual(False, desc.allow_tunneled_dir_requests)
     self.assertEqual(False, desc.extra_info_cache)
     self.assertEqual('D225B728768D7EA4B5587C13A7A9D22EBBEE6E66', desc.extra_info_digest)
+    self.assertEqual(None, desc.extra_info_sha256_digest)
     self.assertEqual(['2'], desc.hidden_service_dir)
+    self.assertEqual(BridgeDistribution.ANY, desc.bridge_distribution)
     self.assertEqual(expected_family, desc.family)
     self.assertEqual(153600, desc.average_bandwidth)
     self.assertEqual(256000, desc.burst_bandwidth)
@@ -183,7 +199,9 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     self.assertEqual(False, desc.allow_tunneled_dir_requests)
     self.assertEqual(False, desc.extra_info_cache)
     self.assertEqual(None, desc.extra_info_digest)
+    self.assertEqual(None, desc.extra_info_sha256_digest)
     self.assertEqual(None, desc.hidden_service_dir)
+    self.assertEqual(BridgeDistribution.ANY, desc.bridge_distribution)
     self.assertEqual(set(), desc.family)
     self.assertEqual(102400, desc.average_bandwidth)
     self.assertEqual(10485760, desc.burst_bandwidth)
@@ -230,7 +248,9 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     self.assertEqual(False, desc.allow_tunneled_dir_requests)
     self.assertEqual(False, desc.extra_info_cache)
     self.assertEqual('56403D838DE152421CD401B8E57DAD4483A3D56B', desc.extra_info_digest)
+    self.assertEqual(None, desc.extra_info_sha256_digest)
     self.assertEqual(['2'], desc.hidden_service_dir)
+    self.assertEqual(BridgeDistribution.ANY, desc.bridge_distribution)
     self.assertEqual(set(), desc.family)
     self.assertEqual(102400, desc.average_bandwidth)
     self.assertEqual(204800, desc.burst_bandwidth)
@@ -245,6 +265,82 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
 
     self.assertTrue(isinstance(str(desc), str))
 
+  @test.require.cryptography
+  def test_descriptor_signing(self):
+    RelayDescriptor.create(sign = True)
+    self.assertRaisesRegexp(NotImplementedError, 'Signing of BridgeDescriptor not implemented', BridgeDescriptor.create, sign = True)
+
+  def test_router_status_entry(self):
+    """
+    Tests creation of router status entries.
+    """
+
+    desc_without_fingerprint = RelayDescriptor.create()
+    exc_msg = 'Server descriptor lacks a fingerprint. This is an optional field, but required to make a router status entry.'
+    self.assertRaisesRegexp(ValueError, exc_msg, desc_without_fingerprint.make_router_status_entry)
+
+    desc = RelayDescriptor.create(OrderedDict((
+      ('router', 'caerSidi 71.35.133.197 9001 0 0'),
+      ('published', '2012-02-29 04:03:19'),
+      ('fingerprint', '4F0C 867D F0EF 6816 0568 C826 838F 482C EA7C FE44'),
+      ('or-address', ['71.35.133.197:9001', '[12ab:2e19:3bcf::02:9970]:9001']),
+      ('onion-key', '\n-----BEGIN RSA PUBLIC KEY-----%s-----END RSA PUBLIC KEY-----' % stem.descriptor.CRYPTO_BLOB),
+      ('signing-key', '\n-----BEGIN RSA PUBLIC KEY-----%s-----END RSA PUBLIC KEY-----' % stem.descriptor.CRYPTO_BLOB),
+    ))).make_router_status_entry()
+
+    self.assertEqual(stem.descriptor.router_status_entry.RouterStatusEntryV3, type(desc))
+    self.assertEqual('caerSidi', desc.nickname)
+    self.assertEqual('4F0C867DF0EF68160568C826838F482CEA7CFE44', desc.fingerprint)
+    self.assertEqual(datetime.datetime(2012, 2, 29, 4, 3, 19), desc.published)
+    self.assertEqual('71.35.133.197', desc.address)
+    self.assertEqual(9001, desc.or_port)
+    self.assertEqual(None, desc.dir_port)
+    self.assertEqual(['Fast', 'Named', 'Running', 'Stable', 'Valid'], desc.flags)
+    self.assertEqual(None, desc.version)
+    self.assertEqual(None, desc.version_line)
+
+    self.assertEqual([('71.35.133.197', 9001, False), ('12ab:2e19:3bcf::02:9970', 9001, True)], desc.or_addresses)
+    self.assertEqual(None, desc.identifier_type)
+    self.assertEqual(None, desc.identifier)
+    self.assertEqual('A863EFE8395C41C880782B89B850D20EDD242BDA', desc.digest)
+    self.assertEqual(153600, desc.bandwidth)
+    self.assertEqual(None, desc.measured)
+    self.assertEqual(False, desc.is_unmeasured)
+    self.assertEqual([], desc.unrecognized_bandwidth_entries)
+    self.assertEqual(stem.exit_policy.MicroExitPolicy('reject 1-65535'), desc.exit_policy)
+    self.assertEqual([], desc.microdescriptor_hashes)
+
+  def test_make_router_status_entry_with_live_descriptor(self):
+    """
+    Tests creation of router status entries with a live server descriptor.
+    """
+
+    with open(get_resource('server_descriptor_with_ed25519'), 'rb') as descriptor_file:
+      desc = next(stem.descriptor.parse_file(descriptor_file, validate = True)).make_router_status_entry()
+
+    self.assertEqual(stem.descriptor.router_status_entry.RouterStatusEntryV3, type(desc))
+    self.assertEqual('destiny', desc.nickname)
+    self.assertEqual('F65E0196C94DFFF48AFBF2F5F9E3E19AAE583FD0', desc.fingerprint)
+    self.assertEqual(datetime.datetime(2015, 8, 22, 15, 21, 45), desc.published)
+    self.assertEqual('94.242.246.23', desc.address)
+    self.assertEqual(9001, desc.or_port)
+    self.assertEqual(443, desc.dir_port)
+    self.assertEqual(['Fast', 'Named', 'Running', 'Stable', 'Valid'], desc.flags)
+    self.assertEqual(stem.version.Version('0.2.7.2-alpha-dev'), desc.version)
+    self.assertEqual('Tor 0.2.7.2-alpha-dev', desc.version_line)
+
+    self.assertEqual([('2a01:608:ffff:ff07::1:23', 9003, True)], desc.or_addresses)
+    self.assertEqual('ed25519', desc.identifier_type)
+    self.assertEqual('pbYagEQPUiNjcDp/oY2oESXkDzd8PZlr26kaR7nUkao', desc.identifier)
+    self.assertEqual('B5E441051D139CCD84BC765D130B01E44DAC29AD', desc.digest)
+    self.assertEqual(149715200, desc.bandwidth)
+    self.assertEqual(None, desc.measured)
+    self.assertEqual(False, desc.is_unmeasured)
+    self.assertEqual([], desc.unrecognized_bandwidth_entries)
+    self.assertEqual(stem.exit_policy.MicroExitPolicy('reject 25,465,587,10000,14464'), desc.exit_policy)
+    self.assertEqual([], desc.microdescriptor_hashes)
+
+  @patch('time.time', Mock(return_value = time.mktime(datetime.date(2010, 1, 1).timetuple())))
   def test_with_ed25519(self):
     """
     Parses a descriptor with a ed25519 identity key, as added by proposal 228
@@ -261,6 +357,21 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
       '$EC116BCB80565A408CE67F8EC3FE3B0B02C3A065',
     ])
 
+    self.assertEqual(1, desc.certificate.version)
+    self.assertEqual(CertType.SIGNING, desc.certificate.type)
+    self.assertEqual(datetime.datetime(2015, 8, 28, 17, 0, 0), desc.certificate.expiration)
+    self.assertEqual(1, desc.certificate.key_type)
+    self.assertTrue(desc.certificate.key.startswith(b'\xa5\xb6\x1a\x80D\x0f'))
+    self.assertTrue(desc.certificate.signature.startswith(b'\xc6\x8e\xd3\xae\x0b'))
+    self.assertEqual(1, len(desc.certificate.extensions))
+    self.assertTrue('bWPo2fIzo3uOywfoM' in desc.certificate.encoded)
+
+    extension = desc.certificate.extensions[0]
+    self.assertEqual(ExtensionType.HAS_SIGNING_KEY, extension.type)
+    self.assertEqual([], extension.flags)
+    self.assertEqual(0, extension.flag_int)
+    self.assertTrue(extension.data.startswith(b'g\xa6\xb5Q\xa6\xd2'))
+
     self.assertEqual('destiny', desc.nickname)
     self.assertEqual('F65E0196C94DFFF48AFBF2F5F9E3E19AAE583FD0', desc.fingerprint)
     self.assertEqual('94.242.246.23', desc.address)
@@ -283,6 +394,7 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     self.assertEqual(False, desc.allow_tunneled_dir_requests)
     self.assertEqual(False, desc.extra_info_cache)
     self.assertEqual('44E9B679AF0B4EB09296985BAF4066AE9CA5BB93', desc.extra_info_digest)
+    self.assertEqual('r+roMxhsjd1GPpn5knQoBvtE9Rhsv8zQHCqiYL6u2CA', desc.extra_info_sha256_digest)
     self.assertEqual(['2'], desc.hidden_service_dir)
     self.assertEqual(family, desc.family)
     self.assertEqual(149715200, desc.average_bandwidth)
@@ -299,6 +411,16 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     self.assertEqual('B5E441051D139CCD84BC765D130B01E44DAC29AD', desc.digest())
     self.assertEqual([], desc.get_unrecognized_lines())
 
+  @patch('time.time', Mock(return_value = time.mktime(datetime.date(2020, 1, 1).timetuple())))
+  def test_with_ed25519_expired_cert(self):
+    """
+    Parses a server descriptor with an expired ed25519 certificate
+    """
+
+    desc_text = open(get_resource('bridge_descriptor_with_ed25519'), 'rb').read()
+    desc_iter = stem.descriptor.server_descriptor._parse_file(io.BytesIO(desc_text), validate = True)
+    self.assertRaises(ValueError, list, desc_iter)
+
   def test_bridge_with_ed25519(self):
     """
     Parses a bridge descriptor with ed25519.
@@ -383,7 +505,9 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     self.assertEqual(False, desc.allow_tunneled_dir_requests)
     self.assertEqual(True, desc.extra_info_cache)
     self.assertEqual('BB1F13AA431421BEA29B840A2E33BB1C31C2990B', desc.extra_info_digest)
+    self.assertEqual(None, desc.extra_info_sha256_digest)
     self.assertEqual(None, desc.hidden_service_dir)
+    self.assertEqual(BridgeDistribution.ANY, desc.bridge_distribution)
     self.assertEqual(set(), desc.family)
     self.assertEqual(3220480, desc.average_bandwidth)
     self.assertEqual(6441984, desc.burst_bandwidth)
@@ -398,19 +522,17 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     attributes.
     """
 
-    desc = get_relay_server_descriptor()
-
+    desc = RelayDescriptor.create({'router': 'caerSidi 71.35.133.197 9001 0 0'})
     self.assertEqual('caerSidi', desc.nickname)
     self.assertEqual('71.35.133.197', desc.address)
     self.assertEqual(None, desc.fingerprint)
-    self.assertTrue(CRYPTO_BLOB in desc.onion_key)
 
   def test_with_opt(self):
     """
     Includes an 'opt <keyword> <value>' entry.
     """
 
-    desc = get_relay_server_descriptor({'opt': 'contact www.atagar.com/contact/'})
+    desc = RelayDescriptor.create({'opt': 'contact www.atagar.com/contact/'})
     self.assertEqual(b'www.atagar.com/contact/', desc.contact)
 
   def test_unrecognized_line(self):
@@ -418,7 +540,7 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     Includes unrecognized content in the descriptor.
     """
 
-    desc = get_relay_server_descriptor({'pepperjack': 'is oh so tasty!'})
+    desc = RelayDescriptor.create({'pepperjack': 'is oh so tasty!'})
     self.assertEqual(['pepperjack is oh so tasty!'], desc.get_unrecognized_lines())
 
   def test_proceeding_line(self):
@@ -426,79 +548,72 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     Includes a line prior to the 'router' entry.
     """
 
-    desc_text = b'hibernate 1\n' + get_relay_server_descriptor(content = True)
-    self._expect_invalid_attr(desc_text)
+    desc_text = b'hibernate 1\n' + RelayDescriptor.content()
+    expect_invalid_attr_for_text(self, desc_text)
 
   def test_trailing_line(self):
     """
     Includes a line after the 'router-signature' entry.
     """
 
-    desc_text = get_relay_server_descriptor(content = True) + b'\nhibernate 1'
-    self._expect_invalid_attr(desc_text)
+    desc_text = RelayDescriptor.content() + b'\nhibernate 1'
+    expect_invalid_attr_for_text(self, desc_text)
 
   def test_nickname_missing(self):
     """
     Constructs with a malformed router entry.
     """
 
-    desc_text = get_relay_server_descriptor({'router': ' 71.35.133.197 9001 0 0'}, content = True)
-    self._expect_invalid_attr(desc_text, 'nickname')
+    expect_invalid_attr(self, {'router': ' 71.35.133.197 9001 0 0'}, 'nickname')
 
   def test_nickname_too_long(self):
     """
     Constructs with a nickname that is an invalid length.
     """
 
-    desc_text = get_relay_server_descriptor({'router': 'saberrider2008ReallyLongNickname 71.35.133.197 9001 0 0'}, content = True)
-    self._expect_invalid_attr(desc_text, 'nickname')
+    expect_invalid_attr(self, {'router': 'saberrider2008ReallyLongNickname 71.35.133.197 9001 0 0'}, 'nickname')
 
   def test_nickname_invalid_char(self):
     """
     Constructs with an invalid relay nickname.
     """
 
-    desc_text = get_relay_server_descriptor({'router': '$aberrider2008 71.35.133.197 9001 0 0'}, content = True)
-    self._expect_invalid_attr(desc_text, 'nickname')
+    expect_invalid_attr(self, {'router': '$aberrider2008 71.35.133.197 9001 0 0'}, 'nickname')
 
   def test_address_malformed(self):
     """
     Constructs with an invalid ip address.
     """
 
-    desc_text = get_relay_server_descriptor({'router': 'caerSidi 371.35.133.197 9001 0 0'}, content = True)
-    self._expect_invalid_attr(desc_text, 'address')
+    expect_invalid_attr(self, {'router': 'caerSidi 371.35.133.197 9001 0 0'}, 'address')
 
   def test_port_too_high(self):
     """
     Constructs with an ORPort that is too large.
     """
 
-    desc_text = get_relay_server_descriptor({'router': 'caerSidi 71.35.133.197 900001 0 0'}, content = True)
-    self._expect_invalid_attr(desc_text, 'or_port')
+    expect_invalid_attr(self, {'router': 'caerSidi 71.35.133.197 900001 0 0'}, 'or_port')
 
   def test_port_malformed(self):
     """
     Constructs with an ORPort that isn't numeric.
     """
 
-    desc_text = get_relay_server_descriptor({'router': 'caerSidi 71.35.133.197 900a1 0 0'}, content = True)
-    self._expect_invalid_attr(desc_text, 'or_port')
+    expect_invalid_attr(self, {'router': 'caerSidi 71.35.133.197 900a1 0 0'}, 'or_port')
 
   def test_port_newline(self):
     """
     Constructs with a newline replacing the ORPort.
     """
 
-    desc_text = get_relay_server_descriptor({'router': 'caerSidi 71.35.133.197 \n 0 0'}, content = True)
-    self._expect_invalid_attr(desc_text, 'or_port')
+    expect_invalid_attr(self, {'router': 'caerSidi 71.35.133.197 \n 0 0'}, 'or_port')
 
   def test_platform_empty(self):
     """
     Constructs with an empty platform entry.
     """
 
-    desc_text = get_relay_server_descriptor({'platform': ''}, content = True)
+    desc_text = RelayDescriptor.content({'platform': ''})
     desc = RelayDescriptor(desc_text, validate = False)
     self.assertEqual(b'', desc.platform)
 
@@ -512,7 +627,7 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     Parse a platform line belonging to a node-Tor relay.
     """
 
-    desc = get_relay_server_descriptor({'platform': 'node-Tor 0.1.0 on Linux x86_64'})
+    desc = RelayDescriptor.create({'platform': 'node-Tor 0.1.0 on Linux x86_64'})
     self.assertEqual(b'node-Tor 0.1.0 on Linux x86_64', desc.platform)
     self.assertEqual(stem.version.Version('0.1.0'), desc.tor_version)
     self.assertEqual('Linux x86_64', desc.operating_system)
@@ -522,8 +637,7 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     Constructs with a protocols line without circuit versions.
     """
 
-    desc_text = get_relay_server_descriptor({'opt': 'protocols Link 1 2'}, content = True)
-    self._expect_invalid_attr(desc_text, 'circuit_protocols')
+    expect_invalid_attr(self, {'opt': 'protocols Link 1 2'}, 'circuit_protocols')
 
   @patch('stem.prereq.is_crypto_available', Mock(return_value = False))
   def test_published_leap_year(self):
@@ -532,20 +646,17 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     invalid.
     """
 
-    desc_text = get_relay_server_descriptor({'published': '2011-02-29 04:03:19'}, content = True)
-    self._expect_invalid_attr(desc_text, 'published')
+    expect_invalid_attr(self, {'published': '2011-02-29 04:03:19'}, 'published')
 
-    desc_text = get_relay_server_descriptor({'published': '2012-02-29 04:03:19'}, content = True)
-    expected_published = datetime.datetime(2012, 2, 29, 4, 3, 19)
-    self.assertEqual(expected_published, RelayDescriptor(desc_text).published)
+    desc = RelayDescriptor.create({'published': '2012-02-29 04:03:19'})
+    self.assertEqual(datetime.datetime(2012, 2, 29, 4, 3, 19), desc.published)
 
   def test_published_no_time(self):
     """
     Constructs with a published entry without a time component.
     """
 
-    desc_text = get_relay_server_descriptor({'published': '2012-01-01'}, content = True)
-    self._expect_invalid_attr(desc_text, 'published')
+    expect_invalid_attr(self, {'published': '2012-01-01'}, 'published')
 
   def test_read_and_write_history(self):
     """
@@ -556,7 +667,7 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
 
     for field in ('read-history', 'write-history'):
       value = '2005-12-16 18:00:48 (900 s) 81,8848,8927,8927,83,8848'
-      desc = get_relay_server_descriptor({'opt %s' % field: value})
+      desc = RelayDescriptor.create({'opt %s' % field: value})
 
       if field == 'read-history':
         attr = (desc.read_history_end, desc.read_history_interval, desc.read_history_values)
@@ -575,8 +686,7 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     Parses a read-history with an empty value.
     """
 
-    value = '2005-12-17 01:23:11 (900 s) '
-    desc = get_relay_server_descriptor({'opt read-history': value})
+    desc = RelayDescriptor.create({'opt read-history': '2005-12-17 01:23:11 (900 s) '})
     self.assertEqual(datetime.datetime(2005, 12, 17, 1, 23, 11), desc.read_history_end)
     self.assertEqual(900, desc.read_history_interval)
     self.assertEqual([], desc.read_history_values)
@@ -588,7 +698,7 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     """
 
     desc_text = b'@pepperjack very tasty\n@mushrooms not so much\n'
-    desc_text += get_relay_server_descriptor(content = True)
+    desc_text += RelayDescriptor.content()
     desc_text += b'\ntrailing text that should be invalid, ho hum'
 
     # running _parse_file should provide an iterator with a single descriptor
@@ -596,7 +706,7 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     self.assertRaises(ValueError, list, desc_iter)
 
     desc_text = b'@pepperjack very tasty\n@mushrooms not so much\n'
-    desc_text += get_relay_server_descriptor(content = True)
+    desc_text += RelayDescriptor.content({'router': 'caerSidi 71.35.133.197 9001 0 0'})
     desc_iter = stem.descriptor.server_descriptor._parse_file(io.BytesIO(desc_text))
 
     desc_entries = list(desc_iter)
@@ -614,9 +724,9 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     Constructs with a field appearing twice.
     """
 
-    desc_text = get_relay_server_descriptor({'<replace>': ''}, content = True)
+    desc_text = RelayDescriptor.content({'<replace>': ''})
     desc_text = desc_text.replace(b'<replace>', b'contact foo\ncontact bar')
-    self._expect_invalid_attr(desc_text, 'contact', b'foo')
+    expect_invalid_attr_for_text(self, desc_text, 'contact', b'foo')
 
   def test_missing_required_attr(self):
     """
@@ -624,7 +734,7 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     """
 
     for attr in stem.descriptor.server_descriptor.REQUIRED_FIELDS:
-      desc_text = get_relay_server_descriptor(exclude = [attr], content = True)
+      desc_text = RelayDescriptor.content(exclude = [attr])
       self.assertRaises(ValueError, RelayDescriptor, desc_text, True)
 
       # check that we can still construct it without validation
@@ -645,24 +755,63 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     """
 
     fingerprint = '4F0C 867D F0EF 6816 0568 C826 838F 482C EA7C FE45'
-    desc_text = get_relay_server_descriptor({'opt fingerprint': fingerprint}, content = True)
-    self._expect_invalid_attr(desc_text, 'fingerprint', fingerprint.replace(' ', ''))
+    expect_invalid_attr(self, {'opt fingerprint': fingerprint}, 'fingerprint', fingerprint.replace(' ', ''))
+
+  def test_with_bridge_distribution(self):
+    """
+    Include a preferred method of bridge distribution.
+    """
+
+    desc = RelayDescriptor.create({'bridge-distribution-request': 'email'})
+    self.assertEqual(BridgeDistribution.EMAIL, desc.bridge_distribution)
 
   def test_ipv6_policy(self):
     """
     Checks a 'ipv6-policy' line.
     """
 
-    expected = stem.exit_policy.MicroExitPolicy('accept 22-23,53,80,110')
-    desc = get_relay_server_descriptor({'ipv6-policy': 'accept 22-23,53,80,110'})
-    self.assertEqual(expected, desc.exit_policy_v6)
+    desc = RelayDescriptor.create({'ipv6-policy': 'accept 22-23,53,80,110'})
+    self.assertEqual(stem.exit_policy.MicroExitPolicy('accept 22-23,53,80,110'), desc.exit_policy_v6)
+
+  def test_extrainfo_sha256_digest(self):
+    """
+    Extrainfo descriptor line with both a hex and base64 encoded sha256 digest.
+    """
+
+    desc = RelayDescriptor.create({'extra-info-digest': '03272BF7C68484AFBDA508DAE3734D809E4A5BC4 DWMz1AEdqPlcroubwx3lPEoGbT+oX7S2BH653sPIqfI'})
+    self.assertEqual('03272BF7C68484AFBDA508DAE3734D809E4A5BC4', desc.extra_info_digest)
+    self.assertEqual('DWMz1AEdqPlcroubwx3lPEoGbT+oX7S2BH653sPIqfI', desc.extra_info_sha256_digest)
+
+  def test_protocols(self):
+    """
+    Checks a 'proto' line.
+    """
+
+    desc = RelayDescriptor.create({'proto': 'Cons=1 Desc=1 DirCache=1 HSDir=1 HSIntro=3 HSRend=1 Link=1-4 LinkAuth=1 Microdesc=1 Relay=1-2'})
+    self.assertEqual({'Cons': [1], 'Desc': [1], 'DirCache': [1], 'HSDir': [1], 'HSIntro': [3], 'HSRend': [1], 'Link': [1, 2, 3, 4], 'LinkAuth': [1], 'Microdesc': [1], 'Relay': [1, 2]}, desc.protocols)
+
+  def test_protocols_with_no_mapping(self):
+    """
+    Checks a 'proto' line when it's not key=value pairs.
+    """
+
+    exc_msg = "Protocol entires are expected to be a series of 'key=value' pairs but was: proto Desc Link=1-4"
+    self.assertRaisesRegexp(ValueError, exc_msg, RelayDescriptor.create, {'proto': 'Desc Link=1-4'})
+
+  def test_parse_with_non_int_version(self):
+    """
+    Checks a 'proto' line with non-numeric content.
+    """
+
+    exc_msg = 'Protocol values should be a number or number range, but was: proto Desc=hi Link=1-4'
+    self.assertRaisesRegexp(ValueError, exc_msg, RelayDescriptor.create, {'proto': 'Desc=hi Link=1-4'})
 
   def test_ntor_onion_key(self):
     """
     Checks a 'ntor-onion-key' line.
     """
 
-    desc = get_relay_server_descriptor({'ntor-onion-key': 'Od2Sj3UXFyDjwESLXk6fhatqW9z/oBL/vAKJ+tbDqUU='})
+    desc = RelayDescriptor.create({'ntor-onion-key': 'Od2Sj3UXFyDjwESLXk6fhatqW9z/oBL/vAKJ+tbDqUU='})
     self.assertEqual('Od2Sj3UXFyDjwESLXk6fhatqW9z/oBL/vAKJ+tbDqUU=', desc.ntor_onion_key)
 
   def test_minimal_bridge_descriptor(self):
@@ -670,10 +819,9 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     Basic sanity check that we can parse a descriptor with minimal attributes.
     """
 
-    desc = get_bridge_server_descriptor()
+    desc = BridgeDescriptor.create()
 
-    self.assertEqual('Unnamed', desc.nickname)
-    self.assertEqual('10.45.227.253', desc.address)
+    self.assertTrue(desc.nickname.startswith('Unnamed'))
     self.assertEqual(None, desc.fingerprint)
     self.assertEqual('006FD96BA35E7785A6A3B8B75FE2E2435A13BDB4', desc.digest())
 
@@ -692,13 +840,13 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
       {'contact': 'Damian'},
       {'or-address': '71.35.133.197:9001'},
       {'or-address': '[12ab:2e19:3bcf::02:9970]:9001'},
-      {'onion-key': '\n-----BEGIN RSA PUBLIC KEY-----%s-----END RSA PUBLIC KEY-----' % CRYPTO_BLOB},
-      {'signing-key': '\n-----BEGIN RSA PUBLIC KEY-----%s-----END RSA PUBLIC KEY-----' % CRYPTO_BLOB},
-      {'router-signature': '\n-----BEGIN SIGNATURE-----%s-----END SIGNATURE-----' % CRYPTO_BLOB},
+      {'onion-key': '\n-----BEGIN RSA PUBLIC KEY-----%s-----END RSA PUBLIC KEY-----' % stem.descriptor.CRYPTO_BLOB},
+      {'signing-key': '\n-----BEGIN RSA PUBLIC KEY-----%s-----END RSA PUBLIC KEY-----' % stem.descriptor.CRYPTO_BLOB},
+      {'router-signature': '\n-----BEGIN SIGNATURE-----%s-----END SIGNATURE-----' % stem.descriptor.CRYPTO_BLOB},
     ]
 
     for attr in unsanitized_attr:
-      desc = get_bridge_server_descriptor(attr)
+      desc = BridgeDescriptor.create(attr)
       self.assertFalse(desc.is_scrubbed())
 
   def test_bridge_unsanitized_relay(self):
@@ -707,7 +855,7 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     its unsanatized content.
     """
 
-    desc_text = get_relay_server_descriptor({'router-digest': '006FD96BA35E7785A6A3B8B75FE2E2435A13BDB4'}, content = True)
+    desc_text = RelayDescriptor.content({'router-digest': '006FD96BA35E7785A6A3B8B75FE2E2435A13BDB4'})
     desc = BridgeDescriptor(desc_text)
     self.assertFalse(desc.is_scrubbed())
 
@@ -719,12 +867,12 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     # checks with valid content
 
     router_digest = '068A2E28D4C934D9490303B7A645BA068DCA0504'
-    desc = get_bridge_server_descriptor({'router-digest': router_digest})
+    desc = BridgeDescriptor.create({'router-digest': router_digest})
     self.assertEqual(router_digest, desc.digest())
 
     # checks when missing
 
-    desc_text = get_bridge_server_descriptor(exclude = ['router-digest'], content = True)
+    desc_text = BridgeDescriptor.content(exclude = ['router-digest'])
     self.assertRaises(ValueError, BridgeDescriptor, desc_text, True)
 
     # check that we can still construct it without validation
@@ -741,7 +889,7 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     )
 
     for value in test_values:
-      desc_text = get_bridge_server_descriptor({'router-digest': value}, content = True)
+      desc_text = BridgeDescriptor.content({'router-digest': value})
       self.assertRaises(ValueError, BridgeDescriptor, desc_text, True)
 
       desc = BridgeDescriptor(desc_text, validate = False)
@@ -752,7 +900,7 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     Constructs a bridge descriptor with a sanatized IPv4 or-address entry.
     """
 
-    desc = get_bridge_server_descriptor({'or-address': '10.45.227.253:9001'})
+    desc = BridgeDescriptor.create({'or-address': '10.45.227.253:9001'})
     self.assertEqual([('10.45.227.253', 9001, False)], desc.or_addresses)
 
   def test_or_address_v6(self):
@@ -760,7 +908,7 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     Constructs a bridge descriptor with a sanatized IPv6 or-address entry.
     """
 
-    desc = get_bridge_server_descriptor({'or-address': '[fd9f:2e19:3bcf::02:9970]:9001'})
+    desc = BridgeDescriptor.create({'or-address': '[fd9f:2e19:3bcf::02:9970]:9001'})
     self.assertEqual([('fd9f:2e19:3bcf::02:9970', 9001, True)], desc.or_addresses)
 
   def test_or_address_multiple(self):
@@ -768,9 +916,11 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     Constructs a bridge descriptor with multiple or-address entries and multiple ports.
     """
 
-    desc_text = b'\n'.join((get_bridge_server_descriptor(content = True),
-                            b'or-address 10.45.227.253:9001',
-                            b'or-address [fd9f:2e19:3bcf::02:9970]:443'))
+    desc_text = b'\n'.join((
+      BridgeDescriptor.content(),
+      b'or-address 10.45.227.253:9001',
+      b'or-address [fd9f:2e19:3bcf::02:9970]:443',
+    ))
 
     expected_or_addresses = [
       ('10.45.227.253', 9001, False),
@@ -780,25 +930,6 @@ Qlx9HNCqCY877ztFRC624ja2ql6A2hBcuoYMbkHjcQ4=
     desc = BridgeDescriptor(desc_text)
     self.assertEqual(expected_or_addresses, desc.or_addresses)
 
-  def _expect_invalid_attr(self, desc_text, attr = None, expected_value = None):
-    """
-    Asserts that construction will fail due to desc_text having a malformed
-    attribute. If an attr is provided then we check that it matches an expected
-    value when we're constructed without validation.
-    """
-
-    self.assertRaises(ValueError, RelayDescriptor, desc_text, True)
-    desc = RelayDescriptor(desc_text, validate = False)
-
-    if attr:
-      # check that the invalid attribute matches the expected value when
-      # constructed without validation
-
-      self.assertEqual(expected_value, getattr(desc, attr))
-    else:
-      # check a default attribute
-      self.assertEqual('caerSidi', desc.nickname)
-
   def test_pickleability(self):
     """
     Checks that we can unpickle lazy loaded server descriptors.
