@@ -11,7 +11,8 @@ import stem.response.events
 import stem.util.log
 
 from stem import *  # enums and exceptions
-from test import mocking
+from stem.response import ControlMessage
+from stem.descriptor.router_status_entry import RouterStatusEntryV3
 
 try:
   # added in python 3.3
@@ -46,6 +47,67 @@ CACHED="NO"'
 ADDRMAP_CACHED_MALFORMED = '650 ADDRMAP example.com 192.0.43.10 "2013-04-03 22:29:11" \
 CACHED="KINDA"'
 
+AUTHDIR_NEWDESC = """\
+650+AUTHDIR_NEWDESCS
+DROPPED
+Not replacing router descriptor; no information has changed since the last one with this identity.
+@uploaded-at 2017-05-25 04:46:21
+@source "127.0.0.1"
+router test002r 127.0.0.1 5002 0 7002
+identity-ed25519
+-----BEGIN ED25519 CERT-----
+AQQABlm9AXVOPVG4KHqFmShWRFPU2oXO15yaS+J8c6SrLBMpnB1vAQAgBABBic8D
++GIdBzNCezf1Lfw8NSpbDL7S4ExBuMXvi6WvEoN1gIGwEwddLvUF91l6BXL5yoXf
+xg7fDhYZ7CDwtVBHSfvmsIKR/QnQyylbDpVllsV9Wz6JLz52JgFGQaNjAgA=
+-----END ED25519 CERT-----
+master-key-ed25519 QYnPA/hiHQczQns39S38PDUqWwy+0uBMQbjF74ulrxI
+platform Tor 0.3.0.7 on Darwin
+proto Cons=1-2 Desc=1-2 DirCache=1 HSDir=1-2 HSIntro=3-4 HSRend=1-2 Link=1-4 LinkAuth=1,3 Microdesc=1-2 Relay=1-2
+published 2017-05-25 04:46:20
+fingerprint 3482 25F8 3C85 4796 B2DD 6364 E65C B189 B33B D696
+uptime 9
+bandwidth 1073741824 1073741824 0
+extra-info-digest 114E4F433D6E08D9E73621BB418EBF02336EB578 i6+wKCBaNqUbNZIM9DFacEM74R5m0lkh8//h6R/nbGc
+onion-key
+-----BEGIN RSA PUBLIC KEY-----
+MIGJAoGBAMMpf3fLSUEme0aQfN2RUfAPhZJMXVSqMpFRdBoKA4AYlz78VA9zxfj3
+Nyir2G2HFaTzeS82p74obc8RufJQcGoDUwDnPlHtjb2ezmr018j8i3fTEvPwj5xC
+5001FRwUVcOaLnxZKSDzpTyKRWGnSQBSbGcyXwMRtySKf0P5yjHDAgMBAAE=
+-----END RSA PUBLIC KEY-----
+signing-key
+-----BEGIN RSA PUBLIC KEY-----
+MIGJAoGBAM7Rn1kQffaJE8rbtgTiMxNY67i27hCBzzr0gE558jARizOJo8lf7get
+rxz92mzYPYskM1V/j16QhoRlrruMn319/l6o97+/ta6qIwlSPXZ1jd/BGs3yqS4X
+2N+N9qW8zC6km88K/YZuIsqYyXL7oHoIGqbERYLmp/JqLlAR52JJAgMBAAE=
+-----END RSA PUBLIC KEY-----
+onion-key-crosscert
+-----BEGIN CROSSCERT-----
+t4BE0PcPra9o5HBJHr9+h1MgP76XY3UGLQX8FGfEPHfNBMtVxRKkhOZ7Ki01+dkK
+IpfkdQn3bOXTIa+FYGvzpyADYx4RDpbHG9/Tna/xR+6LhAQfvcLrlBxsjyntXjkX
+FwE+AemijIU4DM4F1FHIkFz6OgT9B1/G0mr5QggzXS8=
+-----END CROSSCERT-----
+ntor-onion-key-crosscert 0
+-----BEGIN ED25519 CERT-----
+AQoABleVAUGJzwP4Yh0HM0J7N/Ut/Dw1KlsMvtLgTEG4xe+Lpa8SAKgaOVGouOKa
+N64AOq7FdJJM/qgI1r2+jqTj1Mk/a19kfTIQ8hhWxoaJhg1xo8BasnjNQ+4Cm7ds
+vMW0fhwJjQ4=
+-----END ED25519 CERT-----
+hidden-service-dir
+ntor-onion-key tic6dEvMWt3kaUtDhwULgb3qBnc2wd5GkJ5cv0TelhA=
+accept *:*
+ipv6-policy accept 1-65535
+tunnelled-dir-server
+router-sig-ed25519 t7XJfoMOKbk1167f4p5+z5nlXxr+8us3qaLdNeQ1nGpUyQtM4G8Ie9P/oeqwhV4lCx5pQN1vIy3lUaN811WqAA
+router-signature
+-----BEGIN SIGNATURE-----
+j/mMC1JuChvJH/ZP/Ayy0ZAV2P6VoxpRhHJfZMC07rr2ctmxfDTwLQhbYrqJ2adW
+FYy5QTzxYzSFoTA7FvsadJuyvsGlaRfgFs+AxYjBUUoK7Dcd2ri+Y11NmwCBLFqF
+4cYG3pqPHb38gXLj89QXfMJUDbOwrxvkVxIFbdwDCsE=
+-----END SIGNATURE-----
+.
+650 OK
+"""
+
 # BUILDTIMEOUT_SET event from tor 0.2.3.16.
 
 BUILD_TIMEOUT_EVENT = '650 BUILDTIMEOUT_SET COMPUTED \
@@ -202,6 +264,8 @@ $67B2BDA4264D8A189D9270E28B1D30A262838243=europa1 b3oeducbhjmbqmgw2i3jtz4fekkrin
 HS_DESC_NO_DESC_ID = '650 HS_DESC REQUESTED ajhb7kljbiru65qo NO_AUTH \
 $67B2BDA4264D8A189D9270E28B1D30A262838243'
 
+HS_DESC_NOT_FOUND = '650 HS_DESC REQUESTED ajhb7kljbiru65qo NO_AUTH UNKNOWN'
+
 HS_DESC_FAILED = '650 HS_DESC FAILED ajhb7kljbiru65qo NO_AUTH \
 $67B2BDA4264D8A189D9270E28B1D30A262838243 \
 b3oeducbhjmbqmgw2i3jtz4fekkrinwj REASON=NOT_FOUND'
@@ -423,8 +487,10 @@ CONN_BW_BAD_WRITTEN_VALUE = '650 CONN_BW ID=11 TYPE=DIR READ=272 WRITTEN=817.7'
 CONN_BW_BAD_MISSING_ID = '650 CONN_BW TYPE=DIR READ=272 WRITTEN=817'
 
 CIRC_BW = '650 CIRC_BW ID=11 READ=272 WRITTEN=817'
+CIRC_BW_WITH_TIMESTAMP = '650 CIRC_BW ID=11 READ=272 WRITTEN=817 TIME=2012-12-06T13:51:11.433755'
 CIRC_BW_BAD_WRITTEN_VALUE = '650 CIRC_BW ID=11 READ=272 WRITTEN=817.7'
 CIRC_BW_BAD_MISSING_ID = '650 CIRC_BW READ=272 WRITTEN=817'
+CIRC_BW_MALFORMED_TIMESTAMP = '650 CIRC_BW ID=11 READ=272 WRITTEN=817 TIME=boom'
 
 CELL_STATS_1 = '650 CELL_STATS ID=14 \
 OutboundQueue=19403 OutboundConn=15 \
@@ -455,9 +521,7 @@ TB_EMPTY_BAD_2 = '650 TB_EMPTY GLOBAL READ=93 WRITTEN=93 LAST=-100'
 
 
 def _get_event(content):
-  controller_event = mocking.get_message(content)
-  stem.response.convert('EVENT', controller_event)
-  return controller_event
+  return ControlMessage.from_str(content, 'EVENT', normalize = True)
 
 
 class TestEvents(unittest.TestCase):
@@ -476,7 +540,7 @@ class TestEvents(unittest.TestCase):
     def event_sender():
       for index in range(3):
         print_bw(_get_event('650 BW 15 25'))
-        time.sleep(0.05)
+        time.sleep(0.0005)
 
     controller = Mock(spec = Controller)
 
@@ -485,7 +549,7 @@ class TestEvents(unittest.TestCase):
 
     events_thread = threading.Thread(target = event_sender)
     events_thread.start()
-    time.sleep(0.2)
+    time.sleep(0.002)
     events_thread.join()
 
   def test_event(self):
@@ -571,13 +635,19 @@ class TestEvents(unittest.TestCase):
     self.assertRaises(ProtocolError, _get_event, ADDRMAP_CACHED_MALFORMED)
 
   def test_authdir_newdesc_event(self):
-    # TODO: awaiting test data - https://trac.torproject.org/7534
+    minimal_event = _get_event('650+AUTHDIR_NEWDESCS\nAction\nMessage\nDescriptor\n.\n650 OK\n')
 
-    event = _get_event('650+AUTHDIR_NEWDESCS\nAction\nMessage\nDescriptor\n.\n650 OK\n')
+    self.assertTrue(isinstance(minimal_event, stem.response.events.AuthDirNewDescEvent))
+    self.assertEqual('Action', minimal_event.action)
+    self.assertEqual('Message', minimal_event.message)
+    self.assertEqual('Descriptor', minimal_event.descriptor)
+
+    event = _get_event(AUTHDIR_NEWDESC)
 
     self.assertTrue(isinstance(event, stem.response.events.AuthDirNewDescEvent))
-    self.assertEqual([], event.positional_args)
-    self.assertEqual({}, event.keyword_args)
+    self.assertEqual('DROPPED', event.action)
+    self.assertEqual('Not replacing router descriptor; no information has changed since the last one with this identity.', event.message)
+    self.assertTrue('Descripto', event.descriptor.startswith('@uploaded-at 2017-05-25 04:46:21'))
 
   def test_build_timeout_set_event(self):
     event = _get_event(BUILD_TIMEOUT_EVENT)
@@ -860,6 +930,14 @@ class TestEvents(unittest.TestCase):
     self.assertEqual(None, event.descriptor_id)
     self.assertEqual(None, event.reason)
 
+    event = _get_event(HS_DESC_NOT_FOUND)
+
+    self.assertEqual('UNKNOWN', event.directory)
+    self.assertEqual(None, event.directory_fingerprint)
+    self.assertEqual(None, event.directory_nickname)
+    self.assertEqual(None, event.descriptor_id)
+    self.assertEqual(None, event.reason)
+
     event = _get_event(HS_DESC_FAILED)
 
     self.assertTrue(isinstance(event, stem.response.events.HSDescEvent))
@@ -934,12 +1012,12 @@ class TestEvents(unittest.TestCase):
   def test_new_consensus_event(self):
     expected_desc = []
 
-    expected_desc.append(mocking.get_router_status_entry_v3({
+    expected_desc.append(RouterStatusEntryV3.create({
       'r': 'Beaver /96bKo4soysolMgKn5Hex2nyFSY pAJH9dSBp/CG6sPhhVY/5bLaVPM 2012-12-02 22:02:45 77.223.43.54 9001 0',
       's': 'Fast Named Running Stable Valid',
     }))
 
-    expected_desc.append(mocking.get_router_status_entry_v3({
+    expected_desc.append(RouterStatusEntryV3.create({
       'r': 'Unnamed /+fJRWjmIGNAL2C5rRZHq3R91tA 7AnpZjfdBpYzXnMNm+w1bTsFF6Y 2012-12-02 17:51:10 91.121.184.87 9001 0',
       's': 'Fast Guard Running Stable Valid',
     }))
@@ -950,7 +1028,7 @@ class TestEvents(unittest.TestCase):
     self.assertEqual(expected_desc, event.desc)
 
   def test_ns_event(self):
-    expected_desc = mocking.get_router_status_entry_v3({
+    expected_desc = RouterStatusEntryV3.create({
       'r': 'whnetz dbBxYcJriTTrcxsuy4PUZcMRwCA VStM7KAIH/mXXoGDUpoGB1OXufg 2012-12-02 21:03:56 141.70.120.13 9001 9030',
       's': 'Fast HSDir Named Stable V2Dir Valid',
     })
@@ -1381,11 +1459,19 @@ class TestEvents(unittest.TestCase):
     self.assertEqual('2', event.id)
     self.assertEqual(15, event.written)
     self.assertEqual(25, event.read)
+    self.assertEqual(None, event.time)
 
     event = _get_event('650 STREAM_BW Stream02 0 0')
     self.assertEqual('Stream02', event.id)
     self.assertEqual(0, event.written)
     self.assertEqual(0, event.read)
+    self.assertEqual(None, event.time)
+
+    event = _get_event('650 STREAM_BW Stream02 0 0 2012-12-06T13:51:11.433755')
+    self.assertEqual('Stream02', event.id)
+    self.assertEqual(0, event.written)
+    self.assertEqual(0, event.read)
+    self.assertEqual(datetime.datetime(2012, 12, 6, 13, 51, 11, 433755), event.time)
 
     self.assertRaises(ProtocolError, _get_event, '650 STREAM_BW')
     self.assertRaises(ProtocolError, _get_event, '650 STREAM_BW 2')
@@ -1416,7 +1502,7 @@ class TestEvents(unittest.TestCase):
     self.assertTrue(isinstance(event, stem.response.events.ConnectionBandwidthEvent))
     self.assertEqual(CONN_BW.lstrip('650 '), str(event))
     self.assertEqual('11', event.id)
-    self.assertEqual(stem.ConnectionType.DIR, event.type)
+    self.assertEqual(stem.ConnectionType.DIR, event.conn_type)
     self.assertEqual(272, event.read)
     self.assertEqual(817, event.written)
 
@@ -1425,15 +1511,22 @@ class TestEvents(unittest.TestCase):
 
   def test_circ_bw_event(self):
     event = _get_event(CIRC_BW)
-
     self.assertTrue(isinstance(event, stem.response.events.CircuitBandwidthEvent))
     self.assertEqual(CIRC_BW.lstrip('650 '), str(event))
     self.assertEqual('11', event.id)
     self.assertEqual(272, event.read)
     self.assertEqual(817, event.written)
+    self.assertEqual(None, event.time)
+
+    event = _get_event(CIRC_BW_WITH_TIMESTAMP)
+    self.assertEqual('11', event.id)
+    self.assertEqual(272, event.read)
+    self.assertEqual(817, event.written)
+    self.assertEqual(datetime.datetime(2012, 12, 6, 13, 51, 11, 433755), event.time)
 
     self.assertRaises(ProtocolError, _get_event, CIRC_BW_BAD_WRITTEN_VALUE)
     self.assertRaises(ProtocolError, _get_event, CIRC_BW_BAD_MISSING_ID)
+    self.assertRaises(ProtocolError, _get_event, CIRC_BW_MALFORMED_TIMESTAMP)
 
   def test_cell_stats_event(self):
     event = _get_event(CELL_STATS_1)
