@@ -137,9 +137,11 @@ class TestSystem(unittest.TestCase):
       self.assertTrue(system.is_running('irssi'))
       self.assertTrue(system.is_running('moc'))
       self.assertTrue(system.is_running('tor'))
+      self.assertTrue(system.is_running(['funky-tor', 'tor']))
       self.assertTrue(system.is_running('ps'))
       self.assertTrue(system.is_running('firefox'))
-      self.assertEqual(False, system.is_running('something_else'))
+      self.assertFalse(system.is_running('something_else'))
+      self.assertFalse(system.is_running(['funky-tor', 'funkier-tor']))
 
     # mock both calls failing
 
@@ -148,6 +150,15 @@ class TestSystem(unittest.TestCase):
     self.assertFalse(system.is_running('irssi'))
     self.assertEqual(None, system.is_running('irssi'))
 
+  def test_size_of(self):
+    """
+    Exercises the size_of function.
+    """
+
+    self.assertTrue(system.size_of('') < system.size_of('hello') < system.size_of('hello world'))
+    self.assertTrue(system.size_of([]) < system.size_of(['hello']) < system.size_of(['hello', 'world']))
+    self.assertTrue(system.size_of({}) < system.size_of({'hello': 'world'}) < system.size_of({'hello': 'world', 'more': 'stuff'}))
+
   @patch('stem.util.system.call')
   @patch('stem.util.proc.is_available', Mock(return_value = False))
   @patch('stem.util.system.is_available', Mock(return_value = True))
